@@ -13,7 +13,7 @@
 
 #include <telepathy-glib/debug.h>
 #include <telepathy-glib/run.h>
-#include "manager.h"
+#include "connection-manager.h"
 
 static TpBaseConnectionManager *
 construct_cm (void)
