@@ -25,8 +25,9 @@
 #include <gio/gio.h>
 
 #include <telepathy-glib/connection.h>
-#include <telepathy-glib/proxy.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/defs.h>
+#include <telepathy-glib/proxy.h>
 
 G_BEGIN_DECLS
 
@@ -89,6 +90,7 @@ const gchar *tp_account_get_display_name (TpAccount *account);
 const gchar *tp_account_get_connection_manager (TpAccount *account);
 
 const gchar *tp_account_get_protocol (TpAccount *account);
+const gchar *tp_account_get_service (TpAccount *self);
 
 const gchar *tp_account_get_icon_name (TpAccount *account);
 
@@ -135,6 +137,13 @@ void tp_account_set_icon_name_async (TpAccount *account,
 gboolean tp_account_set_icon_name_finish (TpAccount *account,
     GAsyncResult *result, GError **error);
 
+void tp_account_set_service_async (TpAccount *self,
+    const gchar *service, GAsyncReadyCallback callback,
+    gpointer user_data);
+
+gboolean tp_account_set_service_finish (TpAccount *self,
+    GAsyncResult *result, GError **error);
+
 void tp_account_request_presence_async (TpAccount *account,
     TpConnectionPresenceType type, const gchar *status, const gchar *message,
     GAsyncReadyCallback callback, gpointer user_data);
@@ -156,6 +165,9 @@ gboolean tp_account_get_has_been_online (TpAccount *account);
 TpConnectionStatus tp_account_get_connection_status (TpAccount *account,
     TpConnectionStatusReason *reason);
 
+const gchar *tp_account_get_detailed_error (TpAccount *self,
+    const GHashTable **details);
+
 TpConnectionPresenceType tp_account_get_current_presence (TpAccount *account,
     gchar **status, gchar **status_message);
 
@@ -186,6 +198,19 @@ void tp_account_prepare_async (TpAccount *account, const GQuark *features,
 gboolean tp_account_prepare_finish (TpAccount *account, GAsyncResult *result,
     GError **error);
 
+void tp_account_set_avatar_async (TpAccount *self,
+    const guchar *avatar,
+    gsize len,
+    const gchar *mime_type,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+
+gboolean tp_account_set_avatar_finish (TpAccount *self,
+    GAsyncResult *result,
+    GError **error);
+
+gboolean tp_account_get_changing_presence (TpAccount *self);
+
 G_END_DECLS
 
 #include <telepathy-glib/_gen/tp-cli-account.h>
