@@ -29,6 +29,7 @@
 
 #define DEBUG_FLAG TP_DEBUG_GROUPS
 #include "telepathy-glib/debug-internal.h"
+#include "telepathy-glib/proxy-internal.h"
 
 
 /**
@@ -108,11 +109,11 @@ tp_channel_group_get_flags (TpChannel *self)
  * tp_channel_group_get_members:
  * @self: a channel
  *
- * If @self is ready and is a group, return a #TpIntSet containing
- * its members.
+ * If @self is a group and the %TP_CHANNEL_FEATURE_GROUP feature has been
+ * prepared, return a #TpIntSet containing its members.
  *
- * If @self is a group but is not ready, the result may either be a set
- * of members, or %NULL.
+ * If @self is a group but %TP_CHANNEL_FEATURE_GROUP has not been prepared,
+ * the result may either be a set of members, or %NULL.
  *
  * If @self is not a group, return %NULL.
  *
@@ -132,11 +133,11 @@ tp_channel_group_get_members (TpChannel *self)
  * tp_channel_group_get_local_pending:
  * @self: a channel
  *
- * If @self is ready and is a group, return a #TpIntSet containing
- * its local-pending members.
+ * If @self is a group and the %TP_CHANNEL_FEATURE_GROUP feature has been
+ * prepared, return a #TpIntSet containing its local-pending members.
  *
- * If @self is a group but is not ready, the result may either be a set
- * of local-pending members, or %NULL.
+ * If @self is a group but %TP_CHANNEL_FEATURE_GROUP has not been prepared,
+ * the result may either be a set of local-pending members, or %NULL.
  *
  * If @self is not a group, return %NULL.
  *
@@ -156,11 +157,11 @@ tp_channel_group_get_local_pending (TpChannel *self)
  * tp_channel_group_get_remote_pending:
  * @self: a channel
  *
- * If @self is ready and is a group, return a #TpIntSet containing
- * its remote-pending members.
+ * If @self is a group and the %TP_CHANNEL_FEATURE_GROUP feature has been
+ * prepared, return a #TpIntSet containing its remote-pending members.
  *
- * If @self is a group but is not ready, the result may either be a set
- * of remote-pending members, or %NULL.
+ * If @self is a group but %TP_CHANNEL_FEATURE_GROUP has not been prepared,
+ * the result may either be a set of remote-pending members, or %NULL.
  *
  * If @self is not a group, return %NULL.
  *
@@ -260,7 +261,8 @@ tp_channel_group_get_local_pending_info (TpChannel *self,
  *
  * - if @self is not a group or @handle is not a member of this channel,
  *   result is undefined;
- * - if @self does not have #TpChannel:ready = TRUE, result is undefined;
+ * - if %TP_CHANNEL_FEATURE_GROUP has not yet been prepared, result is
+ *   undefined;
  * - if @self does not have flags that include %TP_CHANNEL_FLAG_PROPERTIES,
  *   result is undefined;
  * - if @handle is channel-specific and its globally valid "owner" is known,
@@ -471,7 +473,7 @@ _tp_channel_group_set_one_lp (TpChannel *self,
   tp_intset_remove (self->priv->group_remote_pending, handle);
 
   if (actor == 0 && reason == TP_CHANNEL_GROUP_CHANGE_REASON_NONE &&
-      (message == NULL || message[0] == '\0'))
+      tp_str_empty (message))
     {
       /* we just don't bother storing informationless local-pending */
       if (self->priv->group_local_pending_info != NULL)
@@ -509,7 +511,7 @@ _tp_channel_group_set_one_lp (TpChannel *self,
   info->reason = reason;
   g_free (info->message);
 
-  if (message == NULL || message[0] == '\0')
+  if (tp_str_empty (message))
     info->message = NULL;
   else
     info->message = g_strdup (message);
@@ -666,7 +668,10 @@ _tp_channel_emit_initial_sets (TpChannel *self)
 {
   GArray *added, *remote_pending;
   GArray empty_array = { NULL, 0 };
-  TpIntSetIter iter = TP_INTSET_ITER_INIT (self->priv->group_local_pending);
+  TpIntSetFastIter iter;
+  TpHandle handle;
+
+  tp_intset_fast_iter_init (&iter, self->priv->group_local_pending);
 
   added = tp_intset_to_array (self->priv->group_members);
   remote_pending = tp_intset_to_array (self->priv->group_remote_pending);
@@ -674,15 +679,13 @@ _tp_channel_emit_initial_sets (TpChannel *self)
   g_signal_emit_by_name (self, "group-members-changed", "",
       added, &empty_array, &empty_array, remote_pending, 0, 0);
 
-  while (tp_intset_iter_next (&iter))
+  while (tp_intset_fast_iter_next (&iter, &handle))
     {
-      TpHandle handle;
       GArray local_pending = { (gchar *) &handle, 1 };
       TpHandle actor;
       TpChannelGroupChangeReason reason;
       const gchar *message;
 
-      handle = iter.element;
       tp_channel_group_get_local_pending_info (self, handle, &actor, &reason,
           &message);
 
@@ -817,6 +820,98 @@ OUT:
   _tp_channel_continue_introspection (self);
 }
 
+/*
+ * If the @group_remove_error is derived from a TpChannelGroupChangeReason,
+ * attempt to rewrite it into a TpError.
+ */
+static void
+_tp_channel_group_improve_remove_error (TpChannel *self,
+    TpHandle actor)
+{
+  GError *error = self->priv->group_remove_error;
+
+  if (error == NULL || error->domain != TP_ERRORS_REMOVED_FROM_GROUP)
+    return;
+
+  switch (error->code)
+    {
+    case TP_CHANNEL_GROUP_CHANGE_REASON_NONE:
+      if (actor == self->priv->group_self_handle ||
+          actor == tp_connection_get_self_handle (self->priv->connection))
+        {
+          error->code = TP_ERROR_CANCELLED;
+        }
+      else
+        {
+          error->code = TP_ERROR_TERMINATED;
+        }
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_OFFLINE:
+      error->code = TP_ERROR_OFFLINE;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_KICKED:
+      error->code = TP_ERROR_CHANNEL_KICKED;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_BUSY:
+      error->code = TP_ERROR_BUSY;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_INVITED:
+      DEBUG ("%s: Channel_Group_Change_Reason_Invited makes no sense as a "
+          "removal reason!", tp_proxy_get_object_path (self));
+      error->domain = TP_DBUS_ERRORS;
+      error->code = TP_DBUS_ERROR_INCONSISTENT;
+      return;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_BANNED:
+      error->code = TP_ERROR_CHANNEL_BANNED;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_ERROR:
+      /* hopefully all CMs that use this will also give us an error detail,
+       * but if they didn't, or gave us one we didn't understand... */
+      error->code = TP_ERROR_NOT_AVAILABLE;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_INVALID_CONTACT:
+      error->code = TP_ERROR_DOES_NOT_EXIST;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER:
+      error->code = TP_ERROR_NO_ANSWER;
+      break;
+
+    /* TP_CHANNEL_GROUP_CHANGE_REASON_RENAMED shouldn't be the last error
+     * seen in the channel - we'll get removed again with a real reason,
+     * later, so there's no point in doing anything special with this one */
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_PERMISSION_DENIED:
+      error->code = TP_ERROR_PERMISSION_DENIED;
+      break;
+
+    case TP_CHANNEL_GROUP_CHANGE_REASON_SEPARATED:
+      DEBUG ("%s: Channel_Group_Change_Reason_Separated makes no sense as a "
+          "removal reason!", tp_proxy_get_object_path (self));
+      error->domain = TP_DBUS_ERRORS;
+      error->code = TP_DBUS_ERROR_INCONSISTENT;
+      return;
+
+    /* all values up to and including Separated have been checked */
+
+    default:
+      /* We don't understand this reason code, so keeping the domain and code
+       * the same (i.e. using TP_ERRORS_REMOVED_FROM_GROUP) is no worse than
+       * anything else we could do. */
+      return;
+    }
+
+  /* If we changed the code we also need to change the domain; if not, we did
+   * an early return, so we'll never reach this */
+  error->domain = TP_ERRORS;
+}
 
 static void
 handle_members_changed (TpChannel *self,
@@ -935,21 +1030,51 @@ handle_members_changed (TpChannel *self,
           handle == tp_connection_get_self_handle (self->priv->connection))
         {
           const gchar *error_detail = tp_asv_get_string (details, "error");
+          const gchar *debug_message = tp_asv_get_string (details,
+              "debug-message");
 
-          self->priv->group_remove_reason = reason;
-          g_free (self->priv->group_remove_message);
+          if (debug_message == NULL && message[0] != '\0')
+            debug_message = message;
 
-          /* If there's an error detail and a message, "$error: $message", else
-           * just the non-empty one (or indeed the empty string).
-           */
-          self->priv->group_remove_message = g_strdup_printf ("%s%s%s",
-              (error_detail != NULL ? error_detail : ""),
-              (error_detail != NULL && message[0] != '\0' ? ": " : ""),
-              message);
+          if (debug_message == NULL && error_detail != NULL)
+            debug_message = error_detail;
+
+          if (debug_message == NULL)
+            debug_message = "(no message provided)";
+
+          if (self->priv->group_remove_error != NULL)
+            g_clear_error (&self->priv->group_remove_error);
+
+          if (error_detail != NULL)
+            {
+              /* CM specified a D-Bus error name */
+              tp_proxy_dbus_error_to_gerror (self, error_detail,
+                  debug_message == NULL || debug_message[0] == '\0'
+                      ? error_detail
+                      : debug_message,
+                  &self->priv->group_remove_error);
+
+              /* ... but if we don't know anything about that D-Bus error
+               * name, we can still do better by using RemovedFromGroup */
+              if (g_error_matches (self->priv->group_remove_error,
+                    TP_DBUS_ERRORS, TP_DBUS_ERROR_UNKNOWN_REMOTE_ERROR))
+                {
+                  self->priv->group_remove_error->domain =
+                    TP_ERRORS_REMOVED_FROM_GROUP;
+                  self->priv->group_remove_error->code = reason;
+
+                  _tp_channel_group_improve_remove_error (self, actor);
+                }
+            }
+          else
+            {
+              /* Use our separate error domain */
+              g_set_error_literal (&self->priv->group_remove_error,
+                  TP_ERRORS_REMOVED_FROM_GROUP, reason, debug_message);
+
+              _tp_channel_group_improve_remove_error (self, actor);
+            }
         }
-
-      /* FIXME: should check against the Connection's self-handle too,
-       * after I add that API */
     }
 
   g_signal_emit_by_name (self, "group-members-changed", message,
@@ -1114,6 +1239,9 @@ _tp_channel_get_group_properties (TpChannel *self)
   if (!tp_proxy_has_interface_by_id (self,
         TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP))
     {
+      _tp_proxy_set_feature_prepared ((TpProxy *) self,
+          TP_CHANNEL_FEATURE_GROUP, FALSE);
+
       DEBUG ("%p: not a Group, continuing", self);
       _tp_channel_continue_introspection (self);
       return;
@@ -1126,7 +1254,7 @@ _tp_channel_get_group_properties (TpChannel *self)
    * to these signals must succeed. */
 #define DIE(sig) \
   { \
-    g_critical ("couldn't connect to " sig ": %s", error->message); \
+    CRITICAL ("couldn't connect to " sig ": %s", error->message); \
     g_assert_not_reached (); \
     g_error_free (error); \
     return; \
