@@ -22,6 +22,7 @@
 #ifndef TP_CONNECTION_INTERNAL_H
 #define TP_CONNECTION_INTERNAL_H
 
+#include <telepathy-glib/capabilities.h>
 #include <telepathy-glib/connection.h>
 #include <telepathy-glib/contact.h>
 
@@ -30,13 +31,15 @@ G_BEGIN_DECLS
 typedef void (*TpConnectionProc) (TpConnection *self);
 
 struct _TpConnectionPrivate {
-    /* GArray of TpConnectionProc */
-    GArray *introspect_needed;
+    /* list of TpConnectionProc */
+    GList *introspect_needed;
 
     TpHandle self_handle;
     TpConnectionStatus status;
     TpConnectionStatusReason status_reason;
-    GError *connection_error /* initialized statically */;
+    gchar *connection_error;
+    /* a TP_HASH_TYPE_STRING_VARIANT_MAP */
+    GHashTable *connection_error_details;
 
     /* GArray of GQuark */
     GArray *contact_attribute_interfaces;
@@ -44,14 +47,37 @@ struct _TpConnectionPrivate {
     /* TpHandle => weak ref to TpContact */
     GHashTable *contacts;
 
+    TpCapabilities *capabilities;
+
+    TpAvatarRequirements *avatar_requirements;
+    GArray *avatar_request_queue;
+    guint avatar_request_idle_id;
+
+    TpContactInfoFlags contact_info_flags;
+    GList *contact_info_supported_fields;
+
+    TpProxyPendingCall *introspection_call;
+    unsigned fetching_rcc:1;
+    unsigned fetching_avatar_requirements:1;
+    unsigned contact_info_fetched:1;
+
     unsigned ready:1;
-    unsigned called_get_interfaces:1;
     unsigned tracking_aliases_changed:1;
     unsigned tracking_avatar_updated:1;
+    unsigned tracking_avatar_retrieved:1;
     unsigned tracking_presences_changed:1;
     unsigned tracking_presence_update:1;
+    unsigned tracking_location_changed:1;
+    unsigned tracking_contact_caps_changed:1;
+    unsigned tracking_contact_info_changed:1;
+    unsigned introspecting_after_connected:1;
 };
 
+void _tp_connection_status_reason_to_gerror (TpConnectionStatusReason reason,
+    TpConnectionStatus prev_status,
+    const gchar **ret_str,
+    GError **error);
+
 void _tp_connection_init_handle_refs (TpConnection *self);
 void _tp_connection_clean_up_handle_refs (TpConnection *self);
 
@@ -65,6 +91,14 @@ TpContact *_tp_connection_lookup_contact (TpConnection *self, TpHandle handle);
  * just for this would be overkill */
 void _tp_contact_connection_invalidated (TpContact *contact);
 
+/* connection-contact-info.c */
+void _tp_connection_maybe_prepare_contact_info (TpProxy *proxy);
+TpContactInfoFieldSpec *_tp_contact_info_field_spec_new (const gchar *name,
+    GStrv parameters, TpContactInfoFieldFlags flags, guint max);
+
+/* connection-avatars.c */
+void _tp_connection_maybe_prepare_avatar_requirements (TpProxy *proxy);
+
 G_END_DECLS
 
 #endif
