@@ -23,20 +23,25 @@
 
 #include <string.h>
 
+#include <dbus/dbus-protocol.h>
+
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/defs.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
+#include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/handle.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/proxy-subclass.h>
 #include <telepathy-glib/util.h>
 
 #define DEBUG_FLAG TP_DEBUG_CONNECTION
+#include "telepathy-glib/capabilities-internal.h"
 #include "telepathy-glib/connection-internal.h"
 #include "telepathy-glib/dbus-internal.h"
 #include "telepathy-glib/debug-internal.h"
+#include "telepathy-glib/proxy-internal.h"
 
 #include "_gen/tp-cli-connection-body.h"
 
@@ -60,6 +65,111 @@
  * Since: 0.7.1
  */
 
+/**
+ * TP_CONNECTION_FEATURE_CORE:
+ *
+ * Expands to a call to a function that returns a quark for the "core" feature
+ * on a #TpConnection.
+ *
+ * When this feature is prepared, the basic properties of the Connection have
+ * been retrieved and are available for use, and change-notification has been
+ * set up for those that can change.
+ *
+ * Specifically, this implies that:
+ *
+ * <itemizedlist>
+ *  <listitem>#TpConnection:status has a value other than
+ *    %TP_UNKNOWN_CONNECTION_STATUS, and #TpConnection:status-reason is
+ *    the reason for changing to that status</listitem>
+ *  <listitem>interfaces that are always available have been added to the
+ *    #TpProxy:interfaces (although the set of interfaces is not guaranteed
+ *    to be complete until #TpConnection:status becomes
+ *    %TP_CONNECTION_STATUS_CONNECTED))</listitem>
+ * </itemizedlist>
+ *
+ * <note>
+ *  <title>prepared does not imply connected</title>
+ *  <para>Unlike the older #TpConnection:connection-ready mechanism, this
+ *    feature does not imply that the connection has successfully connected.
+ *    It only implies that an initial status (disconnected, connecting or
+ *    connected) has been discovered. %TP_CONNECTION_FEATURE_CONNECTED
+ *    is the closest equivalent of #TpConnection:connection-ready.</para>
+ * </note>
+ *
+ * One can ask for a feature to be prepared using the
+ * tp_proxy_prepare_async() function, and waiting for it to callback.
+ *
+ * Since: 0.11.3
+ */
+
+GQuark
+tp_connection_get_feature_quark_core (void)
+{
+  return g_quark_from_static_string ("tp-connection-feature-core");
+}
+
+/**
+ * TP_CONNECTION_FEATURE_CONNECTED:
+ *
+ * Expands to a call to a function that returns a #GQuark representing the
+ * "connected" feature.
+ *
+ * When this feature is prepared, it means that the connection has become
+ * connected to the appropriate real-time communications service, and all
+ * information requested via other features has been updated accordingly.
+ * In particular, the following aspects of %TP_CONNECTION_FEATURE_CORE
+ * will be up to date:
+ *
+ * <itemizedlist>
+ *  <listitem>#TpConnection:status is
+ *    %TP_CONNECTION_STATUS_CONNECTED</listitem>
+ *  <listitem>#TpConnection:self-handle is valid and non-zero</listitem>
+ *  <listitem>all interfaces have been added to the set of
+ *    #TpProxy:interfaces, and that set will not change again</listitem>
+ * </itemizedlist>
+ *
+ * <note>
+ *   <title>Someone still has to call Connect()</title>
+ *   <para>Requesting this feature via tp_proxy_prepare_async() means that
+ *     you want to wait for the connection to connect, but it doesn't actually
+ *     start the process of connecting: to do that, call
+ *     tp_cli_connection_call_connect() separately.</para>
+ * </note>
+ *
+ * One can ask for a feature to be prepared using the
+ * tp_proxy_prepare_async() function, and waiting for it to callback.
+ *
+ * Since: 0.11.3
+ */
+
+GQuark
+tp_connection_get_feature_quark_connected (void)
+{
+  return g_quark_from_static_string ("tp-connection-feature-connected");
+}
+
+/**
+ * TP_CONNECTION_FEATURE_CAPABILITIES:
+ *
+ * Expands to a call to a function that returns a #GQuark representing the
+ * "capabilities" feature.
+ *
+ * When this feature is prepared, the Requests.RequestableChannelClasses
+ * property of the Connection has been retrieved.
+ * In particular, the %TpConnection:capabilities property has been set.
+ *
+ * One can ask for a feature to be prepared using the
+ * tp_proxy_prepare_async() function, and waiting for it to callback.
+ *
+ * Since: 0.11.3
+ */
+
+GQuark
+tp_connection_get_feature_quark_capabilities (void)
+{
+  return g_quark_from_static_string ("tp-connection-feature-capabilities");
+}
+
 /**
  * TP_ERRORS_DISCONNECTED:
  *
@@ -69,6 +179,9 @@
  *
  * This macro expands to a function call returning a #GQuark.
  *
+ * Since 0.7.24, this error domain is only used if a connection manager emits
+ * a #TpConnectionStatusReason not known to telepathy-glib.
+ *
  * Since: 0.7.1
  */
 GQuark
@@ -106,10 +219,9 @@ tp_errors_disconnected_quark (void)
 
 /**
  * TpConnection:
- * @parent: the parent class instance
- * @priv: pointer to opaque private data
  *
- * A proxy object for a Telepathy connection.
+ * A proxy object for a Telepathy connection. There are no interesting
+ * public struct fields.
  *
  * (Changed in 0.7.12: the layout of the structure is visible, allowing
  * subclassing.)
@@ -123,6 +235,7 @@ enum
   PROP_STATUS_REASON,
   PROP_CONNECTION_READY,
   PROP_SELF_HANDLE,
+  PROP_CAPABILITIES,
   N_PROPS
 };
 
@@ -152,33 +265,143 @@ tp_connection_get_property (GObject *object,
     case PROP_SELF_HANDLE:
       g_value_set_uint (value, self->priv->self_handle);
       break;
+    case PROP_CAPABILITIES:
+      g_value_set_object (value, self->priv->capabilities);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
     }
 }
 
+static void
+tp_connection_get_rcc_cb (TpProxy *proxy,
+    const GValue *value,
+    const GError *error,
+    gpointer user_data,
+    GObject *weak_object)
+{
+  TpConnection *self = (TpConnection *) proxy;
+
+  self->priv->fetching_rcc = FALSE;
+
+  if (error != NULL)
+    {
+      DEBUG ("Failed to get RequestableChannelClasses property, using an "
+          "empty set: %s", error->message);
+
+      /* it's NULL-safe */
+      self->priv->capabilities = _tp_capabilities_new (NULL, FALSE);
+      goto finally;
+    }
+
+  g_assert (self->priv->capabilities == NULL);
+
+  if (!G_VALUE_HOLDS (value, TP_ARRAY_TYPE_REQUESTABLE_CHANNEL_CLASS_LIST))
+    {
+      DEBUG ("RequestableChannelClasses is not of type a(a{sv}as), using an "
+          "empty set: %s", G_VALUE_TYPE_NAME (value));
+
+      /* it's NULL-safe */
+      self->priv->capabilities = _tp_capabilities_new (NULL, FALSE);
+      goto finally;
+    }
+
+  DEBUG ("CAPABILITIES ready");
+
+  self->priv->capabilities = _tp_capabilities_new (g_value_get_boxed (value),
+      FALSE);
+
+finally:
+  _tp_proxy_set_feature_prepared (proxy, TP_CONNECTION_FEATURE_CAPABILITIES,
+      TRUE);
+
+  g_object_notify ((GObject *) self, "capabilities");
+}
+
+static void
+tp_connection_maybe_prepare_capabilities (TpProxy *proxy)
+{
+  TpConnection *self = (TpConnection *) proxy;
+
+  if (self->priv->capabilities != NULL)
+    return;   /* already done */
+
+  if (!_tp_proxy_is_preparing (proxy, TP_CONNECTION_FEATURE_CAPABILITIES))
+    return;   /* not interested right now */
+
+  if (!self->priv->ready)
+    return;   /* will try again when ready */
+
+  if (self->priv->fetching_rcc)
+    return;   /* Another Get operation is running */
+
+  if (!tp_proxy_has_interface_by_id (proxy,
+        TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
+    {
+      /* Connection doesn't support Requests; set an empty TpCapabilities
+       * object as all calls to CreateChannel/EnsureChannel will fail. */
+
+      self->priv->capabilities = _tp_capabilities_new (NULL, FALSE);
+      _tp_proxy_set_feature_prepared (proxy, TP_CONNECTION_FEATURE_CAPABILITIES,
+          TRUE);
+      return;
+    }
+
+  self->priv->fetching_rcc = TRUE;
+
+  tp_cli_dbus_properties_call_get (self, -1,
+      TP_IFACE_CONNECTION_INTERFACE_REQUESTS, "RequestableChannelClasses",
+      tp_connection_get_rcc_cb, NULL, NULL, NULL);
+}
+
 static void
 tp_connection_continue_introspection (TpConnection *self)
 {
-  g_assert (self->priv->introspect_needed != NULL);
-
-  if (self->priv->introspect_needed->len == 0)
+  if (tp_proxy_get_invalidated (self) != NULL)
     {
-      g_array_free (self->priv->introspect_needed, TRUE);
-      self->priv->introspect_needed = NULL;
+      DEBUG ("Already invalidated: not becoming ready");
+      return;
+    }
 
-      DEBUG ("%p: connection ready", self);
+  if (self->priv->introspect_needed == NULL)
+    {
+      if (!self->priv->introspecting_after_connected)
+        {
+          /* Introspection will restart when we become CONNECTED */
+          DEBUG ("CORE ready, but not CONNECTED");
+          _tp_proxy_set_feature_prepared ((TpProxy *) self,
+            TP_CONNECTION_FEATURE_CORE, TRUE);
+          return;
+        }
+
+      /* signal CONNECTED; we shouldn't have gone to status CONNECTED for any
+       * reason that isn't REQUESTED :-) */
+      DEBUG ("%p: CORE and CONNECTED ready", self);
+      self->priv->status = TP_CONNECTION_STATUS_CONNECTED;
+      self->priv->status_reason = TP_CONNECTION_STATUS_REASON_REQUESTED;
       self->priv->ready = TRUE;
+
+      _tp_proxy_set_feature_prepared ((TpProxy *) self,
+          TP_CONNECTION_FEATURE_CONNECTED, TRUE);
+      _tp_proxy_set_feature_prepared ((TpProxy *) self,
+          TP_CONNECTION_FEATURE_CORE, TRUE);
+
+      g_object_notify ((GObject *) self, "status");
+      g_object_notify ((GObject *) self, "status-reason");
       g_object_notify ((GObject *) self, "connection-ready");
+
+      tp_connection_maybe_prepare_capabilities ((TpProxy *) self);
+      _tp_connection_maybe_prepare_avatar_requirements ((TpProxy *) self);
+      _tp_connection_maybe_prepare_contact_info ((TpProxy *) self);
     }
   else
     {
-      guint i = self->priv->introspect_needed->len - 1;
-      TpConnectionProc next = g_array_index (self->priv->introspect_needed,
-          TpConnectionProc, i);
+      TpConnectionProc next = self->priv->introspect_needed->data;
 
-      g_array_remove_index (self->priv->introspect_needed, i);
+      self->priv->introspect_needed = g_list_delete_link (
+          self->priv->introspect_needed,
+          self->priv->introspect_needed);
       next (self);
     }
 }
@@ -193,6 +416,9 @@ got_contact_attribute_interfaces (TpProxy *proxy,
   TpConnection *self = TP_CONNECTION (proxy);
   GArray *arr;
 
+  g_assert (self->priv->introspection_call != NULL);
+  self->priv->introspection_call = NULL;
+
   if (error == NULL && G_VALUE_HOLDS (value, G_TYPE_STRV))
     {
       gchar **interfaces = g_value_get_boxed (value);
@@ -243,9 +469,15 @@ got_contact_attribute_interfaces (TpProxy *proxy,
 static void
 introspect_contacts (TpConnection *self)
 {
-  g_assert (self->priv->introspect_needed != NULL);
+  /* "This cannot change during the lifetime of the Connection." -- tp-spec */
+  if (self->priv->contact_attribute_interfaces != NULL)
+    {
+      tp_connection_continue_introspection (self);
+      return;
+    }
 
-  tp_cli_dbus_properties_call_get (self, -1,
+  g_assert (self->priv->introspection_call == NULL);
+  self->priv->introspection_call = tp_cli_dbus_properties_call_get (self, -1,
        TP_IFACE_CONNECTION_INTERFACE_CONTACTS, "ContactAttributeInterfaces",
        got_contact_attribute_interfaces, NULL, NULL, NULL);
 }
@@ -268,6 +500,8 @@ got_self_handle (TpConnection *self,
                  gpointer user_data G_GNUC_UNUSED,
                  GObject *user_object G_GNUC_UNUSED)
 {
+  g_assert (self->priv->introspection_call != NULL);
+  self->priv->introspection_call = NULL;
 
   if (error != NULL)
     {
@@ -292,8 +526,14 @@ on_self_handle_changed (TpConnection *self,
 static void
 get_self_handle (TpConnection *self)
 {
-  g_assert (self->priv->introspect_needed != NULL);
+  if (!self->priv->introspecting_after_connected)
+    {
+      tp_connection_continue_introspection (self);
+      return;
+    }
 
+  /* this only happens when we introspect after CONNECTED, so there's no need
+   * to track whether this is the first time */
   tp_cli_connection_connect_to_self_handle_changed (self,
       on_self_handle_changed, NULL, NULL, NULL, NULL);
 
@@ -301,10 +541,14 @@ get_self_handle (TpConnection *self)
    * but until Connection has other interesting properties, there's no point in
    * trying to implement a fast path; GetSelfHandle is the only one guaranteed
    * to work, so we'll sometimes have to call it anyway */
-  tp_cli_connection_call_get_self_handle (self, -1,
-       got_self_handle, NULL, NULL, NULL);
+  g_assert (self->priv->introspection_call == NULL);
+  self->priv->introspection_call = tp_cli_connection_call_get_self_handle (
+      self, -1, got_self_handle, NULL, NULL, NULL);
 }
 
+/* Appending callbacks to self->priv->introspect_needed relies on this */
+G_STATIC_ASSERT (sizeof (TpConnectionProc) <= sizeof (gpointer));
+
 static void
 tp_connection_got_interfaces_cb (TpConnection *self,
                                  const gchar **interfaces,
@@ -312,7 +556,8 @@ tp_connection_got_interfaces_cb (TpConnection *self,
                                  gpointer user_data,
                                  GObject *user_object)
 {
-  TpConnectionProc func;
+  g_assert (self->priv->introspection_call != NULL);
+  self->priv->introspection_call = NULL;
 
   if (error != NULL)
     {
@@ -331,11 +576,8 @@ tp_connection_got_interfaces_cb (TpConnection *self,
     }
 
   g_assert (self->priv->introspect_needed == NULL);
-  self->priv->introspect_needed = g_array_new (FALSE, FALSE,
-      sizeof (TpConnectionProc));
-
-  func = get_self_handle;
-  g_array_append_val (self->priv->introspect_needed, func);
+  self->priv->introspect_needed = g_list_append (self->priv->introspect_needed,
+    get_self_handle);
 
   if (interfaces != NULL)
     {
@@ -347,13 +589,12 @@ tp_connection_got_interfaces_cb (TpConnection *self,
             {
               GQuark q = g_quark_from_string (*iter);
 
-              tp_proxy_add_interface_by_id ((TpProxy *) self,
-                  g_quark_from_string (*iter));
+              tp_proxy_add_interface_by_id ((TpProxy *) self, q);
 
               if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACTS)
                 {
-                  func = introspect_contacts;
-                  g_array_append_val (self->priv->introspect_needed, func);
+                  self->priv->introspect_needed = g_list_append (
+                      self->priv->introspect_needed, introspect_contacts);
                 }
             }
           else
@@ -376,17 +617,36 @@ tp_connection_status_changed (TpConnection *self,
 {
   DEBUG ("%p: %d -> %d because %d", self, self->priv->status, status, reason);
 
-  self->priv->status = status;
-  self->priv->status_reason = reason;
-  g_object_notify ((GObject *) self, "status");
-  g_object_notify ((GObject *) self, "status-reason");
-
-  if (status == TP_CONNECTION_STATUS_CONNECTED &&
-      !self->priv->called_get_interfaces)
+  if (status == TP_CONNECTION_STATUS_CONNECTED)
     {
-      tp_cli_connection_call_get_interfaces (self, -1,
-          tp_connection_got_interfaces_cb, NULL, NULL, NULL);
-      self->priv->called_get_interfaces = TRUE;
+      if (self->priv->introspection_call != NULL &&
+          !self->priv->introspecting_after_connected)
+        {
+          /* We thought we knew what was going on, but now the connection has
+           * gone to CONNECTED and all bets are off. Start again! */
+          DEBUG ("Cancelling pre-CONNECTED introspection and starting again");
+          tp_proxy_pending_call_cancel (self->priv->introspection_call);
+          self->priv->introspection_call = NULL;
+          g_list_free (self->priv->introspect_needed);
+          self->priv->introspect_needed = NULL;
+        }
+
+      self->priv->introspecting_after_connected = TRUE;
+
+      /* we defer the perceived change to CONNECTED until ready */
+      if (self->priv->introspection_call == NULL)
+        {
+          self->priv->introspection_call =
+            tp_cli_connection_call_get_interfaces (self, -1,
+                tp_connection_got_interfaces_cb, NULL, NULL, NULL);
+        }
+    }
+  else
+    {
+      self->priv->status = status;
+      self->priv->status_reason = reason;
+      g_object_notify ((GObject *) self, "status");
+      g_object_notify ((GObject *) self, "status-reason");
     }
 }
 
@@ -397,21 +657,21 @@ tp_connection_connection_error_cb (TpConnection *self,
                                    gpointer user_data,
                                    GObject *weak_object)
 {
-  if (self->priv->connection_error != NULL)
-    {
-      g_error_free (self->priv->connection_error);
-      self->priv->connection_error = NULL;
-    }
+  g_free (self->priv->connection_error);
+  self->priv->connection_error = g_strdup (error_name);
 
-  tp_proxy_dbus_error_to_gerror (self, error_name,
-        tp_asv_get_string (details, "debug-message"),
-        &(self->priv->connection_error));
+  if (self->priv->connection_error_details != NULL)
+    g_hash_table_unref (self->priv->connection_error_details);
+
+  self->priv->connection_error_details = g_boxed_copy (
+      TP_HASH_TYPE_STRING_VARIANT_MAP, details);
 }
 
-static void
-tp_connection_status_reason_to_gerror (TpConnectionStatusReason reason,
-                                       TpConnectionStatus prev_status,
-                                       GError **error)
+void
+_tp_connection_status_reason_to_gerror (TpConnectionStatusReason reason,
+    TpConnectionStatus prev_status,
+    const gchar **ret_str,
+    GError **error)
 {
   TpError code;
   const gchar *message;
@@ -499,10 +759,17 @@ tp_connection_status_reason_to_gerror (TpConnectionStatusReason reason,
     default:
       g_set_error (error, TP_ERRORS_DISCONNECTED, reason,
           "Unknown disconnection reason");
+
+      if (ret_str != NULL)
+        *ret_str = TP_ERROR_STR_DISCONNECTED;
+
       return;
     }
 
   g_set_error (error, TP_ERRORS, code, "%s", message);
+
+  if (ret_str != NULL)
+    *ret_str = tp_error_get_dbus_name (code);
 }
 
 static void
@@ -528,16 +795,38 @@ tp_connection_status_changed_cb (TpConnection *self,
 
   if (status == TP_CONNECTION_STATUS_DISCONNECTED)
     {
+      GError *error = NULL;
+
       if (self->priv->connection_error == NULL)
         {
-          tp_connection_status_reason_to_gerror (reason, prev_status,
-              &(self->priv->connection_error));
+          _tp_connection_status_reason_to_gerror (reason, prev_status,
+              NULL, &error);
+        }
+      else
+        {
+          g_assert (self->priv->connection_error_details != NULL);
+          tp_proxy_dbus_error_to_gerror (self, self->priv->connection_error,
+              tp_asv_get_string (self->priv->connection_error_details,
+                "debug-message"), &error);
+
+          /* ... but if we don't know anything about that D-Bus error
+           * name, we can still be more helpful by deriving an error code from
+           * TpConnectionStatusReason */
+          if (g_error_matches (error, TP_DBUS_ERRORS,
+                TP_DBUS_ERROR_UNKNOWN_REMOTE_ERROR))
+            {
+              GError *from_csr = NULL;
+
+              _tp_connection_status_reason_to_gerror (reason, prev_status,
+                  NULL, &from_csr);
+              error->domain = from_csr->domain;
+              error->code = from_csr->code;
+              g_error_free (from_csr);
+            }
         }
 
-      tp_proxy_invalidate ((TpProxy *) self, self->priv->connection_error);
-
-      g_error_free (self->priv->connection_error);
-      self->priv->connection_error = NULL;
+      tp_proxy_invalidate ((TpProxy *) self, error);
+      g_error_free (error);
     }
 }
 
@@ -550,11 +839,23 @@ tp_connection_got_status_cb (TpConnection *self,
 {
   DEBUG ("%p", self);
 
+  g_assert (self->priv->introspection_call != NULL);
+  self->priv->introspection_call = NULL;
+
   if (error == NULL)
     {
       DEBUG ("%p: Initial status is %d", self, status);
       tp_connection_status_changed (self, status,
           TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED);
+
+      /* try introspecting before CONNECTED - it might work... */
+      if (status != TP_CONNECTION_STATUS_CONNECTED &&
+          self->priv->introspection_call == NULL)
+        {
+          self->priv->introspection_call =
+            tp_cli_connection_call_get_interfaces (self, -1,
+                tp_connection_got_interfaces_cb, NULL, NULL, NULL);
+        }
     }
   else
     {
@@ -567,6 +868,13 @@ tp_connection_got_status_cb (TpConnection *self,
 static void
 tp_connection_invalidated (TpConnection *self)
 {
+  if (self->priv->introspection_call != NULL)
+    {
+      DEBUG ("Cancelling introspection");
+      tp_proxy_pending_call_cancel (self->priv->introspection_call);
+      self->priv->introspection_call = NULL;
+    }
+
   _tp_connection_set_self_handle (self, 0);
   _tp_connection_clean_up_handle_refs (self);
 }
@@ -591,7 +899,8 @@ tp_connection_constructor (GType type,
 
   /* get my initial status */
   DEBUG ("Calling GetStatus");
-  tp_cli_connection_call_get_status (self, -1,
+  g_assert (self->priv->introspection_call == NULL);
+  self->priv->introspection_call = tp_cli_connection_call_get_status (self, -1,
       tp_connection_got_status_cb, NULL, NULL, NULL);
 
   _tp_connection_init_handle_refs (self);
@@ -614,6 +923,7 @@ tp_connection_init (TpConnection *self)
   self->priv->status = TP_UNKNOWN_CONNECTION_STATUS;
   self->priv->status_reason = TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED;
   self->priv->contacts = g_hash_table_new (g_direct_hash, g_direct_equal);
+  self->priv->introspection_call = NULL;
 }
 
 static void
@@ -626,7 +936,7 @@ tp_connection_finalize (GObject *object)
   /* not true unless we were finalized before we were ready */
   if (self->priv->introspect_needed != NULL)
     {
-      g_array_free (self->priv->introspect_needed, TRUE);
+      g_list_free (self->priv->introspect_needed);
       self->priv->introspect_needed = NULL;
     }
 
@@ -636,16 +946,33 @@ tp_connection_finalize (GObject *object)
       self->priv->contact_attribute_interfaces = NULL;
     }
 
-  if (self->priv->connection_error != NULL)
+  g_free (self->priv->connection_error);
+  self->priv->connection_error = NULL;
+
+  if (self->priv->connection_error_details != NULL)
     {
-      g_error_free (self->priv->connection_error);
-      self->priv->connection_error = NULL;
+      g_hash_table_unref (self->priv->connection_error_details);
+      self->priv->connection_error_details = NULL;
     }
 
+  if (self->priv->avatar_request_queue != NULL)
+    {
+      g_array_free (self->priv->avatar_request_queue, TRUE);
+      self->priv->avatar_request_queue = NULL;
+    }
+
+  if (self->priv->avatar_request_idle_id != 0)
+    {
+      g_source_remove (self->priv->avatar_request_idle_id);
+      self->priv->avatar_request_idle_id = 0;
+    }
+
+  tp_contact_info_spec_list_free (self->priv->contact_info_supported_fields);
+  self->priv->contact_info_supported_fields = NULL;
+
   ((GObjectClass *) tp_connection_parent_class)->finalize (object);
 }
 
-
 static void
 contact_notify_invalidated (gpointer k G_GNUC_UNUSED,
                             gpointer v,
@@ -666,13 +993,56 @@ tp_connection_dispose (GObject *object)
     {
       g_hash_table_foreach (self->priv->contacts, contact_notify_invalidated,
           NULL);
-      g_hash_table_destroy (self->priv->contacts);
-      self->priv->contacts = NULL;
+      tp_clear_pointer (&self->priv->contacts, g_hash_table_destroy);
     }
 
+  tp_clear_object (&self->priv->capabilities);
+  tp_clear_pointer (&self->priv->avatar_requirements,
+      tp_avatar_requirements_destroy);
+
   ((GObjectClass *) tp_connection_parent_class)->dispose (object);
 }
 
+enum {
+    FEAT_CORE,
+    FEAT_CONNECTED,
+    FEAT_CAPABILITIES,
+    FEAT_AVATAR_REQUIREMENTS,
+    FEAT_CONTACT_INFO,
+    N_FEAT
+};
+
+static const TpProxyFeature *
+tp_connection_list_features (TpProxyClass *cls G_GNUC_UNUSED)
+{
+  static TpProxyFeature features[N_FEAT + 1] = { { 0 } };
+
+  if (G_LIKELY (features[0].name != 0))
+    return features;
+
+  features[FEAT_CORE].name = TP_CONNECTION_FEATURE_CORE;
+  features[FEAT_CORE].core = TRUE;
+
+  features[FEAT_CONNECTED].name = TP_CONNECTION_FEATURE_CONNECTED;
+
+  features[FEAT_CAPABILITIES].name = TP_CONNECTION_FEATURE_CAPABILITIES;
+  features[FEAT_CAPABILITIES].start_preparing =
+    tp_connection_maybe_prepare_capabilities;
+
+  features[FEAT_AVATAR_REQUIREMENTS].name = TP_CONNECTION_FEATURE_AVATAR_REQUIREMENTS;
+  features[FEAT_AVATAR_REQUIREMENTS].start_preparing =
+    _tp_connection_maybe_prepare_avatar_requirements;
+
+  features[FEAT_CONTACT_INFO].name = TP_CONNECTION_FEATURE_CONTACT_INFO;
+  features[FEAT_CONTACT_INFO].start_preparing =
+    _tp_connection_maybe_prepare_contact_info;
+
+  /* assert that the terminator at the end is there */
+  g_assert (features[N_FEAT].name == 0);
+
+  return features;
+}
+
 static void
 tp_connection_class_init (TpConnectionClass *klass)
 {
@@ -693,12 +1063,20 @@ tp_connection_class_init (TpConnectionClass *klass)
   /* If you change this, you must also change TpChannel to stop asserting
    * that its connection has a unique name */
   proxy_class->must_have_unique_name = TRUE;
+  proxy_class->list_features = tp_connection_list_features;
 
   /**
    * TpConnection:status:
    *
-   * This connection's status, or TP_UNKNOWN_CONNECTION_STATUS if we don't
+   * This connection's status, or %TP_UNKNOWN_CONNECTION_STATUS if we don't
    * know yet.
+   *
+   * To wait for a valid status (and other properties), call
+   * tp_proxy_prepare_async() with the feature %TP_CONNECTION_FEATURE_CORE.
+   *
+   * Since version 0.11.3, the change to status
+   * %TP_CONNECTION_STATUS_CONNECTED is delayed slightly, until introspection
+   * of the connection has finished.
    */
   param_spec = g_param_spec_uint ("status", "Status",
       "The status of this connection", 0, G_MAXUINT32,
@@ -713,6 +1091,10 @@ tp_connection_class_init (TpConnectionClass *klass)
    *
    * The %TP_HANDLE_TYPE_CONTACT handle of the local user on this connection,
    * or 0 if we don't know yet or if the connection has become invalid.
+   *
+   * To wait for a valid self-handle (and other properties), call
+   * tp_proxy_prepare_async() with the feature
+   * %TP_CONNECTION_FEATURE_CONNECTED.
    */
   param_spec = g_param_spec_uint ("self-handle", "Self handle",
       "The local user's Contact handle on this connection", 0, G_MAXUINT32,
@@ -724,6 +1106,9 @@ tp_connection_class_init (TpConnectionClass *klass)
   /**
    * TpConnection:status-reason:
    *
+   * To wait for a valid status (and other properties), call
+   * tp_proxy_prepare_async() with the feature %TP_CONNECTION_FEATURE_CORE.
+   *
    * The reason why #TpConnection:status changed to its current value,
    * or TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED if unknown.
    * know yet.
@@ -745,6 +1130,10 @@ tp_connection_class_init (TpConnectionClass *klass)
    * By the time this property becomes %TRUE, any extra interfaces will
    * have been set up and the #TpProxy:interfaces property will have been
    * populated.
+   *
+   * This is similar to %TP_CONNECTION_FEATURE_CONNECTED, except that once
+   * it has changed to %TRUE, it remains %TRUE even if the connection has
+   * been invalidated.
    */
   param_spec = g_param_spec_boolean ("connection-ready", "Connection ready?",
       "Initially FALSE; changes to TRUE when introspection finishes", FALSE,
@@ -752,18 +1141,34 @@ tp_connection_class_init (TpConnectionClass *klass)
       | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NICK);
   g_object_class_install_property (object_class, PROP_CONNECTION_READY,
       param_spec);
+
+ /**
+   * TpConnection:capabilities:
+   *
+   * The %TpCapabilities object representing the capabilities of this
+   * connection, or NULL if we don't know yet.
+   *
+   * To wait for valid capability information, call tp_proxy_prepare_async()
+   * with the feature %TP_CONNECTION_FEATURE_CAPABILITIES.
+   */
+  param_spec = g_param_spec_object ("capabilities", "Capabilities",
+      "A TpCapabilities object representing the capabilities of the connection",
+      TP_TYPE_CAPABILITIES,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_CAPABILITIES,
+      param_spec);
 }
 
 /**
  * tp_connection_new:
  * @dbus: a D-Bus daemon; may not be %NULL
- * @bus_name: the well-known or unique name of the connection process;
- *  if well-known, this function will make a blocking call to the bus daemon
- *  to resolve the unique name. May be %NULL if @object_path is not, in which
- *  case a well-known name will be derived from @object_path.
- * @object_path: the object path of the connection process. May be %NULL
- *  if @bus_name is a well-known name, in which case the object path will
- *  be derived from @bus_name.
+ * @bus_name: (allow-none): the well-known or unique name of the connection
+ *  process; if well-known, this function will make a blocking call to the bus
+ *  daemon to resolve the unique name. May be %NULL if @object_path is not, in
+ *  which case a well-known name will be derived from @object_path.
+ * @object_path: (allow-none): the object path of the connection process.
+ *  May be %NULL if @bus_name is a well-known name, in which case the object
+ *  path will be derived from @bus_name.
  * @error: used to indicate the error if %NULL is returned
  *
  * <!-- -->
@@ -839,9 +1244,8 @@ finally:
  * @self: a connection
  *
  * Return the %TP_HANDLE_TYPE_CONTACT handle of the local user on this
- * connection, or 0 if the connection is not ready (the
- * TpConnection:connection-ready property is false) or has become invalid
- * (the TpProxy::invalidated signal).
+ * connection, or 0 if the self-handle is not known yet or the connection
+ * has become invalid (the TpProxy::invalidated signal).
  *
  * The returned handle is not necessarily valid forever (the
  * notify::self-handle signal will be emitted if it changes, which can happen
@@ -863,7 +1267,7 @@ tp_connection_get_self_handle (TpConnection *self)
 /**
  * tp_connection_get_status:
  * @self: a connection
- * @reason: a TpConnectionStatusReason, or %NULL
+ * @reason: (out): a TpConnectionStatusReason, or %NULL
  *
  * If @reason is not %NULL it is set to the reason why "status" changed to its
  * current value, or %TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED if unknown.
@@ -886,7 +1290,7 @@ tp_connection_get_status (TpConnection *self,
 }
 
 /**
- * tp_connection_run_until_ready:
+ * tp_connection_run_until_ready: (skip)
  * @self: a connection
  * @connect: if %TRUE, call Connect() if it appears to be necessary;
  *  if %FALSE, rely on Connect() to be called by another client
@@ -904,6 +1308,9 @@ tp_connection_get_status (TpConnection *self,
  *  %FALSE if the connection has become invalid.
  *
  * Since: 0.7.1
+ * Deprecated: 0.11.0: Use tp_connection_call_when_ready,
+ *  or restructure your program in such a way as to avoid re-entering the
+ *  main loop.
  */
 
 typedef struct {
@@ -1014,14 +1421,15 @@ raise_invalidated:
 
 /**
  * TpConnectionNameListCb:
- * @names: %NULL-terminated array of @n connection bus names,
- *   or %NULL on error
+ * @names: (array zero-terminated=1): %NULL-terminated array of @n
+ *  connection bus names, or %NULL on error
  * @n: number of names (not including the final %NULL), or 0 on error
- * @cms: %NULL-terminated array of @n connection manager names
- *   (e.g. "gabble") in the same order as @names, or %NULL on error
- * @protocols: %NULL-terminated array of @n protocol names as defined in the
- *   Telepathy spec (e.g. "jabber") in the same order as @names, or %NULL on
- *   error
+ * @cms: (array zero-terminated=1): %NULL-terminated array of @n
+ *  connection manager names (e.g. "gabble") in the same order as @names, or
+ *  %NULL on error
+ * @protocols: (array zero-terminated=1): %NULL-terminated array of
+ *  @n protocol names as defined in the Telepathy spec (e.g. "jabber") in the
+ *  same order as @names, or %NULL on error
  * @error: %NULL on success, or an error that occurred
  * @user_data: user-supplied data
  * @weak_object: user-supplied weakly referenced object
@@ -1206,7 +1614,7 @@ list_context_free (gpointer p)
  * @destroy: callback to destroy the user-supplied data, called after
  *   @callback, but also if the D-Bus connection fails or if the @weak_object
  *   goes away
- * @weak_object: if not %NULL, will be weakly referenced; the callback will
+ * @weak_object: (allow-none): if not %NULL, will be weakly referenced; the callback will
  *   not be called if the object has vanished
  *
  * List the bus names of all the connections that currently exist, together
@@ -1338,7 +1746,7 @@ cwr_ready (TpConnection *self,
  */
 
 /**
- * tp_connection_call_when_ready:
+ * tp_connection_call_when_ready: (skip)
  * @self: a connection
  * @callback: called when the connection becomes ready or invalidated,
  *  whichever happens first
@@ -1454,9 +1862,10 @@ tp_connection_presence_type_cmp_availability (TpConnectionPresenceType p1,
 /**
  * tp_connection_parse_object_path:
  * @self: a connection
- * @protocol: If not NULL, used to return the protocol of the connection
- * @cm_name: If not NULL, used to return the connection manager name of the
- * connection
+ * @protocol: (out) (transfer full): If not NULL, used to return the protocol
+ *  of the connection
+ * @cm_name: (out) (transfer full): If not NULL, used to return the connection
+ *  manager name of the connection
  *
  * If the object path of @connection is in the correct form, set
  * @protocol and @cm_name, return TRUE. Otherwise leave them unchanged and
@@ -1524,7 +1933,7 @@ _tp_connection_add_contact (TpConnection *self,
 
 
 /**
- * tp_connection_is_ready:
+ * tp_connection_is_ready: (skip)
  * @self: a connection
  *
  * Returns the same thing as the #TpConnection:connection-ready property.
@@ -1539,3 +1948,104 @@ tp_connection_is_ready (TpConnection *self)
 
   return self->priv->ready;
 }
+
+/**
+ * tp_connection_get_capabilities:
+ * @self: a connection
+ *
+ * <!-- -->
+ *
+ * Returns: (transfer none): the same #TpCapabilities as the
+ * #TpConnection:capabilities property
+ * Since: 0.11.3
+ */
+TpCapabilities *
+tp_connection_get_capabilities (TpConnection *self)
+{
+  g_return_val_if_fail (TP_IS_CONNECTION (self), FALSE);
+
+  return self->priv->capabilities;
+}
+
+/**
+ * tp_connection_get_detailed_error:
+ * @self: a connection
+ * @details: (out) (allow-none) (element-type utf8 GObject.Value) (transfer none):
+ *  optionally used to return a map from string to #GValue, which must not be
+ *  modified or destroyed by the caller
+ *
+ * If the connection has disconnected, return the D-Bus error name with which
+ * it disconnected (in particular, this is %TP_ERROR_STR_CANCELLED if it was
+ * disconnected by a user request).
+ *
+ * Otherwise, return %NULL, without altering @details.
+ *
+ * Returns: (transfer none) (allow-none): a D-Bus error name, or %NULL.
+ *
+ * Since: 0.11.4
+ */
+const gchar *
+tp_connection_get_detailed_error (TpConnection *self,
+    const GHashTable **details)
+{
+  TpProxy *proxy = (TpProxy *) self;
+
+  if (proxy->invalidated == NULL)
+    return NULL;
+
+  if (self->priv->connection_error != NULL)
+    {
+      g_assert (self->priv->connection_error_details != NULL);
+
+      if (details != NULL)
+        *details = self->priv->connection_error_details;
+
+      return self->priv->connection_error;
+    }
+  else
+    {
+      /* no detailed error, but we *have* been invalidated - guess one based
+       * on the invalidation reason, and don't give any details */
+
+      if (details != NULL)
+        {
+          if (self->priv->connection_error_details == NULL)
+            self->priv->connection_error_details = g_hash_table_new (
+                g_str_hash, g_str_equal);
+
+          g_assert (g_hash_table_size (self->priv->connection_error_details)
+              == 0);
+
+          *details = self->priv->connection_error_details;
+        }
+
+      if (proxy->invalidated->domain == TP_ERRORS)
+        {
+          return tp_error_get_dbus_name (proxy->invalidated->code);
+        }
+      else if (proxy->invalidated->domain == TP_DBUS_ERRORS)
+        {
+          switch (proxy->invalidated->code)
+            {
+            case TP_DBUS_ERROR_NAME_OWNER_LOST:
+              /* the CM probably crashed */
+              return DBUS_ERROR_NO_REPLY;
+              break;
+
+            case TP_DBUS_ERROR_OBJECT_REMOVED:
+            case TP_DBUS_ERROR_UNKNOWN_REMOTE_ERROR:
+            case TP_DBUS_ERROR_INCONSISTENT:
+            /* ... and all other cases up to and including
+             * TP_DBUS_ERROR_INCONSISTENT don't make sense in this context, so
+             * just use the generic one for them too */
+            default:
+              return TP_ERROR_STR_DISCONNECTED;
+            }
+        }
+      else
+        {
+          /* no idea what that means */
+          return TP_ERROR_STR_DISCONNECTED;
+        }
+    }
+}
