@@ -120,7 +120,7 @@ tp_dbus_daemon_dup (GError **error)
 }
 
 /**
- * tp_dbus_daemon_new:
+ * tp_dbus_daemon_new: (skip)
  * @connection: a connection to D-Bus
  *
  * Returns a proxy for signals and method calls on a particular bus
@@ -505,7 +505,7 @@ tp_dbus_daemon_watch_name_owner (TpDBusDaemon *self,
           DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "GetNameOwner");
 
       if (message == NULL)
-        g_error ("Out of memory");
+        ERROR ("Out of memory");
 
       /* We already checked that @name was in (a small subset of) UTF-8,
        * so OOM is the only thing that can go wrong. The use of &name here
@@ -513,11 +513,11 @@ tp_dbus_daemon_watch_name_owner (TpDBusDaemon *self,
       if (!dbus_message_append_args (message,
             DBUS_TYPE_STRING, &name,
             DBUS_TYPE_INVALID))
-        g_error ("Out of memory");
+        ERROR ("Out of memory");
 
       if (!dbus_connection_send_with_reply (self->priv->libdbus,
           message, &pc, -1))
-        g_error ("Out of memory");
+        ERROR ("Out of memory");
       /* pc is unreffed by _tp_dbus_daemon_get_name_owner_notify */
 
       if (pc == NULL || dbus_pending_call_get_completed (pc))
@@ -530,7 +530,7 @@ tp_dbus_daemon_watch_name_owner (TpDBusDaemon *self,
             _tp_dbus_daemon_get_name_owner_notify,
             context, get_name_owner_context_unref))
         {
-          g_error ("Out of memory");
+          ERROR ("Out of memory");
         }
     }
   else
@@ -703,12 +703,12 @@ _tp_dbus_daemon_get_name_owner (TpDBusDaemon *self,
       DBUS_INTERFACE_DBUS, "GetNameOwner");
 
   if (message == NULL)
-    g_error ("Out of memory");
+    ERROR ("Out of memory");
 
   if (!dbus_message_append_args (message,
         DBUS_TYPE_STRING, &well_known_name,
         DBUS_TYPE_INVALID))
-    g_error ("Out of memory");
+    ERROR ("Out of memory");
 
   dbus_error_init (&dbus_error);
   reply = dbus_connection_send_with_reply_and_block (dbc, message,
@@ -719,7 +719,7 @@ _tp_dbus_daemon_get_name_owner (TpDBusDaemon *self,
   if (reply == NULL)
     {
       if (!tp_strdiff (dbus_error.name, DBUS_ERROR_NO_MEMORY))
-        g_error ("Out of memory");
+        ERROR ("Out of memory");
 
       /* FIXME: ideally we'd use dbus-glib's error mapping for this */
       g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_NAME_OWNER_LOST,
@@ -914,6 +914,55 @@ tp_dbus_daemon_release_name (TpDBusDaemon *self,
     }
 }
 
+/**
+ * tp_dbus_daemon_register_object:
+ * @self: object representing a connection to a bus
+ * @object_path: an object path
+ * @object: (type GObject.Object) (transfer none): an object to export
+ *
+ * Export @object at @object_path. This is a convenience wrapper around
+ * dbus_g_connection_register_g_object(), and behaves similarly.
+ *
+ * Since: 0.11.3
+ */
+void
+tp_dbus_daemon_register_object (TpDBusDaemon *self,
+    const gchar *object_path,
+    gpointer object)
+{
+  TpProxy *as_proxy = (TpProxy *) self;
+
+  g_return_if_fail (TP_IS_DBUS_DAEMON (self));
+  g_return_if_fail (tp_dbus_check_valid_object_path (object_path, NULL));
+  g_return_if_fail (G_IS_OBJECT (object));
+
+  dbus_g_connection_register_g_object (as_proxy->dbus_connection,
+      object_path, object);
+}
+
+/**
+ * tp_dbus_daemon_unregister_object:
+ * @self: object representing a connection to a bus
+ * @object: (type GObject.Object) (transfer none): an object previously exported
+ * with tp_dbus_daemon_register_object()
+ *
+ * Stop exporting @object on D-Bus. This is a convenience wrapper around
+ * dbus_g_connection_unregister_g_object(), and behaves similarly.
+ *
+ * Since: 0.11.3
+ */
+void
+tp_dbus_daemon_unregister_object (TpDBusDaemon *self,
+    gpointer object)
+{
+  TpProxy *as_proxy = (TpProxy *) self;
+
+  g_return_if_fail (TP_IS_DBUS_DAEMON (self));
+  g_return_if_fail (G_IS_OBJECT (object));
+
+  dbus_g_connection_unregister_g_object (as_proxy->dbus_connection, object);
+}
+
 /**
  * tp_dbus_daemon_get_unique_name:
  * @self: object representing a connection to a bus
@@ -1114,11 +1163,11 @@ _tp_dbus_daemon_list_names_common (TpDBusDaemon *self,
       DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, method);
 
   if (message == NULL)
-    g_error ("Out of memory");
+    ERROR ("Out of memory");
 
   if (!dbus_connection_send_with_reply (self->priv->libdbus,
       message, &pc, timeout_ms))
-    g_error ("Out of memory");
+    ERROR ("Out of memory");
   /* pc is unreffed by _tp_dbus_daemon_list_names_notify */
 
   context = list_names_context_new (self, callback, user_data, destroy,
@@ -1134,7 +1183,7 @@ _tp_dbus_daemon_list_names_common (TpDBusDaemon *self,
         _tp_dbus_daemon_list_names_notify, context,
         list_names_context_unref))
     {
-      g_error ("Out of memory");
+      ERROR ("Out of memory");
     }
 }
 
@@ -1237,7 +1286,7 @@ tp_dbus_daemon_constructor (GType type,
 
   /* one ref per TpDBusDaemon, released in finalize */
   if (!dbus_connection_allocate_data_slot (&daemons_slot))
-    g_error ("Out of memory");
+    ERROR ("Out of memory");
 
   daemons = dbus_connection_get_data (self->priv->libdbus, daemons_slot);
 
@@ -1253,7 +1302,7 @@ tp_dbus_daemon_constructor (GType type,
       /* we add this filter at most once per DBusConnection */
       if (!dbus_connection_add_filter (self->priv->libdbus,
             _tp_dbus_daemon_name_owner_changed_filter, NULL, NULL))
-        g_error ("Out of memory");
+        ERROR ("Out of memory");
     }
 
   *daemons = g_slist_prepend (*daemons, self);
@@ -1376,5 +1425,11 @@ tp_dbus_daemon_class_init (TpDBusDaemonClass *klass)
   proxy_class->interface = TP_IFACE_QUARK_DBUS_DAEMON;
 }
 
+gboolean
+_tp_dbus_daemon_is_the_shared_one (TpDBusDaemon *self)
+{
+  return (self != NULL && self == starter_bus_daemon);
+}
+
 /* Auto-generated implementation of _tp_register_dbus_glib_marshallers */
 #include "_gen/register-dbus-glib-marshallers-body.h"
