@@ -64,8 +64,11 @@
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/util.h>
 
+#define DEBUG_FLAG TP_DEBUG_MISC
+#include "debug-internal.h"
+
 /**
- * tp_asv_size:
+ * tp_asv_size: (skip)
  * @asv: a GHashTable
  *
  * Return the size of @asv as if via g_hash_table_size().
@@ -78,7 +81,7 @@
 /* (#define + static inline in dbus.h) */
 
 /**
- * tp_dbus_g_method_return_not_implemented:
+ * tp_dbus_g_method_return_not_implemented: (skip)
  * @context: The D-Bus method invocation context
  *
  * Return the Telepathy error NotImplemented from the method invocation
@@ -106,7 +109,7 @@ _tp_dbus_starter_bus_conn (GError **error)
 }
 
 /**
- * tp_get_bus:
+ * tp_get_bus: (skip)
  *
  * Returns a connection to the D-Bus daemon on which this process was
  * activated if it was launched by D-Bus service activation, or the session
@@ -130,7 +133,7 @@ tp_get_bus (void)
 
   if (bus == NULL)
     {
-      g_warning ("Failed to connect to starter bus: %s", error->message);
+      WARNING ("Failed to connect to starter bus: %s", error->message);
       exit (1);
     }
 
@@ -138,9 +141,10 @@ tp_get_bus (void)
 }
 
 /**
- * tp_get_bus_proxy:
+ * tp_get_bus_proxy: (skip)
  *
- * Return a #DBusGProxy for the bus daemon object.
+ * Return a #DBusGProxy for the bus daemon object. The same caveats as for
+ * tp_get_bus() apply.
  *
  * Returns: a proxy for the bus daemon object on the starter or session bus.
  *
@@ -153,7 +157,14 @@ tp_get_bus_proxy (void)
 
   if (bus_proxy == NULL)
     {
-      DBusGConnection *bus = tp_get_bus ();
+      GError *error = NULL;
+      DBusGConnection *bus = _tp_dbus_starter_bus_conn (&error);
+
+      if (bus == NULL)
+        {
+          WARNING ("Failed to connect to starter bus: %s", error->message);
+          exit (1);
+        }
 
       bus_proxy = dbus_g_proxy_new_for_name (bus,
                                             "org.freedesktop.DBus",
@@ -161,7 +172,7 @@ tp_get_bus_proxy (void)
                                             "org.freedesktop.DBus");
 
       if (bus_proxy == NULL)
-        g_error ("Failed to get proxy object for bus.");
+        ERROR ("Failed to get proxy object for bus.");
     }
 
   return bus_proxy;
@@ -185,9 +196,20 @@ tp_get_bus_proxy (void)
  * types. %TP_DBUS_NAME_TYPE_NOT_BUS_DAEMON and %TP_DBUS_NAME_TYPE_ANY are
  * the bitwise-or of other appropriate types, for convenience.
  *
+ * Since 0.11.5, there is a corresponding #GFlagsClass type,
+ * %TP_TYPE_DBUS_NAME_TYPE.
+ *
  * Since: 0.7.1
  */
 
+/**
+ * TP_TYPE_DBUS_NAME_TYPE:
+ *
+ * The #GFlagsClass type of a #TpDBusNameType or a set of name types.
+ *
+ * Since: 0.11.5
+ */
+
 /**
  * tp_dbus_check_valid_bus_name:
  * @name: a possible bus name
@@ -568,7 +590,7 @@ tp_dbus_check_valid_object_path (const gchar *path, GError **error)
 }
 
 /**
- * tp_g_value_slice_new_bytes:
+ * tp_g_value_slice_new_bytes: (skip)
  * @length: number of bytes to copy
  * @bytes: location of an array of bytes to be copied (this may be %NULL
  *  if and only if length is 0)
@@ -599,7 +621,7 @@ tp_g_value_slice_new_bytes (guint length,
 }
 
 /**
- * tp_g_value_slice_new_take_bytes:
+ * tp_g_value_slice_new_take_bytes: (skip)
  * @bytes: a non-NULL #GArray of guchar, ownership of which will be taken by
  *  the #GValue
  *
@@ -621,7 +643,7 @@ tp_g_value_slice_new_take_bytes (GArray *bytes)
 }
 
 /**
- * tp_g_value_slice_new_object_path:
+ * tp_g_value_slice_new_object_path: (skip)
  * @path: a valid D-Bus object path which will be copied
  *
  * Slice-allocate a #GValue containing an object path, using
@@ -641,7 +663,7 @@ tp_g_value_slice_new_object_path (const gchar *path)
 }
 
 /**
- * tp_g_value_slice_new_static_object_path:
+ * tp_g_value_slice_new_static_object_path: (skip)
  * @path: a valid D-Bus object path which must remain valid forever
  *
  * Slice-allocate a #GValue containing an object path, using
@@ -661,7 +683,7 @@ tp_g_value_slice_new_static_object_path (const gchar *path)
 }
 
 /**
- * tp_g_value_slice_new_take_object_path:
+ * tp_g_value_slice_new_take_object_path: (skip)
  * @path: a valid D-Bus object path which will be freed with g_free() by the
  *  returned #GValue (the caller must own it before calling this function, but
  *  no longer owns it after this function returns)
@@ -683,7 +705,7 @@ tp_g_value_slice_new_take_object_path (gchar *path)
 }
 
 /**
- * tp_asv_new:
+ * tp_asv_new: (skip)
  * @first_key: the name of the first key (or NULL)
  * @...: type and value for the first key, followed by a NULL-terminated list
  *  of (key, type, value) tuples
@@ -748,7 +770,7 @@ tp_asv_new (const gchar *first_key, ...)
 
     if (error != NULL)
     {
-      g_critical ("key %s: %s", key, error);
+      CRITICAL ("key %s: %s", key, error);
       g_free (error);
       error = NULL;
       tp_g_value_slice_free (value);
@@ -765,7 +787,8 @@ tp_asv_new (const gchar *first_key, ...)
 
 /**
  * tp_asv_get_boolean:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location to store %TRUE if the key actually
  *  exists and has a boolean value
@@ -805,7 +828,7 @@ tp_asv_get_boolean (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_boolean:
+ * tp_asv_set_boolean: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -830,7 +853,8 @@ tp_asv_set_boolean (GHashTable *asv,
 
 /**
  * tp_asv_get_bytes:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  *
  * If a value for @key in @asv is present and is an array of bytes
@@ -843,7 +867,7 @@ tp_asv_set_boolean (GHashTable *asv,
  * g_boxed_copy (DBUS_TYPE_G_UCHAR_ARRAY, ...) if you need to keep
  * it for longer.
  *
- * Returns: the string value of @key, or %NULL
+ * Returns: (transfer none) (allow-none): the string value of @key, or %NULL
  * Since: 0.7.9
  */
 const GArray *
@@ -864,7 +888,7 @@ tp_asv_get_bytes (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_bytes:
+ * tp_asv_set_bytes: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @length: the number of bytes to copy
@@ -893,7 +917,7 @@ tp_asv_set_bytes (GHashTable *asv,
 }
 
 /**
- * tp_asv_take_bytes:
+ * tp_asv_take_bytes: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: a non-NULL #GArray of %guchar, ownership of which will be taken by
@@ -921,7 +945,8 @@ tp_asv_take_bytes (GHashTable *asv,
 
 /**
  * tp_asv_get_string:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  *
  * If a value for @key in @asv is present and is a string, return it.
@@ -932,7 +957,7 @@ tp_asv_take_bytes (GHashTable *asv,
  * for @key in @asv is not removed or altered. Copy it with g_strdup() if you
  * need to keep it for longer.
  *
- * Returns: the string value of @key, or %NULL
+ * Returns: (transfer none) (allow-none): the string value of @key, or %NULL
  * Since: 0.7.9
  */
 const gchar *
@@ -953,7 +978,7 @@ tp_asv_get_string (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_string:
+ * tp_asv_set_string: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -977,7 +1002,7 @@ tp_asv_set_string (GHashTable *asv,
 }
 
 /**
- * tp_asv_take_string:
+ * tp_asv_take_string: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1003,7 +1028,7 @@ tp_asv_take_string (GHashTable *asv,
 }
 
 /**
- * tp_asv_set_static_string:
+ * tp_asv_set_static_string: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1030,7 +1055,8 @@ tp_asv_set_static_string (GHashTable *asv,
 
 /**
  * tp_asv_get_int32:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location in which to store %TRUE on success or
  *    %FALSE on failure
@@ -1117,7 +1143,7 @@ return_invalid:
 }
 
 /**
- * tp_asv_set_int32:
+ * tp_asv_set_int32: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1142,7 +1168,8 @@ tp_asv_set_int32 (GHashTable *asv,
 
 /**
  * tp_asv_get_uint32:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location in which to store %TRUE on success or
  *    %FALSE on failure
@@ -1229,7 +1256,7 @@ return_invalid:
 }
 
 /**
- * tp_asv_set_uint32:
+ * tp_asv_set_uint32: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1254,7 +1281,8 @@ tp_asv_set_uint32 (GHashTable *asv,
 
 /**
  * tp_asv_get_int64:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location in which to store %TRUE on success or
  *    %FALSE on failure
@@ -1330,7 +1358,7 @@ return_invalid:
 }
 
 /**
- * tp_asv_set_int64:
+ * tp_asv_set_int64: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1355,7 +1383,8 @@ tp_asv_set_int64 (GHashTable *asv,
 
 /**
  * tp_asv_get_uint64:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location in which to store %TRUE on success or
  *    %FALSE on failure
@@ -1435,7 +1464,7 @@ return_invalid:
 }
 
 /**
- * tp_asv_set_uint64:
+ * tp_asv_set_uint64: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1460,7 +1489,8 @@ tp_asv_set_uint64 (GHashTable *asv,
 
 /**
  * tp_asv_get_double:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @valid: Either %NULL, or a location in which to store %TRUE on success or
  *    %FALSE on failure
@@ -1533,7 +1563,7 @@ return_invalid:
 }
 
 /**
- * tp_asv_set_double:
+ * tp_asv_set_double: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1558,7 +1588,8 @@ tp_asv_set_double (GHashTable *asv,
 
 /**
  * tp_asv_get_object_path:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  *
  * If a value for @key in @asv is present and is an object path, return it.
@@ -1569,7 +1600,8 @@ tp_asv_set_double (GHashTable *asv,
  * for @key in @asv is not removed or altered. Copy it with g_strdup() if you
  * need to keep it for longer.
  *
- * Returns: the object-path value of @key, or %NULL
+ * Returns: (transfer none) (allow-none): the object-path value of @key, or
+ * %NULL
  * Since: 0.7.9
  */
 const gchar *
@@ -1590,7 +1622,7 @@ tp_asv_get_object_path (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_object_path:
+ * tp_asv_set_object_path: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1616,7 +1648,7 @@ tp_asv_set_object_path (GHashTable *asv,
 }
 
 /**
- * tp_asv_take_object_path:
+ * tp_asv_take_object_path: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1642,7 +1674,7 @@ tp_asv_take_object_path (GHashTable *asv,
 }
 
 /**
- * tp_asv_set_static_object_path:
+ * tp_asv_set_static_object_path: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: value
@@ -1669,7 +1701,8 @@ tp_asv_set_static_object_path (GHashTable *asv,
 
 /**
  * tp_asv_get_boxed:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  * @type: The type that the key's value should have, which must be derived
  *  from %G_TYPE_BOXED
@@ -1683,7 +1716,7 @@ tp_asv_set_static_object_path (GHashTable *asv,
  * for @key in @asv is not removed or altered. Copy it, for instance with
  * g_boxed_copy(), if you need to keep it for longer.
  *
- * Returns: the value of @key, or %NULL
+ * Returns: (transfer none) (allow-none): the value of @key, or %NULL
  * Since: 0.7.9
  */
 gpointer
@@ -1706,7 +1739,7 @@ tp_asv_get_boxed (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_boxed:
+ * tp_asv_set_boxed: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @type: the type of the key's value, which must be derived from %G_TYPE_BOXED
@@ -1734,7 +1767,7 @@ tp_asv_set_boxed (GHashTable *asv,
 }
 
 /**
- * tp_asv_take_boxed:
+ * tp_asv_take_boxed: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @type: the type of the key's value, which must be derived from %G_TYPE_BOXED
@@ -1762,7 +1795,7 @@ tp_asv_take_boxed (GHashTable *asv,
 }
 
 /**
- * tp_asv_set_static_boxed:
+ * tp_asv_set_static_boxed: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @type: the type of the key's value, which must be derived from %G_TYPE_BOXED
@@ -1792,7 +1825,8 @@ tp_asv_set_static_boxed (GHashTable *asv,
 
 /**
  * tp_asv_get_strv:
- * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @asv: (element-type utf8 GObject.Value): A GHashTable where the keys are
+ * strings and the values are GValues
  * @key: The key to look up
  *
  * If a value for @key in @asv is present and is an array of strings (strv),
@@ -1804,7 +1838,8 @@ tp_asv_set_static_boxed (GHashTable *asv,
  * for @key in @asv is not removed or altered. Copy it with g_strdupv() if you
  * need to keep it for longer.
  *
- * Returns: the %NULL-terminated string-array value of @key, or %NULL
+ * Returns: (transfer none) (allow-none): the %NULL-terminated string-array
+ * value of @key, or %NULL
  * Since: 0.7.9
  */
 const gchar * const *
@@ -1825,7 +1860,7 @@ tp_asv_get_strv (const GHashTable *asv,
 }
 
 /**
- * tp_asv_set_strv:
+ * tp_asv_set_strv: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  * @key: string key
  * @value: a %NULL-terminated string array
@@ -1850,7 +1885,7 @@ tp_asv_set_strv (GHashTable *asv,
 }
 
 /**
- * tp_asv_lookup:
+ * tp_asv_lookup: (skip)
  * @asv: A GHashTable where the keys are strings and the values are GValues
  * @key: The key to look up
  *
@@ -1874,7 +1909,7 @@ tp_asv_lookup (const GHashTable *asv,
 }
 
 /**
- * tp_asv_dump:
+ * tp_asv_dump: (skip)
  * @asv: a #GHashTable created with tp_asv_new()
  *
  * Dumps the a{sv} map to the debugging console.
