@@ -166,6 +166,12 @@
  *     Raised when a local process rejects a request because it does not have
  *     enough of a resource, such as memory.
  *     Since 0.7.34
+ * @TP_ERROR_WOULD_BREAK_ANONYMITY: org.freedesktop.Telepathy.Error.WouldBreakAnonymity:
+ *     Raised when a request cannot be satisfied without violating an
+ *     earlier request for anonymity, and the earlier request specified
+ *     that raising an error is preferable to disclosing the user's
+ *     identity
+ *     Since 0.11.7
  *
  * Enumerated type representing the Telepathy D-Bus errors.
  */
@@ -229,8 +235,6 @@ tp_errors_quark (void)
     {
       GQuark domain = g_quark_from_static_string ("tp_errors");
 
-      tp_verify_statement (sizeof (GQuark) <= sizeof (gsize));
-
       g_type_init ();
       dbus_g_error_domain_register (domain, TP_ERROR_PREFIX,
           TP_TYPE_ERROR);
@@ -240,60 +244,5 @@ tp_errors_quark (void)
   return (GQuark) quark;
 }
 
-GType
-tp_error_get_type (void)
-{
-  static GType etype = 0;
-  if (G_UNLIKELY (etype == 0))
-    {
-      static const GEnumValue values[] = {
-        { TP_ERROR_NETWORK_ERROR, "TP_ERROR_NETWORK_ERROR", "NetworkError" },
-        { TP_ERROR_NOT_IMPLEMENTED, "TP_ERROR_NOT_IMPLEMENTED", "NotImplemented" },
-        { TP_ERROR_INVALID_ARGUMENT, "TP_ERROR_INVALID_ARGUMENT", "InvalidArgument" },
-        { TP_ERROR_NOT_AVAILABLE, "TP_ERROR_NOT_AVAILABLE", "NotAvailable" },
-        { TP_ERROR_PERMISSION_DENIED, "TP_ERROR_PERMISSION_DENIED", "PermissionDenied" },
-        { TP_ERROR_DISCONNECTED, "TP_ERROR_DISCONNECTED", "Disconnected" },
-        { TP_ERROR_INVALID_HANDLE, "TP_ERROR_INVALID_HANDLE", "InvalidHandle" },
-        { TP_ERROR_CHANNEL_BANNED, "TP_ERROR_CHANNEL_BANNED", "Channel.Banned" },
-        { TP_ERROR_CHANNEL_FULL, "TP_ERROR_CHANNEL_FULL", "Channel.Full" },
-        { TP_ERROR_CHANNEL_INVITE_ONLY, "TP_ERROR_CHANNEL_INVITE_ONLY", "Channel.InviteOnly" },
-        { TP_ERROR_NOT_YOURS, "TP_ERROR_NOT_YOURS", "NotYours" },
-        { TP_ERROR_CANCELLED, "TP_ERROR_CANCELLED", "Cancelled" },
-        { TP_ERROR_AUTHENTICATION_FAILED, "TP_ERROR_AUTHENTICATION_FAILED", "AuthenticationFailed" },
-        { TP_ERROR_ENCRYPTION_NOT_AVAILABLE, "TP_ERROR_ENCRYPTION_NOT_AVAILABLE", "EncryptionNotAvailable" },
-        { TP_ERROR_ENCRYPTION_ERROR, "TP_ERROR_ENCRYPTION_ERROR", "EncryptionError" },
-        { TP_ERROR_CERT_NOT_PROVIDED, "TP_ERROR_CERT_NOT_PROVIDED", "Cert.NotProvided" },
-        { TP_ERROR_CERT_UNTRUSTED, "TP_ERROR_CERT_UNTRUSTED", "Cert.Untrusted" },
-        { TP_ERROR_CERT_EXPIRED, "TP_ERROR_CERT_EXPIRED", "Cert.Expired" },
-        { TP_ERROR_CERT_NOT_ACTIVATED, "TP_ERROR_CERT_NOT_ACTIVATED", "Cert.NotActivated" },
-        { TP_ERROR_CERT_FINGERPRINT_MISMATCH, "TP_ERROR_CERT_FINGERPRINT_MISMATCH", "Cert.FingerprintMismatch" },
-        { TP_ERROR_CERT_HOSTNAME_MISMATCH, "TP_ERROR_CERT_HOSTNAME_MISMATCH", "Cert.HostnameMismatch" },
-        { TP_ERROR_CERT_SELF_SIGNED, "TP_ERROR_CERT_SELF_SIGNED", "Cert.SelfSigned" },
-        { TP_ERROR_CERT_INVALID, "TP_ERROR_CERT_INVALID", "Cert.Invalid" },
-        { TP_ERROR_NOT_CAPABLE, "TP_ERROR_NOT_CAPABLE", "NotCapable" },
-        { TP_ERROR_OFFLINE, "TP_ERROR_OFFLINE", "Offline" },
-        { TP_ERROR_CHANNEL_KICKED, "TP_ERROR_CHANNEL_KICKED", "Channel.Kicked" },
-        { TP_ERROR_BUSY, "TP_ERROR_BUSY", "Busy" },
-        { TP_ERROR_NO_ANSWER, "TP_ERROR_NO_ANSWER", "NoAnswer" },
-        { TP_ERROR_DOES_NOT_EXIST, "TP_ERROR_DOES_NOT_EXIST", "DoesNotExist" },
-        { TP_ERROR_TERMINATED, "TP_ERROR_TERMINATED", "Terminated" },
-        { TP_ERROR_CONNECTION_REFUSED, "TP_ERROR_CONNECTION_REFUSED", "ConnectionRefused" },
-        { TP_ERROR_CONNECTION_FAILED, "TP_ERROR_CONNECTION_FAILED", "ConnectionFailed" },
-        { TP_ERROR_CONNECTION_LOST, "TP_ERROR_CONNECTION_LOST", "ConnectionLost" },
-        /* new in telepathy-spec 0.17.27 / telepathy-glib 0.7.34 */
-        { TP_ERROR_ALREADY_CONNECTED, "TP_ERROR_ALREADY_CONNECTED",
-          "AlreadyConnected" },
-        { TP_ERROR_CONNECTION_REPLACED, "TP_ERROR_CONNECTION_REPLACED",
-          "ConnectionReplaced" },
-        { TP_ERROR_REGISTRATION_EXISTS, "TP_ERROR_REGISTRATION_EXISTS",
-          "RegistrationExists" },
-        { TP_ERROR_SERVICE_BUSY, "TP_ERROR_SERVICE_BUSY", "ServiceBusy" },
-        { TP_ERROR_RESOURCE_UNAVAILABLE, "TP_ERROR_RESOURCE_UNAVAILABLE",
-          "ResourceUnavailable" },
-        { 0 }
-      };
-
-      etype = g_enum_register_static ("TpError", values);
-    }
-  return etype;
-}
+/* tp_errors_quark assumes this */
+G_STATIC_ASSERT (sizeof (GQuark) <= sizeof (gsize));
