@@ -165,6 +165,7 @@ GQuark tp_group_mixin_get_offset_quark (void);
 void tp_group_mixin_class_init (GObjectClass *obj_cls,
     glong offset, TpGroupMixinAddMemberFunc add_func,
     TpGroupMixinRemMemberFunc rem_func);
+void tp_group_mixin_class_allow_self_removal (GObjectClass *obj_cls);
 
 void tp_group_mixin_init (GObject *obj, glong offset,
     TpHandleRepoIface *handle_repo, TpHandle self_handle);
@@ -201,11 +202,12 @@ gboolean tp_group_mixin_get_handle_owners (GObject *obj,
 void tp_group_mixin_change_flags (GObject *obj,
     TpChannelGroupFlags add, TpChannelGroupFlags del);
 gboolean tp_group_mixin_change_members (GObject *obj,
-    const gchar *message, TpIntSet *add, TpIntSet *del,
-    TpIntSet *add_local_pending, TpIntSet *add_remote_pending, TpHandle actor,
-    TpChannelGroupChangeReason reason);
-gboolean tp_group_mixin_change_members_detailed (GObject *obj, TpIntSet *add,
-    TpIntSet *del, TpIntSet *add_local_pending, TpIntSet *add_remote_pending,
+    const gchar *message, const TpIntSet *add, const TpIntSet *del,
+    const TpIntSet *add_local_pending, const TpIntSet *add_remote_pending,
+    TpHandle actor, TpChannelGroupChangeReason reason);
+gboolean tp_group_mixin_change_members_detailed (GObject *obj,
+    const TpIntSet *add, const TpIntSet *del,
+    const TpIntSet *add_local_pending, const TpIntSet *add_remote_pending,
     const GHashTable *details);
 void tp_group_mixin_change_self_handle (GObject *obj,
     TpHandle new_self_handle);
