@@ -48,6 +48,10 @@
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/heap.h>
 #include <telepathy-glib/handle-repo-internal.h>
+#include <telepathy-glib/util.h>
+
+#define DEBUG_FLAG TP_DEBUG_HANDLES
+#include "telepathy-glib/debug-internal.h"
 
 /**
  * TpDynamicHandleRepoNormalizeFunc:
@@ -281,7 +285,7 @@ handles_name_owner_changed_cb (TpDBusDaemon *dbus_daemon,
 {
   TpDynamicHandleRepo *repo = user_data;
 
-  if (new_owner == NULL || new_owner[0] == '\0')
+  if (tp_str_empty (new_owner))
     {
       tp_dbus_daemon_cancel_name_owner_watch (dbus_daemon, name,
           handles_name_owner_changed_cb, repo);
@@ -302,7 +306,7 @@ tp_dynamic_handle_repo_init (TpDynamicHandleRepo *self)
   self->bus_daemon = tp_dbus_daemon_dup (NULL);
 
   if (self->bus_daemon == NULL)
-    g_error ("Unable to connect to starter bus");
+    ERROR ("Unable to connect to starter bus");
 
   return;
 }
@@ -640,7 +644,7 @@ dynamic_client_hold_handle (TpHandleRepoIface *repo,
 
   if (!client_name || *client_name == '\0')
     {
-      g_critical ("%s: called with invalid client name", G_STRFUNC);
+      CRITICAL ("called with invalid client name");
       g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "invalid client name");
       return FALSE;
@@ -682,7 +686,7 @@ dynamic_client_release_handle (TpHandleRepoIface *repo,
 
   if (!client_name || *client_name == '\0')
     {
-      g_critical ("%s: called with invalid client name", G_STRFUNC);
+      CRITICAL ("called with invalid client name");
       g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "invalid client name");
       return FALSE;
