@@ -33,12 +33,9 @@ G_BEGIN_DECLS
 
 /* Forward declaration because it's in the HandleRepo API */
 
-/**
- * TpHandleSet:
- *
- * A set of handles. This is similar to a #TpIntSet (and implemented using
- * one), but adding a handle to the set also references it.
- */
+#define TP_TYPE_HANDLE_SET (tp_handle_set_get_type ())
+GType tp_handle_set_get_type (void);
+
 typedef struct _TpHandleSet TpHandleSet;
 
 /* Handle repository abstract interface */
@@ -111,33 +108,31 @@ gpointer tp_handle_get_qdata (TpHandleRepoIface *repo, TpHandle handle,
 
 /* Handle set helper class */
 
-/**
- * TpHandleSetMemberFunc:
- * @set: The set of handles on which tp_handle_set_foreach() was called
- * @handle: A handle in the set
- * @userdata: Arbitrary user data as supplied to tp_handle_set_foreach()
- *
- * Signature of the callback used to iterate over the handle set in
- * tp_handle_set_foreach().
- */
 typedef void (*TpHandleSetMemberFunc)(TpHandleSet *set, TpHandle handle,
     gpointer userdata);
 
 TpHandleSet * tp_handle_set_new (TpHandleRepoIface *repo)
   G_GNUC_WARN_UNUSED_RESULT;
+TpHandleSet *tp_handle_set_copy (const TpHandleSet *other)
+  G_GNUC_WARN_UNUSED_RESULT;
+void tp_handle_set_clear (TpHandleSet *set);
 void tp_handle_set_destroy (TpHandleSet *set);
 
 TpIntSet *tp_handle_set_peek (TpHandleSet *set) G_GNUC_WARN_UNUSED_RESULT;
 
 void tp_handle_set_add (TpHandleSet *set, TpHandle handle);
 gboolean tp_handle_set_remove (TpHandleSet *set, TpHandle handle);
-gboolean tp_handle_set_is_member (TpHandleSet *set, TpHandle handle);
+gboolean tp_handle_set_is_member (const TpHandleSet *set, TpHandle handle);
 
 void tp_handle_set_foreach (TpHandleSet *set, TpHandleSetMemberFunc func,
     gpointer userdata);
 
-int tp_handle_set_size (TpHandleSet *set);
-GArray *tp_handle_set_to_array (TpHandleSet *set) G_GNUC_WARN_UNUSED_RESULT;
+gboolean tp_handle_set_is_empty (const TpHandleSet *set);
+int tp_handle_set_size (const TpHandleSet *set);
+GArray *tp_handle_set_to_array (const TpHandleSet *set)
+  G_GNUC_WARN_UNUSED_RESULT;
+TpHandleSet *tp_handle_set_new_from_array (TpHandleRepoIface *repo,
+    const GArray *array) G_GNUC_WARN_UNUSED_RESULT;
 
 TpIntSet *tp_handle_set_update (TpHandleSet *set, const TpIntSet *add)
   G_GNUC_WARN_UNUSED_RESULT;
@@ -146,7 +141,7 @@ TpIntSet *tp_handle_set_difference_update (TpHandleSet *set,
 
 /* static inline because it relies on NUM_TP_HANDLE_TYPES */
 /**
- * tp_handles_supported_and_valid:
+ * tp_handles_supported_and_valid: (skip)
  * @repos: An array of possibly null pointers to handle repositories, indexed
  *         by handle type, where a null pointer means an unsupported handle
  *         type
