@@ -29,6 +29,12 @@
 
 #include <telepathy-glib/intset.h>
 
+/**
+ * TpHandleSet:
+ *
+ * A set of handles. This is similar to a #TpIntSet (and implemented using
+ * one), but adding a handle to the set also references it.
+ */
 struct _TpHandleSet
 {
   TpHandleRepoIface *repo;
@@ -36,7 +42,31 @@ struct _TpHandleSet
 };
 
 /**
- * tp_handle_set_new:
+ * TP_TYPE_HANDLE_SET: (skip)
+ *
+ * The boxed type of a #TpHandleSet.
+ *
+ * Since: 0.11.6
+ */
+
+GType
+tp_handle_set_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    {
+      type = g_boxed_type_register_static (
+          g_intern_static_string ("TpHandleSet"),
+          (GBoxedCopyFunc) tp_handle_set_copy,
+          (GBoxedFreeFunc) tp_handle_set_destroy);
+    }
+
+  return type;
+}
+
+/**
+ * tp_handle_set_new: (skip)
  * @repo: #TpHandleRepoIface that holds the handles to be reffed by this set
  *
  * Creates a new #TpHandleSet
@@ -56,6 +86,29 @@ tp_handle_set_new (TpHandleRepoIface *repo)
   return set;
 }
 
+/**
+ * tp_handle_set_new_from_array: (skip)
+ * @repo: #TpHandleRepoIface that holds the handles to be reffed by this set
+ * @array: (element-type uint): array of handles to be referenced by this set
+ *
+ * Creates a new #TpHandleSet
+ *
+ * Returns: A new #TpHandleSet
+ *
+ * Since: 0.11.7
+ */
+TpHandleSet *
+tp_handle_set_new_from_array (TpHandleRepoIface *repo,
+    const GArray *array)
+{
+  TpHandleSet *set = tp_handle_set_new (repo);
+  TpIntSet *tmp = tp_intset_from_array (array);
+
+  tp_intset_destroy (tp_handle_set_update (set, tmp));
+  tp_intset_destroy (tmp);
+  return set;
+}
+
 static void
 freer (TpHandleSet *set, TpHandle handle, gpointer userdata)
 {
@@ -63,7 +116,7 @@ freer (TpHandleSet *set, TpHandle handle, gpointer userdata)
 }
 
 /**
- * tp_handle_set_destroy:
+ * tp_handle_set_destroy: (skip)
  * @set:#TpHandleSet to destroy
  *
  * Delete a #TpHandleSet and unreference any handles that it holds
@@ -77,12 +130,44 @@ tp_handle_set_destroy (TpHandleSet *set)
 }
 
 /**
- * tp_handle_set_peek:
+ * tp_handle_set_clear: (skip)
+ * @set:#TpHandleSet to clear
+ *
+ * Remove every handle from @set, releasing the references it holds.
+ *
+ * Since: 0.11.6
+ */
+void
+tp_handle_set_clear (TpHandleSet *set)
+{
+  tp_handle_set_foreach (set, freer, NULL);
+  g_assert (tp_handle_set_is_empty (set));
+}
+
+/**
+ * tp_handle_set_is_empty: (skip)
+ * @set:#TpHandleSet to check
+ *
+ * Return the same thing as <code>(tp_handle_set_size (set) == 0)</code>,
+ * but calculated more efficiently.
+ *
+ * Returns: %TRUE if the set has no members
+ *
+ * Since: 0.11.6
+ */
+gboolean
+tp_handle_set_is_empty (const TpHandleSet *set)
+{
+  return tp_intset_is_empty (set->intset);
+}
+
+/**
+ * tp_handle_set_peek: (skip)
  * @set:#TpHandleSet to peek at
  *
  * <!--Returns: says it all, this comment is just to keep gtkdoc happy-->
  *
- * Returns: the underlying #TpIntSet used by this #TpHandleSet
+ * Returns: (transfer none): the underlying #TpIntSet used by this #TpHandleSet
  */
 TpIntSet *
 tp_handle_set_peek (TpHandleSet *set)
@@ -91,7 +176,7 @@ tp_handle_set_peek (TpHandleSet *set)
 }
 
 /**
- * tp_handle_set_add:
+ * tp_handle_set_add: (skip)
  * @set: #TpHandleSet to add this handle to
  * @handle: handle to add
  *
@@ -113,7 +198,7 @@ tp_handle_set_add (TpHandleSet *set, TpHandle handle)
 }
 
 /**
- * tp_handle_set_remove:
+ * tp_handle_set_remove: (skip)
  * @set: #TpHandleSet to remove this handle from
  * @handle: handle to remove
  *
@@ -140,7 +225,7 @@ tp_handle_set_remove (TpHandleSet *set, TpHandle handle)
 }
 
 /**
- * tp_handle_set_is_member:
+ * tp_handle_set_is_member: (skip)
  * @set: A #TpHandleSet
  * @handle: handle to check
  *
@@ -150,7 +235,8 @@ tp_handle_set_remove (TpHandleSet *set, TpHandle handle)
  *
  */
 gboolean
-tp_handle_set_is_member (TpHandleSet *set, TpHandle handle)
+tp_handle_set_is_member (const TpHandleSet *set,
+    TpHandle handle)
 {
   return tp_intset_is_member (set->intset, handle);
 }
@@ -171,10 +257,20 @@ foreach_helper (guint i, gpointer userdata)
 }
 
 /**
- * tp_handle_set_foreach:
+ * TpHandleSetMemberFunc: (skip)
+ * @set: The set of handles on which tp_handle_set_foreach() was called
+ * @handle: A handle in the set
+ * @userdata: Arbitrary user data as supplied to tp_handle_set_foreach()
+ *
+ * Signature of the callback used to iterate over the handle set in
+ * tp_handle_set_foreach().
+ */
+
+/**
+ * tp_handle_set_foreach: (skip)
  * @set: A set of handles
- * @func: A callback
- * @userdata: Arbitrary data to pass to @func
+ * @func: (scope call): A callback
+ * @userdata: (closure): Arbitrary data to pass to @func
  *
  * Call @func(@set, @handle, @userdata) for each handle in @set.
  */
@@ -190,7 +286,7 @@ tp_handle_set_foreach (TpHandleSet *set, TpHandleSetMemberFunc func,
 }
 
 /**
- * tp_handle_set_size:
+ * tp_handle_set_size: (skip)
  * @set: A set of handles
  *
  * <!--no further documentation needed-->
@@ -198,22 +294,22 @@ tp_handle_set_foreach (TpHandleSet *set, TpHandleSetMemberFunc func,
  * Returns: the number of handles in this set
  */
 int
-tp_handle_set_size (TpHandleSet *set)
+tp_handle_set_size (const TpHandleSet *set)
 {
   return tp_intset_size (set->intset);
 }
 
 /**
- * tp_handle_set_to_array:
+ * tp_handle_set_to_array: (skip)
  * @set: A handle set
  *
  * <!--Returns: says it all, this comment is just to keep gtkdoc happy-->
  *
- * Returns: a newly-allocated GArray of guint representing the handles
- * in the set
+ * Returns: (element-type uint): a newly-allocated GArray of guint representing
+ * the handles in the set
  */
 GArray *
-tp_handle_set_to_array (TpHandleSet *set)
+tp_handle_set_to_array (const TpHandleSet *set)
 {
   g_return_val_if_fail (set != NULL, NULL);
 
@@ -228,7 +324,31 @@ ref_one (guint handle, gpointer data)
 }
 
 /**
- * tp_handle_set_update:
+ * tp_handle_set_copy: (skip)
+ * @other: another handle set
+ *
+ * Creates a new #TpHandleSet with the same contents as @other.
+ *
+ * Returns: a new set
+ *
+ * Since: 0.11.6
+ */
+TpHandleSet *
+tp_handle_set_copy (const TpHandleSet *other)
+{
+  TpHandleSet *set;
+
+  g_return_val_if_fail (other != NULL, NULL);
+
+  set = g_slice_new0 (TpHandleSet);
+  set->repo = other->repo;
+  set->intset = tp_intset_copy (other->intset);
+  tp_intset_foreach (set->intset, ref_one, set);
+  return set;
+}
+
+/**
+ * tp_handle_set_update: (skip)
  * @set: a #TpHandleSet to update
  * @add: a #TpIntSet of handles to add
  *
@@ -265,7 +385,7 @@ unref_one (guint handle, gpointer data)
 }
 
 /**
- * tp_handle_set_difference_update:
+ * tp_handle_set_difference_update: (skip)
  * @set: a #TpHandleSet to update
  * @remove: a #TpIntSet of handles to remove
  *
