@@ -1,7 +1,7 @@
 /* tp-intset.h - Headers for a Glib-link set of integers
  *
- * Copyright (C) 2005, 2006 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2005, 2006 Nokia Corporation
+ * Copyright © 2005-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright © 2005-2006 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -27,20 +27,11 @@
 
 G_BEGIN_DECLS
 
-/**
- * TpIntSet:
- *
- * Opaque type representing a set of unsigned integers.
- */
+#define TP_TYPE_INTSET (tp_intset_get_type ())
+GType tp_intset_get_type (void);
+
 typedef struct _TpIntSet TpIntSet;
 
-/**
- * TpIntFunc:
- * @i: The relevant integer
- * @userdata: Opaque user data
- *
- * A callback function acting on unsigned integers.
- */
 typedef void (*TpIntFunc) (guint i, gpointer userdata);
 
 TpIntSet *tp_intset_new (void) G_GNUC_WARN_UNUSED_RESULT;
@@ -59,6 +50,7 @@ void tp_intset_foreach (const TpIntSet *set, TpIntFunc func,
 GArray *tp_intset_to_array (const TpIntSet *set) G_GNUC_WARN_UNUSED_RESULT;
 TpIntSet *tp_intset_from_array (const GArray *array) G_GNUC_WARN_UNUSED_RESULT;
 
+gboolean tp_intset_is_empty (const TpIntSet *set) G_GNUC_WARN_UNUSED_RESULT;
 guint tp_intset_size (const TpIntSet *set) G_GNUC_WARN_UNUSED_RESULT;
 
 gboolean tp_intset_is_equal (const TpIntSet *left, const TpIntSet *right)
@@ -78,46 +70,14 @@ gchar *tp_intset_dump (const TpIntSet *set) G_GNUC_WARN_UNUSED_RESULT;
 
 typedef struct _TpIntSetIter TpIntSetIter;
 
-/**
- * TpIntSetIter:
- * @set: The set iterated over.
- * @element: Must be (guint)(-1) before iteration starts. Set to the next
- *  element in the set by tp_intset_iter_next(); undefined after
- *  tp_intset_iter_next() returns %FALSE.
- *
- * A structure representing iteration over a set of integers. Must be
- * initialized with either TP_INTSET_ITER_INIT() or tp_intset_iter_init().
- */
 struct _TpIntSetIter
 {
     const TpIntSet *set;
     guint element;
 };
 
-/**
- * TP_INTSET_ITER_INIT:
- * @set: A set of integers
- *
- * A suitable static initializer for a #TpIntSetIter, to be used as follows:
- *
- * <informalexample><programlisting>
- * void
- * do_something (const TpIntSet *intset)
- * {
- *   TpIntSetIter iter = TP_INTSET_ITER_INIT (intset);
- *   /<!-- -->* ... do something with iter ... *<!-- -->/
- * }
- * </programlisting></informalexample>
- */
 #define TP_INTSET_ITER_INIT(set) { (set), (guint)(-1) }
 
-/**
- * tp_intset_iter_init:
- * @iter: An integer set iterator to be initialized.
- * @set: An integer set to be used by that iterator
- *
- * Reset the iterator @iter to the beginning and make it iterate over @set.
- */
 #define tp_intset_iter_init(iter, set) tp_intset_iter_init_inline (iter, set)
 static inline void
 tp_intset_iter_init_inline (TpIntSetIter *iter, const TpIntSet *set)
@@ -127,13 +87,6 @@ tp_intset_iter_init_inline (TpIntSetIter *iter, const TpIntSet *set)
   iter->element = (guint)(-1);
 }
 
-/**
- * tp_intset_iter_reset:
- * @iter: An integer set iterator to be reset.
- *
- * Reset the iterator @iter to the beginning. It must already be associated
- * with a set.
- */
 #define tp_intset_iter_reset(iter) tp_intset_iter_reset_inline (iter)
 static inline void
 tp_intset_iter_reset_inline (TpIntSetIter *iter)
@@ -145,6 +98,17 @@ tp_intset_iter_reset_inline (TpIntSetIter *iter)
 
 gboolean tp_intset_iter_next (TpIntSetIter *iter);
 
+typedef struct {
+    /*<private>*/
+    gpointer _dummy[16];
+} TpIntSetFastIter;
+
+void tp_intset_fast_iter_init (TpIntSetFastIter *iter,
+    const TpIntSet *set);
+
+gboolean tp_intset_fast_iter_next (TpIntSetFastIter *iter,
+    guint *output);
+
 G_END_DECLS
 
 #endif /*__TP_INTSET_H__*/
