@@ -956,7 +956,7 @@ parts_to_text (const GPtrArray *parts,
         {
           DEBUG ("... wondering whether this is NON_TEXT_CONTENT?");
 
-          if (alternative == NULL || alternative[0] == '\0')
+          if (tp_str_empty (alternative))
             {
               /* This part can't possibly have a text alternative, since it
                * isn't part of a multipart/alternative group
@@ -1565,7 +1565,7 @@ tp_message_mixin_has_pending_messages (GObject *object,
       if (valid)
         *first_sender = h;
       else
-        g_warning ("%s: oldest message's message-sender is mistyped", G_STRFUNC);
+        WARNING ("oldest message's message-sender is mistyped");
     }
 
   return (msg != NULL);
@@ -1923,15 +1923,15 @@ tp_message_mixin_get_dbus_property (GObject *object,
     {
       GPtrArray *arrays = g_ptr_array_sized_new (g_queue_get_length (
             mixin->priv->pending));
-      GList *link;
+      GList *l;
       GType type = dbus_g_type_get_collection ("GPtrArray",
           TP_HASH_TYPE_MESSAGE_PART);
 
-      for (link = g_queue_peek_head_link (mixin->priv->pending);
-           link != NULL;
-           link = g_list_next (link))
+      for (l = g_queue_peek_head_link (mixin->priv->pending);
+           l != NULL;
+           l = g_list_next (l))
         {
-          TpMessage *msg = link->data;
+          TpMessage *msg = l->data;
 
           g_ptr_array_add (arrays, g_boxed_copy (type, msg->parts));
         }
