@@ -105,7 +105,7 @@ struct _TpPresenceStatus {
 };
 
 TpPresenceStatus *tp_presence_status_new (guint which,
-    GHashTable *optional_arguments);
+    GHashTable *optional_arguments) G_GNUC_WARN_UNUSED_RESULT;
 void tp_presence_status_free (TpPresenceStatus *status);
 
 /**
@@ -225,16 +225,14 @@ struct _TpPresenceMixin {
 #define TP_PRESENCE_MIXIN_CLASS_OFFSET_QUARK \
   (tp_presence_mixin_class_get_offset_quark ())
 #define TP_PRESENCE_MIXIN_CLASS_OFFSET(o) \
-  (GPOINTER_TO_UINT (g_type_get_qdata (G_OBJECT_CLASS_TYPE (o), \
-                                       TP_PRESENCE_MIXIN_CLASS_OFFSET_QUARK)))
+  tp_mixin_class_get_offset (o, TP_PRESENCE_MIXIN_CLASS_OFFSET_QUARK)
 #define TP_PRESENCE_MIXIN_CLASS(o) \
   ((TpPresenceMixinClass *) tp_mixin_offset_cast (o, \
     TP_PRESENCE_MIXIN_CLASS_OFFSET (o)))
 
 #define TP_PRESENCE_MIXIN_OFFSET_QUARK (tp_presence_mixin_get_offset_quark ())
 #define TP_PRESENCE_MIXIN_OFFSET(o) \
-  (GPOINTER_TO_UINT (g_type_get_qdata (G_OBJECT_TYPE (o), \
-                                       TP_PRESENCE_MIXIN_OFFSET_QUARK)))
+  tp_mixin_instance_get_offset (o, TP_PRESENCE_MIXIN_OFFSET_QUARK)
 #define TP_PRESENCE_MIXIN(o) \
   ((TpPresenceMixin *) tp_mixin_offset_cast (o, TP_PRESENCE_MIXIN_OFFSET (o)))
 
