@@ -23,7 +23,10 @@
 #define __TP_PROXY_H__
 
 #include <dbus/dbus-glib.h>
-#include <glib-object.h>
+#include <gio/gio.h>
+
+#include <telepathy-glib/defs.h>
+#include <telepathy-glib/_gen/genums.h>
 
 G_BEGIN_DECLS
 
@@ -49,13 +52,13 @@ typedef enum {
     TP_DBUS_ERROR_OBJECT_REMOVED = 8,
     TP_DBUS_ERROR_CANCELLED = 9,
     TP_DBUS_ERROR_INCONSISTENT = 10,
-    NUM_TP_DBUS_ERRORS
+#define NUM_TP_DBUS_ERRORS (TP_DBUS_ERROR_INCONSISTENT + 1)
 } TpDBusError;
 
 struct _TpProxy {
+    /*<private>*/
     GObject parent;
 
-    /*<private>*/
     TpDBusDaemon *dbus_daemon;
     DBusGConnection *dbus_connection;
     gchar *bus_name;
@@ -68,6 +71,15 @@ struct _TpProxy {
 
 typedef struct _TpProxyClass TpProxyClass;
 
+/* defined in proxy-internal.h for now */
+typedef struct _TpProxyFeature TpProxyFeature;
+
+/* XXX: hide this from the g-i scanner, since vapigen can't cope */
+#ifndef __GI_SCANNER__
+typedef const TpProxyFeature *(*TpProxyClassFeatureListFunc) (
+    TpProxyClass *cls);
+#endif /* __GI_SCANNER__ */
+
 struct _TpProxyClass {
     /*<public>*/
     GObjectClass parent_class;
@@ -75,9 +87,16 @@ struct _TpProxyClass {
     GQuark interface;
 
     unsigned int must_have_unique_name:1;
+    /*<private>*/
     guint _reserved_flags:31;
 
-    GCallback _reserved[4];
+/* XXX: hide this from the g-i scanner, since vapigen can't cope */
+#ifdef __GI_SCANNER__
+    GCallback _internal_list_features;
+#else
+    TpProxyClassFeatureListFunc list_features;
+#endif /* __GI_SCANNER__ */
+    GCallback _reserved[3];
     gpointer priv;
 };
 
@@ -109,17 +128,7 @@ GType tp_proxy_get_type (void);
                               TpProxyClass))
 
 gboolean tp_proxy_has_interface_by_id (gpointer self, GQuark iface);
-
-static inline gboolean
-_tp_proxy_inline_has_interface (gpointer self, const gchar *iface)
-{
-  GQuark q = g_quark_try_string (iface);
-
-  return q != 0 && tp_proxy_has_interface_by_id (self, q);
-}
-
-#define tp_proxy_has_interface(self, iface) \
-    (_tp_proxy_inline_has_interface (self, iface))
+gboolean tp_proxy_has_interface (gpointer self, const gchar *iface);
 
 TpDBusDaemon *tp_proxy_get_dbus_daemon (gpointer self);
 
@@ -134,6 +143,15 @@ const GError *tp_proxy_get_invalidated (gpointer self);
 void tp_proxy_dbus_error_to_gerror (gpointer self,
     const char *dbus_error, const char *debug_message, GError **error);
 
+gboolean tp_proxy_is_prepared (gpointer self, GQuark feature);
+void tp_proxy_prepare_async (gpointer self,
+    const GQuark *features,
+    GAsyncReadyCallback callback,
+    gpointer user_data);
+gboolean tp_proxy_prepare_finish (gpointer self,
+    GAsyncResult *result,
+    GError **error);
+
 G_END_DECLS
 
 #include <telepathy-glib/_gen/tp-cli-generic.h>
