@@ -1,8 +1,7 @@
 /*
  * util.c - Source for telepathy-glib utility functions
- * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2006-2007 Nokia Corporation
- *   @author Robert McQueen <robert.mcqueen@collabora.co.uk>
+ * Copyright © 2006-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright © 2006-2008 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -35,6 +34,9 @@
 
 #include <string.h>
 
+#define DEBUG_FLAG TP_DEBUG_MISC
+#include "debug-internal.h"
+
 /**
  * tp_verify:
  * @R: a requirement (constant expression) to be checked at compile-time
@@ -49,6 +51,9 @@
  * (This is gnulib's verify macro, written by Paul Eggert, Bruno Haible and
  * Jim Meyering.)
  *
+ * This macro will be deprecated in a future telepathy-glib release. Please
+ * use GLib 2.20's G_STATIC_ASSERT() macro in new code.
+ *
  * Since: 0.7.34
  */
 
@@ -67,6 +72,9 @@
  * (This is gnulib's verify_true macro, written by Paul Eggert, Bruno Haible
  * and Jim Meyering.)
  *
+ * This macro will be deprecated in a future telepathy-glib release. Please
+ * use GLib 2.20's G_STATIC_ASSERT() macro in new code.
+ *
  * Returns: 1
  *
  * Since: 0.7.34
@@ -83,6 +91,9 @@
  * This macro can be used anywhere that a statement would be allowed; it
  * is equivalent to ((void) tp_verify_true (R)).
  *
+ * This macro will be deprecated in a future telepathy-glib release. Please
+ * use GLib 2.20's G_STATIC_ASSERT() macro in new code.
+ *
  * Since: 0.7.34
  */
 
@@ -194,6 +205,27 @@ tp_g_value_slice_new_int64 (gint64 n)
   return v;
 }
 
+/**
+ * tp_g_value_slice_new_byte:
+ * @n: an unsigned integer
+ *
+ * Slice-allocate and initialize a #GValue. This function is convenient to
+ * use when constructing hash tables from string to #GValue, for example.
+ *
+ * Returns: a #GValue of type %G_TYPE_UCHAR with value @n, to be freed with
+ * tp_g_value_slice_free() or g_slice_free()
+ *
+ * Since: 0.11.0
+ */
+GValue *
+tp_g_value_slice_new_byte (guchar n)
+{
+  GValue *v = tp_g_value_slice_new (G_TYPE_UCHAR);
+
+  g_value_set_uchar (v, n);
+  return v;
+}
+
 /**
  * tp_g_value_slice_new_uint:
  * @n: an unsigned integer
@@ -488,6 +520,17 @@ tp_g_hash_table_update (GHashTable *target,
   g_hash_table_foreach (source, _tp_g_hash_table_update_helper, &data);
 }
 
+/**
+ * tp_str_empty:
+ * @s: (type utf8) (transfer none): a string
+ *
+ * Return %TRUE if @s is empty, counting %NULL as empty.
+ *
+ * Returns: (type boolean): %TRUE if @s is either %NULL or ""
+ *
+ * Since: 0.11.1
+ */
+/* no definition here - it's inlined */
 
 /**
  * tp_strdiff:
@@ -1029,7 +1072,7 @@ tp_value_array_build (gsize length,
 
       if (error != NULL)
         {
-          g_critical ("%s", error);
+          CRITICAL ("%s", error);
           g_free (error);
 
           g_value_array_free (arr);
@@ -1043,3 +1086,282 @@ tp_value_array_build (gsize length,
   va_end (var_args);
   return arr;
 }
+
+/**
+ * tp_value_array_unpack:
+ * @array: the array to unpack
+ * @len: The number of elements that should be in the array
+ * @...: a list of correctly typed pointers to store the values in
+ *
+ * Unpacks a #GValueArray into separate variables.
+ *
+ * The contents of the values aren't copied into the variables, and so become
+ * invalid when @array is freed.
+ *
+ * <example>
+ *   <title>using tp_value_array_unpack</title>
+ *    <programlisting>
+ * const gchar *host;
+ * guint port;
+ *
+ * tp_value_array_unpack (array, 2,
+ *    &host,
+ *    &port);
+ *    </programlisting>
+ * </example>
+ *
+ * Since: 0.11.0
+ */
+void
+tp_value_array_unpack (GValueArray *array,
+    gsize len,
+    ...)
+{
+  va_list var_args;
+  guint i;
+
+  va_start (var_args, len);
+
+  for (i = 0; i < len; i++)
+    {
+      GValue *value;
+      char *error = NULL;
+
+      if (G_UNLIKELY (i > array->n_values))
+        {
+          WARNING ("More parameters than entries in the struct!");
+          break;
+        }
+
+      value = g_value_array_get_nth (array, i);
+
+      G_VALUE_LCOPY (value, var_args, G_VALUE_NOCOPY_CONTENTS, &error);
+      if (error != NULL)
+        {
+          WARNING ("%s", error);
+          g_free (error);
+          break;
+        }
+    }
+
+  va_end (var_args);
+}
+
+/**
+ * TpWeakRef:
+ *
+ * A simple wrapper for a weak reference to a #GObject, suitable for use in
+ * asynchronous calls which should only affect the object if it hasn't already
+ * been freed.
+ *
+ * As well as wrapping a weak reference to an object, this structure can
+ * contain an extra pointer to arbitrary data. This is useful for asynchronous
+ * calls which act on an object and some second piece of data, which are quite
+ * common in practice.
+ *
+ * If more than one piece of auxiliary data is required, the @user_data
+ * argument to the constructor can be a struct or a #GValueArray.
+ *
+ * Since: 0.11.3
+ */
+struct _TpWeakRef {
+    /*<private>*/
+    gpointer object;
+    gpointer user_data;
+    GDestroyNotify destroy;
+};
+
+/**
+ * tp_weak_ref_new:
+ * @object: (type GObject.Object): an object to which to take a weak reference
+ * @user_data: optional additional data to store alongside the weak ref
+ * @destroy: destructor for @user_data, called when the weak ref
+ *  is freed
+ *
+ * Return a new weak reference wrapper for @object.
+ *
+ * Returns: a new weak-reference wrapper
+ *
+ * Free-function: tp_weak_ref_destroy()
+ *
+ * Since: 0.11.3
+ */
+TpWeakRef *
+tp_weak_ref_new (gpointer object,
+    gpointer user_data,
+    GDestroyNotify destroy)
+{
+  TpWeakRef *self;
+
+  g_return_val_if_fail (G_IS_OBJECT (object), NULL);
+
+  self = g_slice_new (TpWeakRef);
+  self->object = object;
+  g_object_add_weak_pointer (self->object, &self->object);
+  self->user_data = user_data;
+  self->destroy = destroy;
+  return self;
+}
+
+/**
+ * tp_weak_ref_get_user_data:
+ * @self: a weak reference
+ *
+ * Return the additional data that was passed to tp_weak_ref_new().
+ *
+ * Returns: the additional data supplied in tp_weak_ref_new(), which may be
+ *  %NULL
+ *
+ * Since: 0.11.3
+ */
+gpointer
+tp_weak_ref_get_user_data (TpWeakRef *self)
+{
+  return self->user_data;
+}
+
+/**
+ * tp_weak_ref_dup_object:
+ * @self: a weak reference
+ *
+ * If the weakly referenced object still exists, return a new reference to
+ * it. Otherwise, return %NULL.
+ *
+ * Returns: (type GObject.Object) (transfer full): a new reference, or %NULL
+ *
+ * Since: 0.11.3
+ */
+gpointer
+tp_weak_ref_dup_object (TpWeakRef *self)
+{
+  if (self->object != NULL)
+    return g_object_ref (self->object);
+
+  return NULL;
+}
+
+/**
+ * tp_weak_ref_destroy:
+ * @self: a weak reference
+ *
+ * Free a weak reference wrapper. This drops the weak reference to the
+ * object (if it still exists), and frees the user data with the user-supplied
+ * destructor function if one was provided.
+ *
+ * Since: 0.11.3
+ */
+void
+tp_weak_ref_destroy (TpWeakRef *self)
+{
+  if (self->object != NULL)
+    g_object_remove_weak_pointer (self->object, &self->object);
+
+  if (self->destroy != NULL)
+    (self->destroy) (self->user_data);
+
+  g_slice_free (TpWeakRef, self);
+}
+
+/**
+ * tp_clear_object: (skip)
+ * @op: (allow-none): a pointer to a variable, struct member etc. holding a
+ *  #GObject
+ *
+ * Set a variable holding a #GObject to %NULL. If it was not already %NULL,
+ * unref the object it previously pointed to.
+ *
+ * This is exactly equivalent to calling tp_clear_pointer() on @op,
+ * with @destroy = g_object_unref(). See tp_clear_pointer() for example usage.
+ *
+ * Since: 0.11.7
+ */
+
+/**
+ * tp_clear_pointer: (skip)
+ * @pp: (allow-none): a pointer to a variable, struct member etc. holding a
+ *  pointer
+ * @destroy: a function to which a gpointer can be passed, to destroy *@pp
+ *  (if calling this macro from C++, explicitly casting the function to
+ *  #GDestroyNotify may be necessary)
+ *
+ * Set a variable holding a pointer to %NULL. If it was not already %NULL,
+ * unref or destroy the object it previously pointed to with @destroy.
+ *
+ * More precisely, if both @pp and *@pp are non-%NULL, set *@pp to %NULL, then
+ * call @destroy on the object that *@pp previously pointed to.
+ *
+ * This is analogous to g_clear_error() for non-error objects, but also
+ * ensures that @pp is already %NULL before the destructor is run.
+ *
+ * Typical usage is something like this:
+ *
+ * |[
+ * typedef struct {
+ *   TpConnection *conn;
+ *   GError *error;
+ *   GHashTable *table;
+ *   MyStruct *misc;
+ * } Foo;
+ * Foo *foo;
+ *
+ * ...
+ *
+ * tp_clear_object (&amp;foo->conn);
+ * g_clear_error (&amp;foo->error);
+ * tp_clear_boxed (G_TYPE_HASH_TABLE, &amp;foo->table);
+ * tp_clear_pointer (&amp;foo->misc, my_struct_destroy);
+ * ]|
+ *
+ * Since: 0.11.7
+ */
+
+/**
+ * tp_clear_boxed: (skip)
+ * @gtype: (type GObject.Type): the #GType of *@pp, e.g. %G_TYPE_HASH_TABLE
+ * @pp: (allow-none): a pointer to a variable, struct member etc. holding a
+ *  boxed object
+ *
+ * Set a variable holding a boxed object to %NULL. If it was not already %NULL,
+ * destroy the boxed object it previously pointed to, as appropriate for
+ * @gtype.
+ *
+ * More precisely, if both @pp and *@pp are non-%NULL, set *@pp to %NULL, then
+ * call g_boxed_free() on the object that *@pp previously pointed to.
+ *
+ * This is similar to tp_clear_pointer(); see that function's documentation
+ * for typical usage.
+ *
+ * Since: 0.11.7
+ */
+
+/**
+ * tp_simple_async_report_success_in_idle:
+ * @source: the source object
+ * @callback: the callback
+ * @user_data: user data for @callback
+ * @source_tag: the source tag for the #GSimpleAsyncResult
+ *
+ * Create a new #GSimpleAsyncResult with no operation result, and call
+ * g_simple_async_result_complete_in_idle() on it.
+ *
+ * This is like a successful version of g_simple_async_report_error_in_idle(),
+ * suitable for asynchronous functions that (conceptually) either succeed and
+ * return nothing, or raise an error, such as tp_proxy_prepare_async().
+ *
+ * The corresponding finish function should not call a function that attempts
+ * to get a result, such as g_simple_async_result_get_op_res_gpointer().
+ *
+ * Since: 0.11.9
+ */
+void
+tp_simple_async_report_success_in_idle (GObject *source,
+    GAsyncReadyCallback callback,
+    gpointer user_data,
+    gpointer source_tag)
+{
+  GSimpleAsyncResult *simple;
+
+  simple = g_simple_async_result_new (source, callback, user_data, source_tag);
+  g_simple_async_result_complete_in_idle (simple);
+  g_object_unref (simple);
+}
