@@ -1,9 +1,8 @@
 /*
  * util.h - Headers for telepathy-glib utility functions
  *
- * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2006-2007 Nokia Corporation
- *   @author Robert McQueen <robert.mcqueen@collabora.co.uk>
+ * Copyright © 2006-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright © 2006-2008 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,7 +23,7 @@
 #define __TP_UTIL_H__
 #define __TP_IN_UTIL_H__
 
-#include <glib-object.h>
+#include <gio/gio.h>
 
 #include <telepathy-glib/verify.h>
 
@@ -39,6 +38,7 @@ GValue *tp_g_value_slice_new (GType type) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_boolean (gboolean b) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_int (gint n) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_int64 (gint64 n) G_GNUC_WARN_UNUSED_RESULT;
+GValue *tp_g_value_slice_new_byte (guchar n) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_uint (guint n) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_uint64 (guint64 n) G_GNUC_WARN_UNUSED_RESULT;
 GValue *tp_g_value_slice_new_double (double d) G_GNUC_WARN_UNUSED_RESULT;
@@ -64,6 +64,14 @@ GValue *tp_g_value_slice_dup (const GValue *value) G_GNUC_WARN_UNUSED_RESULT;
 void tp_g_hash_table_update (GHashTable *target, GHashTable *source,
     GBoxedCopyFunc key_dup, GBoxedCopyFunc value_dup);
 
+#define tp_str_empty(s) (tp_str_empty_inline (s))
+
+static inline gboolean
+tp_str_empty_inline (const gchar *s)
+{
+  return (s == NULL || s[0] == '\0');
+}
+
 gboolean tp_strdiff (const gchar *left, const gchar *right);
 
 gpointer tp_mixin_offset_cast (gpointer instance, guint offset);
@@ -86,6 +94,52 @@ gulong tp_g_signal_connect_object (gpointer instance,
 GValueArray *tp_value_array_build (gsize length,
   GType type,
   ...) G_GNUC_WARN_UNUSED_RESULT;
+void tp_value_array_unpack (GValueArray *array,
+    gsize len,
+    ...);
+
+typedef struct _TpWeakRef TpWeakRef;
+TpWeakRef *tp_weak_ref_new (gpointer object,
+    gpointer user_data,
+    GDestroyNotify destroy) G_GNUC_WARN_UNUSED_RESULT;
+gpointer tp_weak_ref_get_user_data (TpWeakRef *self) G_GNUC_WARN_UNUSED_RESULT;
+gpointer tp_weak_ref_dup_object (TpWeakRef *self) G_GNUC_WARN_UNUSED_RESULT;
+void tp_weak_ref_destroy (TpWeakRef *self);
+
+#define tp_clear_pointer(pp, destroy) \
+  G_STMT_START \
+    { \
+      if (pp != NULL) \
+        { \
+          gpointer _tp_clear_pointer_tmp = *(pp); \
+          \
+          *(pp) = NULL; \
+          \
+          if (_tp_clear_pointer_tmp != NULL) \
+            (destroy) (_tp_clear_pointer_tmp); \
+        } \
+    } \
+  G_STMT_END
+
+#define tp_clear_object(op) tp_clear_pointer ((op), g_object_unref)
+
+#define tp_clear_boxed(gtype, pp) \
+  G_STMT_START \
+    { \
+      if (pp != NULL) \
+        { \
+          gpointer _tp_clear_boxed_tmp = *(pp); \
+          \
+          *(pp) = NULL; \
+          \
+          if (_tp_clear_boxed_tmp != NULL) \
+            g_boxed_free (gtype, _tp_clear_boxed_tmp); \
+        } \
+    } \
+  G_STMT_END
+
+void tp_simple_async_report_success_in_idle (GObject *source,
+    GAsyncReadyCallback callback, gpointer user_data, gpointer source_tag);
 
 G_END_DECLS
 
