@@ -87,7 +87,7 @@ typedef struct {
     GHashTable *times_notified;
     GError *error /* initialized where needed */;
 
-    SimpleAccount *account_service /* initialized in prepare_service */;
+    TpTestsSimpleAccount *account_service /* initialized in prepare_service */;
 } Test;
 
 static void
@@ -95,7 +95,7 @@ setup (Test *test,
        gconstpointer data)
 {
   test->mainloop = g_main_loop_new (NULL, FALSE);
-  test->dbus = tp_dbus_daemon_dup (NULL);
+  test->dbus = tp_tests_dbus_daemon_dup_or_die ();
   g_assert (test->dbus != NULL);
 
   test->account = NULL;
@@ -122,11 +122,9 @@ setup_service (Test *test,
       CONN2_BUS_NAME, FALSE, &test->error);
   g_assert_no_error (test->error);
 
-  test->account_service = g_object_new (SIMPLE_TYPE_ACCOUNT, NULL);
-
-  dbus_g_connection_register_g_object (
-      tp_proxy_get_dbus_connection (test->dbus), ACCOUNT_PATH,
-      G_OBJECT (test->account_service));
+  test->account_service = g_object_new (TP_TESTS_TYPE_SIMPLE_ACCOUNT, NULL);
+  tp_dbus_daemon_register_object (test->dbus, ACCOUNT_PATH,
+      test->account_service);
 }
 
 static guint
@@ -170,7 +168,7 @@ teardown (Test *test,
 {
   if (test->account != NULL)
     {
-      test_proxy_run_until_dbus_queue_processed (test->account);
+      tp_tests_proxy_run_until_dbus_queue_processed (test->account);
 
       if (test->notify_id != 0)
         {
@@ -184,6 +182,9 @@ teardown (Test *test,
   g_hash_table_destroy (test->times_notified);
   test->times_notified = NULL;
 
+  /* make sure any pending calls on the account have happened, so it can die */
+  tp_tests_proxy_run_until_dbus_queue_processed (test->dbus);
+
   g_object_unref (test->dbus);
   test->dbus = NULL;
   g_main_loop_unref (test->mainloop);
@@ -204,9 +205,7 @@ teardown_service (Test *test,
       &test->error);
   g_assert_no_error (test->error);
 
-  dbus_g_connection_unregister_g_object (
-      tp_proxy_get_dbus_connection (test->dbus),
-      G_OBJECT (test->account_service));
+  tp_dbus_daemon_unregister_object (test->dbus, test->account_service);
 
   g_object_unref (test->account_service);
   test->account_service = NULL;
@@ -239,7 +238,7 @@ account_prepare_cb (GObject *source,
   Test *test = user_data;
   GError *error = NULL;
 
-  tp_account_prepare_finish (TP_ACCOUNT (source), result, &error);
+  tp_proxy_prepare_finish (source, result, &error);
   g_assert_no_error (error);
 
   g_main_loop_quit (test->mainloop);
@@ -253,11 +252,12 @@ test_prepare_success (Test *test,
   TpConnectionStatusReason reason;
   gchar *status = NULL;
   gchar *message = NULL;
+  const GHashTable *details = GUINT_TO_POINTER (666);
 
   test->account = tp_account_new (test->dbus, ACCOUNT_PATH, NULL);
   g_assert (test->account != NULL);
 
-  tp_account_prepare_async (test->account, account_features,
+  tp_proxy_prepare_async (test->account, account_features,
       account_prepare_cb, test);
   g_main_loop_run (test->mainloop);
 
@@ -277,6 +277,12 @@ test_prepare_success (Test *test,
   g_assert_cmpint (tp_account_get_connection_status (test->account, &reason),
       ==, TP_CONNECTION_STATUS_CONNECTED);
   g_assert_cmpint (reason, ==, TP_CONNECTION_STATUS_REASON_REQUESTED);
+  g_assert_cmpstr (tp_account_get_detailed_error (test->account, NULL), ==,
+      NULL);
+  g_assert_cmpstr (tp_account_get_detailed_error (test->account, &details), ==,
+      NULL);
+  /* this is documented to be untouched */
+  g_assert_cmpuint (GPOINTER_TO_UINT (details), ==, 666);
 
   /* the CM and protocol come from the object path */
   g_assert_cmpstr (tp_account_get_connection_manager (test->account),
@@ -320,11 +326,12 @@ test_connection (Test *test,
   GQuark account_features[] = { TP_ACCOUNT_FEATURE_CORE, 0 };
   GHashTable *change = tp_asv_new (NULL, NULL);
   TpConnection *conn;
+  const GHashTable *details;
 
   test->account = tp_account_new (test->dbus, ACCOUNT_PATH, NULL);
   g_assert (test->account != NULL);
 
-  tp_account_prepare_async (test->account, account_features,
+  tp_proxy_prepare_async (test->account, account_features,
       account_prepare_cb, test);
   g_main_loop_run (test->mainloop);
 
@@ -349,6 +356,9 @@ test_connection (Test *test,
   g_assert_cmpstr (tp_proxy_get_object_path (conn), ==, CONN1_PATH);
   g_assert_cmpuint (test_get_times_notified (test, "connection"), ==, 1);
 
+  g_assert_cmpstr (tp_account_get_detailed_error (test->account, NULL), ==,
+      TP_ERROR_STR_CANCELLED);
+
   /* ensure the same connection - no change notification */
 
   test_set_up_account_notify (test);
@@ -367,7 +377,7 @@ test_connection (Test *test,
   tp_svc_account_emit_account_property_changed (test->account_service, change);
   g_hash_table_remove_all (change);
 
-  test_proxy_run_until_dbus_queue_processed (test->account);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->account);
 
   g_assert_cmpuint (test_get_times_notified (test, "connection"), ==, 0);
   conn = tp_account_get_connection (test->account);
@@ -411,6 +421,54 @@ test_connection (Test *test,
   conn = tp_account_get_connection (test->account);
   g_assert (conn == NULL);
 
+  g_assert_cmpstr (tp_account_get_detailed_error (test->account, NULL), ==,
+      TP_ERROR_STR_ENCRYPTION_ERROR);
+
+  /* another connection */
+
+  test_set_up_account_notify (test);
+  tp_asv_set_object_path (change, "Connection", CONN1_PATH);
+  tp_asv_set_uint32 (change, "ConnectionStatus",
+      TP_CONNECTION_STATUS_CONNECTING);
+  tp_asv_set_uint32 (change, "ConnectionStatusReason",
+      TP_CONNECTION_STATUS_REASON_REQUESTED);
+  tp_svc_account_emit_account_property_changed (test->account_service, change);
+  g_hash_table_remove_all (change);
+
+  tp_tests_proxy_run_until_dbus_queue_processed (test->account);
+  g_assert_cmpuint (test_get_times_notified (test, "connection"), ==, 1);
+
+  /* lose the connection again */
+
+  test_set_up_account_notify (test);
+  tp_asv_set_object_path (change, "Connection", "/");
+  tp_asv_set_uint32 (change, "ConnectionStatus",
+      TP_CONNECTION_STATUS_DISCONNECTED);
+  tp_asv_set_uint32 (change, "ConnectionStatusReason",
+      TP_CONNECTION_STATUS_REASON_ENCRYPTION_ERROR);
+  tp_asv_set_static_string (change, "ConnectionError",
+      "org.debian.packages.OpenSSL.NotRandomEnough");
+  tp_asv_take_boxed (change, "ConnectionErrorDetails",
+      TP_HASH_TYPE_STRING_VARIANT_MAP,
+      tp_asv_new (
+        "bits-of-entropy", G_TYPE_UINT, 15,
+        "debug-message", G_TYPE_STRING, "shiiiiii-",
+        NULL));
+  tp_svc_account_emit_account_property_changed (test->account_service, change);
+  g_hash_table_remove_all (change);
+
+  tp_tests_proxy_run_until_dbus_queue_processed (test->account);
+  g_assert_cmpuint (test_get_times_notified (test, "connection"), ==, 1);
+  g_assert_cmpuint (test_get_times_notified (test, "connection-error"), ==, 1);
+
+  g_assert_cmpstr (tp_account_get_detailed_error (test->account, &details), ==,
+      "org.debian.packages.OpenSSL.NotRandomEnough");
+  g_assert_cmpuint (tp_asv_size (details), >=, 2);
+  g_assert_cmpstr (tp_asv_get_string (details, "debug-message"), ==,
+      "shiiiiii-");
+  g_assert_cmpuint (tp_asv_get_uint32 (details, "bits-of-entropy", NULL), ==,
+      15);
+
   /* staple on a Connection (this is intended for use in e.g. observers,
    * if they're told about a Connection that the Account hasn't told them
    * about yet) */
