@@ -32,24 +32,6 @@
 
 #include "tests/lib/util.h"
 
-#define CLEAR_OBJECT(o) \
-  G_STMT_START { \
-      if (*(o) != NULL) \
-        { \
-          g_object_unref (*(o)); \
-          *(o) = NULL; \
-        } \
-  } G_STMT_END
-
-#define CLEAR_BOXED(g, o) \
-  G_STMT_START { \
-      if (*(o) != NULL) \
-        { \
-          g_boxed_free ((g), *(o)); \
-          *(o) = NULL; \
-        } \
-  } G_STMT_END
-
 static void
 test_assert_uu_hash_contains (GHashTable *hash,
                               guint key,
@@ -172,13 +154,13 @@ static void test_dump_stream_events (Test *test) G_GNUC_UNUSED;
 
 static void test_dump_stream_events (Test *test)
 {
-  GSList *link;
+  GSList *l;
 
   g_message ("Stream events (most recent first):");
 
-  for (link = test->stream_events; link != NULL; link = link->next)
+  for (l = test->stream_events; l != NULL; l = l->next)
     {
-      StreamEvent *se = link->data;
+      StreamEvent *se = l->data;
 
       switch (se->type)
         {
@@ -214,29 +196,6 @@ static void test_dump_stream_events (Test *test)
     }
 }
 
-static void
-cm_ready_cb (TpConnectionManager *cm G_GNUC_UNUSED,
-             const GError *error,
-             gpointer user_data,
-             GObject *weak_object G_GNUC_UNUSED)
-{
-  Test *test = user_data;
-
-  test_assert_no_error (error);
-  g_main_loop_quit (test->mainloop);
-}
-
-static void
-conn_ready_cb (TpConnection *conn G_GNUC_UNUSED,
-               const GError *error,
-               gpointer user_data)
-{
-  Test *test = user_data;
-
-  test_assert_no_error (error);
-  g_main_loop_quit (test->mainloop);
-}
-
 static void
 setup (Test *test,
        gconstpointer data G_GNUC_UNUSED)
@@ -249,6 +208,7 @@ setup (Test *test,
   guint audio = TP_MEDIA_STREAM_TYPE_AUDIO;
   guint video = TP_MEDIA_STREAM_TYPE_VIDEO;
   guint not_a_media_type = 31337;
+  GQuark conn_features[] = { TP_CONNECTION_FEATURE_CONNECTED, 0 };
 
   g_type_init ();
   tp_debug_set_flags ("all");
@@ -261,10 +221,10 @@ setup (Test *test,
   test->stream_states = g_hash_table_new (NULL, NULL);
 
   test->mainloop = g_main_loop_new (NULL, FALSE);
-  test->dbus = tp_dbus_daemon_dup (NULL);
-  g_assert (test->dbus != NULL);
+  test->dbus = tp_tests_dbus_daemon_dup_or_die ();
 
-  test->service_cm = EXAMPLE_CALLABLE_CONNECTION_MANAGER (g_object_new (
+  test->service_cm = EXAMPLE_CALLABLE_CONNECTION_MANAGER (
+      tp_tests_object_new_static_class (
         EXAMPLE_TYPE_CALLABLE_CONNECTION_MANAGER,
         NULL));
   g_assert (test->service_cm != NULL);
@@ -277,9 +237,7 @@ setup (Test *test,
   test->cm = tp_connection_manager_new (test->dbus, "example_callable",
       NULL, &test->error);
   g_assert (test->cm != NULL);
-  tp_connection_manager_call_when_ready (test->cm, cm_ready_cb, test, NULL,
-      NULL);
-  g_main_loop_run (test->mainloop);
+  tp_tests_proxy_run_until_prepared (test->cm, NULL);
 
   parameters = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
       (GDestroyNotify) tp_g_value_slice_free);
@@ -290,15 +248,14 @@ setup (Test *test,
 
   tp_cli_connection_manager_run_request_connection (test->cm, -1,
       "example", parameters, &bus_name, &object_path, &test->error, NULL);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   test->conn = tp_connection_new (test->dbus, bus_name, object_path,
       &test->error);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
   g_assert (test->conn != NULL);
   tp_cli_connection_call_connect (test->conn, -1, NULL, NULL, NULL, NULL);
-  tp_connection_call_when_ready (test->conn, conn_ready_cb, test);
-  g_main_loop_run (test->mainloop);
+  tp_tests_proxy_run_until_prepared (test->conn, conn_features);
 
   test->self_handle = tp_connection_get_self_handle (test->conn);
   g_assert (test->self_handle != 0);
@@ -331,11 +288,11 @@ channel_created_cb (TpConnection *connection,
   Test *test = user_data;
   GError *new_error = NULL;
 
-  test_assert_no_error (error);
+  g_assert_no_error ((GError *) error);
 
   test->chan = tp_channel_new_from_properties (connection, object_path,
       immutable_properties, &new_error);
-  test_assert_no_error (new_error);
+  g_assert_no_error (new_error);
 
   g_main_loop_quit (test->mainloop);
 }
@@ -347,7 +304,7 @@ channel_ready_cb (TpChannel *channel G_GNUC_UNUSED,
 {
   Test *test = user_data;
 
-  test_assert_no_error (error);
+  g_assert_no_error ((GError *) error);
   g_main_loop_quit (test->mainloop);
 }
 
@@ -360,7 +317,7 @@ requested_streams_cb (TpChannel *chan G_GNUC_UNUSED,
 {
   Test *test = user_data;
 
-  CLEAR_BOXED (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
+  tp_clear_boxed (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
       &test->request_streams_return);
 
   if (error != NULL)
@@ -386,9 +343,9 @@ listed_streams_cb (TpChannel *chan G_GNUC_UNUSED,
   Test *test = user_data;
 
   /* ListStreams shouldn't fail in any of these tests */
-  test_assert_no_error (error);
+  g_assert_no_error ((GError *) error);
 
-  CLEAR_BOXED (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
+  tp_clear_boxed (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
       &test->list_streams_return);
 
   test->list_streams_return = g_boxed_copy (
@@ -639,7 +596,7 @@ test_basics (Test *test,
   tp_cli_channel_type_streamed_media_call_list_streams (test->chan, -1,
       listed_streams_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->list_streams_return->len, ==, 0);
 
@@ -670,7 +627,7 @@ test_basics (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->request_streams_return->len, ==, 1);
   audio_info = g_ptr_array_index (test->request_streams_return, 0);
@@ -702,7 +659,7 @@ test_basics (Test *test,
   tp_cli_channel_type_streamed_media_call_list_streams (test->chan, -1,
       listed_streams_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->list_streams_return->len, ==, 1);
   audio_info = g_ptr_array_index (test->list_streams_return, 0);
@@ -798,7 +755,7 @@ test_basics (Test *test,
 
   /* Immediately the call is accepted, the remote peer accepts our proposed
    * stream direction */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   se = g_slist_nth_data (test->stream_events, 0);
   g_assert_cmpuint (se->type, ==, STREAM_EVENT_DIRECTION_CHANGED);
@@ -831,7 +788,7 @@ test_basics (Test *test,
       test->video_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->request_streams_return->len, ==, 1);
 
@@ -862,7 +819,7 @@ test_basics (Test *test,
   tp_cli_channel_type_streamed_media_call_list_streams (test->chan, -1,
       listed_streams_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->list_streams_return->len, ==, 2);
 
@@ -939,14 +896,14 @@ test_basics (Test *test,
       test->stream_ids,
       void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* List streams again: now there's only the audio */
 
   tp_cli_channel_type_streamed_media_call_list_streams (test->chan, -1,
       listed_streams_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->list_streams_return->len, ==, 1);
   audio_info = g_ptr_array_index (test->list_streams_return, 0);
@@ -978,10 +935,10 @@ test_basics (Test *test,
       -1, test->contacts, "", TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
       void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* In response to hanging up, the channel closes */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
   g_assert (tp_proxy_get_invalidated (test->chan) != NULL);
 
   /* The last event should be that the peer and the self-handle were both
@@ -1033,9 +990,9 @@ test_no_answer (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   maybe_pop_stream_direction (test);
   g_assert_cmpuint (g_slist_length (test->stream_events), ==, 1);
@@ -1067,10 +1024,10 @@ test_no_answer (Test *test,
       -1, test->contacts, "", TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
       void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* In response to hanging up, the channel closes */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
   g_assert (tp_proxy_get_invalidated (test->chan) != NULL);
 
   /* The last event should be that the peer and the self-handle were both
@@ -1108,7 +1065,7 @@ test_busy (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* Wait for the remote contact to reject the call */
   while (tp_proxy_get_invalidated (test->chan) != NULL)
@@ -1118,7 +1075,7 @@ test_busy (Test *test,
 
   /* The last stream event should be the removal of the stream */
 
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   se = g_slist_nth_data (test->stream_events, 0);
   g_assert_cmpuint (se->type, ==, STREAM_EVENT_REMOVED);
@@ -1158,7 +1115,7 @@ test_terminated_by_peer (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* Wait for the remote contact to answer, if they haven't already */
 
@@ -1174,7 +1131,7 @@ test_terminated_by_peer (Test *test,
 
   /* The last stream event should be the removal of the stream */
 
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   se = g_slist_nth_data (test->stream_events, 0);
   g_assert_cmpuint (se->type, ==, STREAM_EVENT_REMOVED);
@@ -1212,9 +1169,9 @@ test_terminate_via_close (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   maybe_pop_stream_direction (test);
   g_assert_cmpuint (g_slist_length (test->stream_events), ==, 1);
@@ -1232,10 +1189,10 @@ test_terminate_via_close (Test *test,
 
   tp_cli_channel_call_close (test->chan, -1, void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* In response to hanging up, the channel closes */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
   g_assert (tp_proxy_get_invalidated (test->chan) != NULL);
 
   /* The last event should be that the peer and the self-handle were both
@@ -1278,9 +1235,9 @@ test_terminate_via_no_streams (Test *test,
       test->audio_request, requested_streams_cb,
       test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   maybe_pop_stream_direction (test);
   g_assert_cmpuint (g_slist_length (test->stream_events), ==, 1);
@@ -1302,10 +1259,10 @@ test_terminate_via_no_streams (Test *test,
       test->stream_ids,
       void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* In response to hanging up, the channel closes */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
   g_assert (tp_proxy_get_invalidated (test->chan) != NULL);
 
   /* The last event should be that the peer and the self-handle were both
@@ -1371,7 +1328,7 @@ expect_incoming_call_cb (TpConnection *conn,
       /* save the channel */
       test->chan = tp_channel_new_from_properties (conn, object_path,
           properties, &test->error);
-      test_assert_no_error (test->error);
+      g_assert_no_error (test->error);
     }
 }
 
@@ -1386,16 +1343,16 @@ trigger_incoming_call (Test *test,
 
   tp_cli_connection_interface_simple_presence_run_set_presence (test->conn, -1,
       "away", "preparing for a test", &test->error, NULL);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   new_channels_sig =
     tp_cli_connection_interface_requests_connect_to_new_channels (test->conn,
         expect_incoming_call_cb, test, NULL, NULL, &test->error);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   tp_cli_connection_interface_simple_presence_run_set_presence (test->conn, -1,
       "available", message, &test->error, NULL);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* wait for the call to happen if it hasn't already */
   while (test->chan == NULL)
@@ -1440,7 +1397,7 @@ test_incoming (Test *test,
   tp_cli_channel_type_streamed_media_call_list_streams (test->chan, -1,
       listed_streams_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   g_assert_cmpuint (test->list_streams_return->len, ==, 1);
   audio_info = g_ptr_array_index (test->list_streams_return, 0);
@@ -1485,7 +1442,7 @@ test_incoming (Test *test,
   tp_cli_channel_interface_group_call_add_members (test->chan,
       -1, test->contacts, "", void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* The self-handle and the peer are now the channel's members */
   g_assert_cmpuint (tp_channel_group_get_handle_owner (test->chan,
@@ -1506,7 +1463,7 @@ test_incoming (Test *test,
 
   /* Immediately the call is accepted, we accept the remote peer's proposed
    * stream direction */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
 
   test_assert_uu_hash_contains (test->stream_directions, test->audio_stream_id,
       TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL);
@@ -1532,10 +1489,10 @@ test_incoming (Test *test,
       -1, test->contacts, "", TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
       void_cb, test, NULL, NULL);
   g_main_loop_run (test->mainloop);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   /* In response to hanging up, the channel closes */
-  test_connection_run_until_dbus_queue_processed (test->conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (test->conn);
   g_assert (tp_proxy_get_invalidated (test->chan) != NULL);
 }
 
@@ -1544,7 +1501,7 @@ teardown (Test *test,
           gconstpointer data G_GNUC_UNUSED)
 {
   tp_cli_connection_run_disconnect (test->conn, -1, &test->error, NULL);
-  test_assert_no_error (test->error);
+  g_assert_no_error (test->error);
 
   if (test->members_changed_detailed_id != 0)
     {
@@ -1564,22 +1521,25 @@ teardown (Test *test,
   g_slist_foreach (test->stream_events, (GFunc) stream_event_destroy, NULL);
   g_slist_free (test->stream_events);
 
-  CLEAR_BOXED (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
+  tp_clear_boxed (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
       &test->list_streams_return);
-  CLEAR_BOXED (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
+  tp_clear_boxed (TP_ARRAY_TYPE_MEDIA_STREAM_INFO_LIST,
       &test->request_streams_return);
 
   g_hash_table_destroy (test->stream_directions);
   g_hash_table_destroy (test->stream_pending_sends);
   g_hash_table_destroy (test->stream_states);
 
-  CLEAR_OBJECT (&test->chan);
-  CLEAR_OBJECT (&test->conn);
-  CLEAR_OBJECT (&test->cm);
+  tp_clear_object (&test->chan);
+  tp_clear_object (&test->conn);
+  tp_clear_object (&test->cm);
 
-  CLEAR_OBJECT (&test->service_cm);
+  tp_clear_object (&test->service_cm);
 
-  CLEAR_OBJECT (&test->dbus);
+  /* make sure any pending things have happened */
+  tp_tests_proxy_run_until_dbus_queue_processed (test->dbus);
+
+  tp_clear_object (&test->dbus);
   g_main_loop_unref (test->mainloop);
   test->mainloop = NULL;
 }
