@@ -50,6 +50,20 @@ channel_ready (TpChannel *channel,
     g_main_loop_quit (mainloop);
 }
 
+static void
+channel_prepared_cb (GObject *object,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GAsyncResult **output = user_data;
+
+  g_message ("channel %p prepared", object);
+  *output = g_object_ref (res);
+
+  if (mainloop != NULL)
+    g_main_loop_quit (mainloop);
+}
+
 static void
 assert_chan_sane (TpChannel *chan,
                   TpHandle handle)
@@ -66,21 +80,21 @@ assert_chan_sane (TpChannel *chan,
   MYASSERT (tp_channel_get_channel_type_id (chan) ==
         TP_IFACE_QUARK_CHANNEL_TYPE_TEXT, "");
   MYASSERT (TP_IS_CONNECTION (tp_channel_borrow_connection (chan)), "");
-  MYASSERT_SAME_STRING (tp_channel_get_identifier (chan), IDENTIFIER);
+  g_assert_cmpstr (tp_channel_get_identifier (chan), ==, IDENTIFIER);
 
   asv = tp_channel_borrow_immutable_properties (chan);
   MYASSERT (asv != NULL, "");
-  MYASSERT_SAME_STRING (
-      tp_asv_get_string (asv, TP_PROP_CHANNEL_CHANNEL_TYPE),
+  g_assert_cmpstr (
+      tp_asv_get_string (asv, TP_PROP_CHANNEL_CHANNEL_TYPE), ==,
       TP_IFACE_CHANNEL_TYPE_TEXT);
-  MYASSERT_SAME_UINT (
-      tp_asv_get_uint32 (asv, TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL),
+  g_assert_cmpuint (
+      tp_asv_get_uint32 (asv, TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL), ==,
       TP_HANDLE_TYPE_CONTACT);
-  MYASSERT_SAME_UINT (
-      tp_asv_get_uint32 (asv, TP_PROP_CHANNEL_TARGET_HANDLE, NULL),
+  g_assert_cmpuint (
+      tp_asv_get_uint32 (asv, TP_PROP_CHANNEL_TARGET_HANDLE, NULL), ==,
       handle);
-  MYASSERT_SAME_STRING (
-      tp_asv_get_string (asv, TP_PROP_CHANNEL_TARGET_ID),
+  g_assert_cmpstr (
+      tp_asv_get_string (asv, TP_PROP_CHANNEL_TARGET_ID), ==,
       IDENTIFIER);
 }
 
@@ -88,12 +102,12 @@ int
 main (int argc,
       char **argv)
 {
-  SimpleConnection *service_conn;
+  TpTestsSimpleConnection *service_conn;
   TpBaseConnection *service_conn_as_base;
   TpHandleRepoIface *contact_repo;
-  TestTextChannelNull *service_chan;
-  TestPropsTextChannel *service_props_chan;
-  TestPropsGroupTextChannel *service_props_group_chan;
+  TpTestsTextChannelNull *service_chan;
+  TpTestsPropsTextChannel *service_props_chan;
+  TpTestsPropsGroupTextChannel *service_props_group_chan;
   TpDBusDaemon *dbus;
   TpConnection *conn;
   TpChannel *chan;
@@ -109,12 +123,16 @@ main (int argc,
   GError invalidated_for_test = { TP_ERRORS, TP_ERROR_PERMISSION_DENIED,
       "No channel for you!" };
   GHashTable *asv;
+  GAsyncResult *prepare_result;
+  GQuark some_features[] = { TP_CHANNEL_FEATURE_CORE,
+      TP_CHANNEL_FEATURE_CHAT_STATES, 0 };
 
   g_type_init ();
   tp_debug_set_flags ("all");
-  dbus = tp_dbus_daemon_new (tp_get_bus ());
+  dbus = tp_tests_dbus_daemon_dup_or_die ();
 
-  service_conn = SIMPLE_CONNECTION (g_object_new (SIMPLE_TYPE_CONNECTION,
+  service_conn = TP_TESTS_SIMPLE_CONNECTION (tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_SIMPLE_CONNECTION,
         "account", "me@example.com",
         "protocol", "simple",
         NULL));
@@ -124,27 +142,27 @@ main (int argc,
 
   MYASSERT (tp_base_connection_register (service_conn_as_base, "simple",
         &name, &conn_path, &error), "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   conn = tp_connection_new (dbus, name, conn_path, &error);
   MYASSERT (conn != NULL, "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_connection_run_until_ready (conn, TRUE, &error, NULL),
       "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   contact_repo = tp_base_connection_get_handles (service_conn_as_base,
       TP_HANDLE_TYPE_CONTACT);
   MYASSERT (contact_repo != NULL, "");
 
   handle = tp_handle_ensure (contact_repo, IDENTIFIER, NULL, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   chan_path = g_strdup_printf ("%s/Channel", conn_path);
 
-  service_chan = TEST_TEXT_CHANNEL_NULL (g_object_new (
-        TEST_TYPE_TEXT_CHANNEL_NULL,
+  service_chan = TP_TESTS_TEXT_CHANNEL_NULL (tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_TEXT_CHANNEL_NULL,
         "connection", service_conn,
         "object-path", chan_path,
         "handle", handle,
@@ -152,8 +170,9 @@ main (int argc,
 
   props_chan_path = g_strdup_printf ("%s/PropertiesChannel", conn_path);
 
-  service_props_chan = TEST_PROPS_TEXT_CHANNEL (g_object_new (
-        TEST_TYPE_PROPS_TEXT_CHANNEL,
+  service_props_chan = TP_TESTS_PROPS_TEXT_CHANNEL (
+      tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_PROPS_TEXT_CHANNEL,
         "connection", service_conn,
         "object-path", props_chan_path,
         "handle", handle,
@@ -161,8 +180,9 @@ main (int argc,
 
   props_group_chan_path = g_strdup_printf ("%s/PropsGroupChannel", conn_path);
 
-  service_props_group_chan = TEST_PROPS_GROUP_TEXT_CHANNEL (g_object_new (
-        TEST_TYPE_PROPS_GROUP_TEXT_CHANNEL,
+  service_props_group_chan = TP_TESTS_PROPS_GROUP_TEXT_CHANNEL (
+      tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_PROPS_GROUP_TEXT_CHANNEL,
         "connection", service_conn,
         "object-path", props_group_chan_path,
         "handle", handle,
@@ -174,15 +194,43 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
+
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      FALSE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
   tp_proxy_invalidate ((TpProxy *) chan, &invalidated_for_test);
 
+  prepare_result = NULL;
+  tp_proxy_prepare_async (chan, NULL, channel_prepared_cb, &prepare_result);
+
   MYASSERT (!tp_channel_run_until_ready (chan, &error, NULL), "");
   MYASSERT (error != NULL, "");
-  MYASSERT_SAME_ERROR (&invalidated_for_test, error);
+  g_assert_error (error, invalidated_for_test.domain,
+      invalidated_for_test.code);
+  g_assert_cmpstr (error->message, ==, invalidated_for_test.message);
   g_error_free (error);
   error = NULL;
 
+  if (prepare_result == NULL)
+    g_main_loop_run (mainloop);
+
+  MYASSERT (!tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_error (error, invalidated_for_test.domain,
+      invalidated_for_test.code);
+  g_assert_cmpstr (error->message, ==, invalidated_for_test.message);
+  g_clear_error (&error);
+  /* it was never ready */
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      FALSE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
+  g_object_unref (prepare_result);
+  prepare_result = NULL;
+
   g_object_unref (chan);
   chan = NULL;
 
@@ -190,23 +238,46 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   was_ready = FALSE;
+  tp_proxy_prepare_async (chan, NULL, channel_prepared_cb, &prepare_result);
+
+  /* no way to see what this is doing - just make sure it doesn't crash */
+  tp_proxy_prepare_async (chan, some_features, NULL, NULL);
+
   tp_channel_call_when_ready (chan, channel_ready, &was_ready);
   tp_proxy_invalidate ((TpProxy *) chan, &invalidated_for_test);
   MYASSERT (was_ready == TRUE, "");
   MYASSERT (invalidated != NULL, "");
-  MYASSERT_SAME_ERROR (&invalidated_for_test, invalidated);
+  g_assert_error (invalidated, invalidated_for_test.domain,
+      invalidated_for_test.code);
+  g_assert_cmpstr (invalidated->message, ==, invalidated_for_test.message);
   g_error_free (invalidated);
   invalidated = NULL;
 
+  /* prepare_async never calls back synchronously */
+  MYASSERT (prepare_result == NULL, "");
+  g_main_loop_run (mainloop);
+  MYASSERT (!tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_error (error, invalidated_for_test.domain,
+      invalidated_for_test.code);
+  g_assert_cmpstr (error->message, ==, invalidated_for_test.message);
+  g_clear_error (&error);
+  g_object_unref (prepare_result);
+  prepare_result = NULL;
+  /* it was never ready */
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      FALSE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
   g_object_unref (chan);
   chan = NULL;
 
   g_message ("Channel becomes ready while we wait");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -214,13 +285,30 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
+
+  prepare_result = NULL;
+  tp_proxy_prepare_async (chan, NULL, channel_prepared_cb, &prepare_result);
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 0);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 0);
+  g_assert_no_error (error);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 0);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 0);
+
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      TRUE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
+  if (prepare_result == NULL)
+    g_main_loop_run (mainloop);
+
+  MYASSERT (tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_no_error (error);
+
+  g_object_unref (prepare_result);
+  prepare_result = NULL;
 
   assert_chan_sane (chan, handle);
 
@@ -230,40 +318,82 @@ main (int argc,
   g_message ("Channel becomes ready while we wait (the version with "
       "Properties)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called = 0;
 
   chan = tp_channel_new (conn, props_chan_path, NULL,
       TP_UNKNOWN_HANDLE_TYPE, 0, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
+
+  prepare_result = NULL;
+  tp_proxy_prepare_async (chan, some_features, channel_prepared_cb,
+      &prepare_result);
+
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      FALSE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called, 0);
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called, 0);
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called, 0);
+  g_assert_no_error (error);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called, ==,
+      0);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called,
+      ==, 0);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called,
+      ==, 0);
+
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      TRUE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
+  if (prepare_result == NULL)
+    g_main_loop_run (mainloop);
+
+  MYASSERT (tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_no_error (error);
+
+  g_object_unref (prepare_result);
+  prepare_result = NULL;
 
   assert_chan_sane (chan, handle);
 
+  /* no way to see what this is doing - just make sure it doesn't crash */
+  tp_proxy_prepare_async (chan, some_features, NULL, NULL);
+
+  prepare_result = NULL;
+  tp_proxy_prepare_async (chan, some_features, channel_prepared_cb,
+      &prepare_result);
+
+  if (prepare_result == NULL)
+    g_main_loop_run (mainloop);
+
+  MYASSERT (tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_no_error (error);
+
+  g_object_unref (prepare_result);
+  prepare_result = NULL;
+
   g_object_unref (chan);
   chan = NULL;
 
   g_message ("Channel becomes ready while we wait (preloading immutable "
       "properties)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called = 0;
 
-  g_hash_table_remove_all (TEST_PROPS_TEXT_CHANNEL (service_props_chan)
+  g_hash_table_remove_all (TP_TESTS_PROPS_TEXT_CHANNEL (service_props_chan)
       ->dbus_property_interfaces_retrieved);
 
   asv = tp_asv_new (
@@ -279,23 +409,25 @@ main (int argc,
       NULL);
 
   chan = tp_channel_new_from_properties (conn, props_chan_path, asv, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   g_hash_table_destroy (asv);
   asv = NULL;
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (g_hash_table_size (
-      service_props_chan->dbus_property_interfaces_retrieved), 0);
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called, 0);
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called, 0);
+  g_assert_no_error (error);
+  g_assert_cmpuint (g_hash_table_size (
+      service_props_chan->dbus_property_interfaces_retrieved), ==, 0);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_handle_called, ==, 0);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_channel_type_called,
+      ==, 0);
   /* FIXME: with an improved fast-path we could avoid this one too maybe? */
   /*
-  MYASSERT_SAME_UINT (
-      TEST_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called, 0);
+  g_assert_cmpuint (
+      TP_TESTS_TEXT_CHANNEL_NULL (service_props_chan)->get_interfaces_called,
+      ==, 0);
    */
 
   assert_chan_sane (chan, handle);
@@ -306,14 +438,15 @@ main (int argc,
   g_message ("Group channel becomes ready while we wait (preloading immutable "
       "properties)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
-  TEST_TEXT_CHANNEL_NULL (service_props_group_chan)->get_handle_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_group_chan)->get_interfaces_called = 0;
-  TEST_TEXT_CHANNEL_NULL (service_props_group_chan)->get_channel_type_called
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)->get_handle_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)->get_interfaces_called = 0;
+  TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)->get_channel_type_called
       = 0;
 
-  g_hash_table_remove_all (TEST_PROPS_TEXT_CHANNEL (service_props_group_chan)
+  g_hash_table_remove_all (TP_TESTS_PROPS_TEXT_CHANNEL (
+        service_props_group_chan)
       ->dbus_property_interfaces_retrieved);
 
   {
@@ -337,24 +470,24 @@ main (int argc,
   }
 
   chan = tp_channel_new_from_properties (conn, props_group_chan_path, asv, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   g_hash_table_destroy (asv);
   asv = NULL;
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (TEST_TEXT_CHANNEL_NULL (service_props_group_chan)
-      ->get_handle_called, 0);
-  MYASSERT_SAME_UINT (TEST_TEXT_CHANNEL_NULL (service_props_group_chan)
-      ->get_channel_type_called, 0);
-  MYASSERT_SAME_UINT (TEST_TEXT_CHANNEL_NULL (service_props_group_chan)
-      ->get_interfaces_called, 0);
-  MYASSERT_SAME_UINT (g_hash_table_size (
-      TEST_PROPS_TEXT_CHANNEL (service_props_group_chan)
-      ->dbus_property_interfaces_retrieved), 1);
+  g_assert_no_error (error);
+  g_assert_cmpuint (TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)
+      ->get_handle_called, ==, 0);
+  g_assert_cmpuint (TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)
+      ->get_channel_type_called, ==, 0);
+  g_assert_cmpuint (TP_TESTS_TEXT_CHANNEL_NULL (service_props_group_chan)
+      ->get_interfaces_called, ==, 0);
+  g_assert_cmpuint (g_hash_table_size (
+      TP_TESTS_PROPS_TEXT_CHANNEL (service_props_group_chan)
+      ->dbus_property_interfaces_retrieved), ==, 1);
   MYASSERT (g_hash_table_lookup (
-      TEST_PROPS_TEXT_CHANNEL (service_props_group_chan)
+      TP_TESTS_PROPS_TEXT_CHANNEL (service_props_group_chan)
       ->dbus_property_interfaces_retrieved,
       GUINT_TO_POINTER (TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) != NULL,
       "Only Chan.I.Group's properties should have been retrieved");
@@ -367,7 +500,7 @@ main (int argc,
   g_message ("Channel becomes ready while we wait (in the case where we "
       "have to discover the channel type)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -375,13 +508,13 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, NULL,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 0);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 1);
+  g_assert_no_error (error);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 0);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 1);
 
   assert_chan_sane (chan, handle);
 
@@ -391,7 +524,7 @@ main (int argc,
   g_message ("Channel becomes ready while we wait (in the case where we "
       "have to discover the handle type)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -399,13 +532,13 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_UNKNOWN_HANDLE_TYPE, 0, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 0);
+  g_assert_no_error (error);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 0);
 
   assert_chan_sane (chan, handle);
 
@@ -415,7 +548,7 @@ main (int argc,
   g_message ("Channel becomes ready while we wait (in the case where we "
       "have to discover the handle)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -423,13 +556,13 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, 0, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_channel_run_until_ready (chan, &error, NULL), "");
-  test_assert_no_error (error);
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 0);
+  g_assert_no_error (error);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 0);
 
   assert_chan_sane (chan, handle);
 
@@ -441,7 +574,7 @@ main (int argc,
   bad_chan_path = g_strdup_printf ("%s/Does/Not/Actually/Exist", conn_path);
   chan = tp_channel_new (conn, bad_chan_path, NULL,
       TP_UNKNOWN_HANDLE_TYPE, 0, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   was_ready = FALSE;
   tp_channel_call_when_ready (chan, channel_ready, &was_ready);
@@ -465,7 +598,7 @@ main (int argc,
   bad_chan_path = g_strdup_printf ("%s/Does/Not/Actually/Exist", conn_path);
   chan = tp_channel_new (conn, bad_chan_path, NULL,
       TP_UNKNOWN_HANDLE_TYPE, 0, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (!tp_channel_run_until_ready (chan, &error, NULL), "");
   MYASSERT (error != NULL, "");
@@ -484,7 +617,7 @@ main (int argc,
   g_message ("Channel doesn't actually implement Group (preloading immutable "
       "properties)");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -514,7 +647,7 @@ main (int argc,
    * should make introspection fail.
    */
   chan = tp_channel_new_from_properties (conn, chan_path, asv, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   g_hash_table_destroy (asv);
   asv = NULL;
@@ -528,16 +661,16 @@ main (int argc,
   g_error_free (error);
   error = NULL;
 
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 0);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 0);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 0);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 0);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 0);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 0);
 
   g_object_unref (chan);
   chan = NULL;
 
   g_message ("Channel becomes ready and we are called back");
 
-  test_connection_run_until_dbus_queue_processed (conn);
+  tp_tests_proxy_run_until_dbus_queue_processed (conn);
 
   service_chan->get_handle_called = 0;
   service_chan->get_interfaces_called = 0;
@@ -545,7 +678,7 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   was_ready = FALSE;
   tp_channel_call_when_ready (chan, channel_ready, &was_ready);
@@ -553,10 +686,10 @@ main (int argc,
   g_main_loop_run (mainloop);
   g_message ("Leaving main loop");
   MYASSERT (was_ready == TRUE, "");
-  test_assert_no_error (invalidated);
-  MYASSERT_SAME_UINT (service_chan->get_handle_called, 0);
-  MYASSERT_SAME_UINT (service_chan->get_interfaces_called, 1);
-  MYASSERT_SAME_UINT (service_chan->get_channel_type_called, 0);
+  g_assert_no_error (invalidated);
+  g_assert_cmpuint (service_chan->get_handle_called, ==, 0);
+  g_assert_cmpuint (service_chan->get_interfaces_called, ==, 1);
+  g_assert_cmpuint (service_chan->get_channel_type_called, ==, 0);
 
   assert_chan_sane (chan, handle);
 
@@ -567,7 +700,7 @@ main (int argc,
   was_ready = FALSE;
   tp_channel_call_when_ready (chan, channel_ready, &was_ready);
   MYASSERT (was_ready == TRUE, "");
-  test_assert_no_error (invalidated);
+  g_assert_no_error (invalidated);
 
   assert_chan_sane (chan, handle);
 
@@ -575,10 +708,20 @@ main (int argc,
 
   g_message ("Channel already dead, so we are called back synchronously");
 
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      TRUE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+
   MYASSERT (tp_cli_connection_run_disconnect (conn, -1, &error, NULL), "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   was_ready = FALSE;
+
+  prepare_result = NULL;
+  tp_proxy_prepare_async (chan, some_features, channel_prepared_cb,
+      &prepare_result);
+
   tp_channel_call_when_ready (chan, channel_ready, &was_ready);
   MYASSERT (was_ready == TRUE, "");
   MYASSERT (invalidated != NULL, "");
@@ -586,8 +729,27 @@ main (int argc,
       ": %s", g_quark_to_string (invalidated->domain));
   MYASSERT (invalidated->code == TP_ERROR_CANCELLED,
       ": %u", invalidated->code);
-  g_error_free (invalidated);
-  invalidated = NULL;
+
+  /* is_prepared becomes FALSE because the channel broke */
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CORE), ==,
+      FALSE);
+  g_assert_cmpint (tp_proxy_is_prepared (chan, TP_CHANNEL_FEATURE_CHAT_STATES),
+      ==, FALSE);
+  g_assert_error (invalidated, tp_proxy_get_invalidated (chan)->domain,
+      tp_proxy_get_invalidated (chan)->code);
+  g_assert_cmpstr (invalidated->message, ==,
+      tp_proxy_get_invalidated (chan)->message);
+
+  /* ... but prepare_async still hasn't finished until we run the main loop */
+  g_assert (prepare_result == NULL);
+  g_main_loop_run (mainloop);
+  g_assert (prepare_result != NULL);
+  MYASSERT (!tp_proxy_prepare_finish (chan, prepare_result, &error), "");
+  g_assert_error (error, invalidated->domain, invalidated->code);
+  g_assert_cmpstr (error->message, ==, invalidated->message);
+
+  g_clear_error (&error);
+  g_clear_error (&invalidated);
 
   g_object_unref (chan);
   chan = NULL;
