@@ -68,7 +68,7 @@ requested_name (TpDBusDaemon *proxy,
   g_message ("RequestName raised %s",
       (error == NULL ? "no error" : error->message));
   /* we're on a private bus, so certainly nobody else should own this name */
-  test_assert_no_error (error);
+  g_assert_no_error ((GError *) error);
   MYASSERT (result == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER, ": %u", result);
 }
 
@@ -172,30 +172,30 @@ main (int argc,
   /* We use TpDBusDaemon because it's a convenient concrete subclass of
    * TpProxy. */
   g_message ("Creating proxies");
-  a = tp_dbus_daemon_new (tp_get_bus ());
+  a = tp_tests_dbus_daemon_dup_or_die ();
   g_message ("a=%p", a);
-  b = tp_dbus_daemon_new (tp_get_bus ());
+  b = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("b=%p", b);
-  c = tp_dbus_daemon_new (tp_get_bus ());
+  c = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("c=%p", c);
-  d = tp_dbus_daemon_new (tp_get_bus ());
+  d = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("d=%p", d);
-  e = tp_dbus_daemon_new (tp_get_bus ());
+  e = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("e=%p", e);
-  f = tp_dbus_daemon_new (tp_get_bus ());
+  f = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("f=%p", f);
-  g = tp_dbus_daemon_new (tp_get_bus ());
+  g = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("g=%p", g);
-  h = tp_dbus_daemon_new (tp_get_bus ());
+  h = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("h=%p", h);
-  z = tp_dbus_daemon_new (tp_get_bus ());
+  z = tp_dbus_daemon_new (tp_proxy_get_dbus_connection (a));
   g_message ("z=%p", z);
 
   /* a survives */
   g_message ("Connecting signal to a");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (a, noc, PTR (TEST_A),
       destroy_user_data, (GObject *) z, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
 
   /* assert that connecting to a signal on an interface we don't have fails */
   freed = FALSE;
@@ -209,11 +209,12 @@ main (int argc,
 
   /* b gets its signal connection cancelled because stub is
    * destroyed */
-  stub = g_object_new (stub_object_get_type (), NULL);
+  stub = tp_tests_object_new_static_class (tp_tests_stub_object_get_type (),
+      NULL);
   g_message ("Connecting signal to b");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (b, noc, PTR (TEST_B),
       destroy_user_data, stub, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_B), "");
   g_object_unref (stub);
 
@@ -222,7 +223,7 @@ main (int argc,
   g_message ("Connecting signal to c");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (c, noc, PTR (TEST_C),
       destroy_user_data, NULL, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_C), "");
   g_message ("Forcibly invalidating c");
   tp_proxy_invalidate ((TpProxy *) c, &err);
@@ -244,7 +245,7 @@ main (int argc,
   g_message ("Connecting signal to d");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (d, noc, PTR (TEST_D),
       destroy_user_data, NULL, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_D), "");
   g_message ("Destroying d");
   tmp_obj = d;
@@ -257,7 +258,7 @@ main (int argc,
   g_message ("Connecting signal to e");
   sc = tp_cli_dbus_daemon_connect_to_name_owner_changed (e, noc, PTR (TEST_E),
       destroy_user_data, NULL, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_E), "");
   g_message ("Disconnecting signal from e");
   tp_proxy_signal_connection_disconnect (sc);
@@ -270,7 +271,7 @@ main (int argc,
   g_message ("Connecting signal to f");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (f, noc, PTR (TEST_F),
       destroy_user_data, NULL, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_F), "");
   g_message ("Forcibly disposing f's DBusGProxy to simulate name owner loss");
   tmp_obj = tp_proxy_borrow_interface_by_id ((TpProxy *) f,
@@ -294,7 +295,7 @@ main (int argc,
   g_message ("Connecting signal to g");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (g, noc, PTR (TEST_G),
       destroy_user_data, (GObject *) g, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_G), "");
   g_message ("Destroying g");
   tmp_obj = g;
@@ -306,12 +307,13 @@ main (int argc,
   /* h gets its signal connection cancelled because its weak object is
    * destroyed, meaning there are simultaneously two reasons for it to become
    * cancelled (fd.o#14750) */
-  stub = g_object_new (stub_object_get_type (), NULL);
+  stub = tp_tests_object_new_static_class (tp_tests_stub_object_get_type (),
+      NULL);
   g_object_weak_ref (stub, h_stub_destroyed, &sc);
   g_message ("Connecting signal to h");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (h, noc, PTR (TEST_H),
       destroy_user_data, stub, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
   MYASSERT (!tp_intset_is_member (freed_user_data, TEST_H), "");
   g_object_unref (stub);
 
@@ -321,7 +323,7 @@ main (int argc,
   g_message ("Connecting signal to z");
   tp_cli_dbus_daemon_connect_to_name_owner_changed (z, noc, PTR (TEST_Z),
       destroy_user_data, (GObject *) a, &error_out);
-  test_assert_no_error (error_out);
+  g_assert_no_error (error_out);
 
   /* make sure a NameOwnerChanged signal occurs */
   g_message ("Requesting name");
