@@ -17,15 +17,11 @@
 #include "tests/lib/myassert.h"
 #include "tests/lib/simple-conn.h"
 #include "tests/lib/textchan-null.h"
+#include "tests/lib/util.h"
 
-static int fail = 0;
 static GMainLoop *mainloop;
-
-static void
-myassert_failed (void)
-{
-  fail = 1;
-}
+gboolean shutdown_finished = FALSE;
+gboolean invalidated = FALSE;
 
 static void
 on_invalidated (TpChannel *chan,
@@ -43,12 +39,17 @@ on_invalidated (TpChannel *chan,
   MYASSERT (*client == chan, "%p vs %p", *client, chan);
   g_object_unref (*client);
   *client = NULL;
+
+  invalidated = TRUE;
+
+  if (shutdown_finished)
+    g_main_loop_quit (mainloop);
 }
 
 static gboolean
 disconnect (gpointer data)
 {
-  simple_connection_inject_disconnect (data);
+  tp_tests_simple_connection_inject_disconnect (data);
 
   return FALSE;
 }
@@ -57,17 +58,20 @@ static void
 on_shutdown_finished (TpBaseConnection *base_conn,
                       gpointer user_data)
 {
-  g_main_loop_quit (mainloop);
+  shutdown_finished = TRUE;
+
+  if (invalidated)
+    g_main_loop_quit (mainloop);
 }
 
 int
 main (int argc,
       char **argv)
 {
-  SimpleConnection *service_conn;
+  TpTestsSimpleConnection *service_conn;
   TpBaseConnection *service_conn_as_base;
   TpHandleRepoIface *contact_repo;
-  TestTextChannelNull *service_chan;
+  TpTestsTextChannelNull *service_chan;
   TpDBusDaemon *dbus;
   TpConnection *conn;
   TpChannel *chan;
@@ -80,9 +84,10 @@ main (int argc,
   g_type_init ();
   tp_debug_set_flags ("all");
   mainloop = g_main_loop_new (NULL, FALSE);
-  dbus = tp_dbus_daemon_new (tp_get_bus ());
+  dbus = tp_tests_dbus_daemon_dup_or_die ();
 
-  service_conn = SIMPLE_CONNECTION (g_object_new (SIMPLE_TYPE_CONNECTION,
+  service_conn = TP_TESTS_SIMPLE_CONNECTION (tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_SIMPLE_CONNECTION,
         "account", "me@example.com",
         "protocol", "simple",
         NULL));
@@ -95,14 +100,14 @@ main (int argc,
 
   MYASSERT (tp_base_connection_register (service_conn_as_base, "simple",
         &name, &conn_path, &error), "");
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
 
   conn = tp_connection_new (dbus, name, conn_path, &error);
   MYASSERT (conn != NULL, "");
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_connection_run_until_ready (conn, TRUE, &error, NULL), "");
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
 
   /* Paste on a channel */
 
@@ -111,11 +116,11 @@ main (int argc,
   MYASSERT (contact_repo != NULL, "");
 
   handle = tp_handle_ensure (contact_repo, "them@example.org", NULL, &error);
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
   chan_path = g_strdup_printf ("%s/Channel", conn_path);
 
-  service_chan = TEST_TEXT_CHANNEL_NULL (g_object_new (
-        TEST_TYPE_TEXT_CHANNEL_NULL,
+  service_chan = TP_TESTS_TEXT_CHANNEL_NULL (tp_tests_object_new_static_class (
+        TP_TESTS_TYPE_TEXT_CHANNEL_NULL,
         "connection", service_conn,
         "object-path", chan_path,
         "handle", handle,
@@ -123,10 +128,10 @@ main (int argc,
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
 
   tp_channel_run_until_ready (chan, &error, NULL);
-  MYASSERT_NO_ERROR (error);
+  g_assert_no_error (error);
 
   g_signal_connect (chan, "invalidated", G_CALLBACK (on_invalidated),
       &chan);
@@ -150,5 +155,5 @@ main (int argc,
   g_free (conn_path);
   g_free (chan_path);
 
-  return fail;
+  return 0;
 }
