@@ -15,6 +15,7 @@
 #include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/interfaces.h>
 
+#include "examples/cm/echo-message-parts/connection-manager.h"
 #include "examples/cm/echo-message-parts/chan.h"
 #include "examples/cm/echo-message-parts/conn.h"
 #include "tests/lib/myassert.h"
@@ -59,11 +60,8 @@ on_received (TpChannel *chan,
              gpointer data,
              GObject *object)
 {
-  TpHandleRepoIface *contact_repo = data;
-
-  g_print ("%p: Received #%u: time %u, sender %u '%s', type %u, flags %u, "
-      "text '%s'\n", chan, id, timestamp, sender,
-      tp_handle_inspect (contact_repo, sender), type, flags, text);
+  g_print ("%p: Received #%u: time %u, sender %u, type %u, flags %u, "
+      "text '%s'\n", chan, id, timestamp, sender, type, flags, text);
 
   received_count++;
   last_received_id = id;
@@ -114,7 +112,6 @@ on_message_received (TpChannel *chan,
                      gpointer data,
                      GObject *object)
 {
-  TpHandleRepoIface *contact_repo = data;
   guint i;
   GHashTable *headers = g_ptr_array_index (parts, 0);
   guint id;
@@ -129,9 +126,8 @@ on_message_received (TpChannel *chan,
   sender = tp_asv_get_uint32 (headers, "message-sender", NULL);
   received = tp_asv_get_uint32 (headers, "message-received", NULL);
 
-  g_print ("%p: MessageReceived #%u: received at %u, sender %u '%s', type %u, "
-      "%u parts\n", chan, id, received, sender,
-      tp_handle_inspect (contact_repo, sender), type, parts->len);
+  g_print ("%p: MessageReceived #%u: received at %u, sender %u, type %u, "
+      "%u parts\n", chan, id, received, sender, type, parts->len);
 
   for (i = 0; i < parts->len; i++)
     {
@@ -197,10 +193,10 @@ int
 main (int argc,
       char **argv)
 {
-  ExampleEcho2Connection *service_conn;
-  TpBaseConnection *service_conn_as_base;
-  TpHandleRepoIface *contact_repo;
+  ExampleEcho2ConnectionManager *service_cm;
+  TpBaseConnectionManager *service_cm_as_base;
   TpDBusDaemon *dbus;
+  TpConnectionManager *cm;
   TpConnection *conn;
   TpChannel *chan;
   GError *error = NULL;
@@ -208,38 +204,64 @@ main (int argc,
   gchar *conn_path;
   gchar *chan_path;
   TpHandle handle;
+  gboolean ok;
+  GHashTable *parameters;
 
   g_type_init ();
   tp_debug_set_flags ("all");
-  dbus = tp_dbus_daemon_new (tp_get_bus ());
+  dbus = tp_tests_dbus_daemon_dup_or_die ();
 
-  service_conn = EXAMPLE_ECHO_2_CONNECTION (g_object_new (
-        EXAMPLE_TYPE_ECHO_2_CONNECTION,
-        "account", "me@example.com",
-        "protocol", "example",
+  service_cm = EXAMPLE_ECHO_2_CONNECTION_MANAGER (
+      tp_tests_object_new_static_class (
+        EXAMPLE_TYPE_ECHO_2_CONNECTION_MANAGER,
         NULL));
-  service_conn_as_base = TP_BASE_CONNECTION (service_conn);
-  MYASSERT (service_conn != NULL, "");
-  MYASSERT (service_conn_as_base != NULL, "");
+  g_assert (service_cm != NULL);
+  service_cm_as_base = TP_BASE_CONNECTION_MANAGER (service_cm);
+  g_assert (service_cm_as_base != NULL);
 
-  MYASSERT (tp_base_connection_register (service_conn_as_base, "example",
-        &name, &conn_path, &error), "");
-  test_assert_no_error (error);
+  ok = tp_base_connection_manager_register (service_cm_as_base);
+  g_assert (ok);
+
+  cm = tp_connection_manager_new (dbus, "example_echo_2", NULL, &error);
+  g_assert (cm != NULL);
+  tp_tests_proxy_run_until_prepared (cm, NULL);
+
+  parameters = tp_asv_new (
+      "account", G_TYPE_STRING, "me@example.com",
+      NULL);
+
+  tp_cli_connection_manager_run_request_connection (cm, -1,
+      "example", parameters, &name, &conn_path, &error, NULL);
+  g_assert_no_error (error);
+
+  g_hash_table_unref (parameters);
 
   conn = tp_connection_new (dbus, name, conn_path, &error);
   MYASSERT (conn != NULL, "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_connection_run_until_ready (conn, TRUE, &error, NULL),
       "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
-  contact_repo = tp_base_connection_get_handles (service_conn_as_base,
-      TP_HANDLE_TYPE_CONTACT);
-  MYASSERT (contact_repo != NULL, "");
+    {
+      GHashTable *properties = NULL;
+      GPtrArray *arr;
 
-  handle = tp_handle_ensure (contact_repo, "them@example.org", NULL, &error);
-  test_assert_no_error (error);
+      /* check that it has the requestable channel class */
+      tp_cli_dbus_properties_run_get_all (conn, -1,
+          TP_IFACE_CONNECTION_INTERFACE_REQUESTS, &properties, &error, NULL);
+      g_assert_no_error (error);
+
+      arr = tp_asv_get_boxed (properties, "RequestableChannelClasses",
+          TP_ARRAY_TYPE_REQUESTABLE_CHANNEL_CLASS_LIST);
+      g_assert_cmpuint (arr->len, ==, 1);
+
+      g_hash_table_unref (properties);
+    }
+
+  handle = tp_tests_connection_run_request_contact_handle (conn,
+      "them@example.com");
 
     {
       GHashTable *request = tp_asv_new (
@@ -252,27 +274,26 @@ main (int argc,
 
       tp_cli_connection_interface_requests_run_create_channel (conn, -1,
           request, &chan_path, NULL, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_hash_table_destroy (request);
     }
 
   chan = tp_channel_new (conn, chan_path, TP_IFACE_CHANNEL_TYPE_TEXT,
       TP_HANDLE_TYPE_CONTACT, handle, &error);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   tp_channel_run_until_ready (chan, &error, NULL);
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
   MYASSERT (tp_cli_channel_type_text_connect_to_received (chan, on_received,
-      g_object_ref (contact_repo), g_object_unref, NULL, NULL) != NULL, "");
+      NULL, NULL, NULL, NULL) != NULL, "");
   MYASSERT (tp_cli_channel_type_text_connect_to_sent (chan, on_sent,
       NULL, NULL, NULL, NULL) != NULL, "");
 
   MYASSERT (
       tp_cli_channel_interface_messages_connect_to_message_received (chan,
-          on_message_received, g_object_ref (contact_repo), g_object_unref,
-          NULL, NULL) != NULL, "");
+          on_message_received, NULL, NULL, NULL, NULL) != NULL, "");
   MYASSERT (tp_cli_channel_interface_messages_connect_to_message_sent (
         chan, on_message_sent, NULL, NULL, NULL, NULL) != NULL, "");
   MYASSERT (
@@ -289,7 +310,7 @@ main (int argc,
 
       tp_cli_dbus_properties_run_get_all (chan, -1,
           TP_IFACE_CHANNEL_INTERFACE_MESSAGES, &properties, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_print ("\n\n==== Examining properties ====\n\n");
 
@@ -341,7 +362,7 @@ main (int argc,
   while (received_count < 1 || message_received_count < 1)
     g_main_context_iteration (NULL, TRUE);
 
-  test_assert_no_error (error);
+  g_assert_no_error (error);
   MYASSERT (sent_count == 1, ": %u != 1", sent_count);
   MYASSERT (received_count == 1, ": %u != 1", received_count);
   MYASSERT (last_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
@@ -381,7 +402,7 @@ main (int argc,
   while (received_count < 1 || message_received_count < 1)
     g_main_context_iteration (NULL, TRUE);
 
-  test_assert_no_error (error);
+  g_assert_no_error (error);
   MYASSERT (sent_count == 1, ": %u != 1", sent_count);
   MYASSERT (received_count == 1, ": %u != 1", received_count);
   MYASSERT (last_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
@@ -422,7 +443,7 @@ main (int argc,
   while (received_count < 1 || message_received_count < 1)
     g_main_context_iteration (NULL, TRUE);
 
-  test_assert_no_error (error);
+  g_assert_no_error (error);
   MYASSERT (sent_count == 1, ": %u != 1", sent_count);
   MYASSERT (received_count == 1, ": %u != 1", received_count);
   MYASSERT (last_sent_type == TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
@@ -479,47 +500,37 @@ main (int argc,
       gchar *token = NULL;
 
       /* Empty headers part */
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
+      part = tp_asv_new (NULL, NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("main"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/html"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string (
+      part = tp_asv_new (
+          "alternative", G_TYPE_STRING, "main",
+          "content-type", G_TYPE_STRING, "text/html",
+          "content", G_TYPE_STRING,
             "Here is a photo of a cat:<br />"
             "<img src=\"cid:lolcat\" alt=\"lol!\" /><br />"
-            "It's in ur regression tests verifying ur designs!"
-            ));
+            "It's in ur regression tests verifying ur designs!",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("main"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string (EXPECTED_TEXT));
+      part = tp_asv_new (
+          "alternative", G_TYPE_STRING, "main",
+          "content-type", G_TYPE_STRING, "text/plain",
+          "content", G_TYPE_STRING, EXPECTED_TEXT,
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "identifier",
-          tp_g_value_slice_new_string ("lolcat"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("image/jpeg"));
+      part = tp_asv_new (
+          "identifier", G_TYPE_STRING, "lolcat",
+          "content-type", G_TYPE_STRING, "image/jpeg",
+          NULL);
       g_hash_table_insert (part, "content", tp_g_value_slice_new_bytes (
             14, "\xff\xd8\xff\xe0\x00\x10JFIF\x00..."));
       g_ptr_array_add (send_parts, part);
 
       tp_cli_channel_interface_messages_run_send_message (chan, -1,
           send_parts, 0 /* flags */, &token, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       /* wait for pending events to be delivered */
       while (received_count < 1 || message_received_count < 1)
@@ -583,65 +594,49 @@ main (int argc,
       gchar *token = NULL;
 
       /* Empty headers part */
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
+      part = tp_asv_new (NULL, NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("main"));
-      g_hash_table_insert (part, "identifier",
-          tp_g_value_slice_new_string ("html"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/html"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string (
+      part = tp_asv_new (
+          "alternative", G_TYPE_STRING, "main",
+          "identifier", G_TYPE_STRING, "html",
+          "content-type", G_TYPE_STRING, "text/html",
+          "content", G_TYPE_STRING,
             "Here is a photo of a cat:<br />"
             "<img src=\"cid:lolcat\" alt=\"lol!\" /><br />"
-            "It's in ur regression tests verifying ur designs!"
-            ));
+            "It's in ur regression tests verifying ur designs!",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("main"));
-      g_hash_table_insert (part, "identifier",
-          tp_g_value_slice_new_string ("text"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string (EXPECTED_TEXT));
+      part = tp_asv_new (
+          "alternative", G_TYPE_STRING, "main",
+          "identifier", G_TYPE_STRING, "text",
+          "content-type", G_TYPE_STRING, "text/plain",
+          "content", G_TYPE_STRING, EXPECTED_TEXT,
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "identifier",
-          tp_g_value_slice_new_string ("jpeg"));
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("lolcat"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("image/jpeg"));
+      part = tp_asv_new (
+          "identifier", G_TYPE_STRING, "jpeg",
+          "alternative", G_TYPE_STRING, "lolcat",
+          "content-type", G_TYPE_STRING, "image/jpeg",
+          NULL);
       g_hash_table_insert (part, "content", tp_g_value_slice_new_bytes (
             14, "\xff\xd8\xff\xe0\x00\x10JFIF\x00..."));
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "identifier",
-          tp_g_value_slice_new_string ("png"));
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("lolcat"));
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("image/png"));
+      part = tp_asv_new (
+          "identifier", G_TYPE_STRING, "png",
+          "alternative", G_TYPE_STRING, "lolcat",
+          "content-type", G_TYPE_STRING, "image/png",
+          NULL);
       g_hash_table_insert (part, "content", tp_g_value_slice_new_bytes (
             12, "\x89PNG\x0d\x0a\x1a\x0a\x00..."));
       g_ptr_array_add (send_parts, part);
 
       tp_cli_channel_interface_messages_run_send_message (chan, -1,
           send_parts, 0 /* flags */, &token, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       /* wait for pending events to be delivered */
       while (received_count < 1 || message_received_count < 1)
@@ -708,37 +703,30 @@ main (int argc,
       gchar *token = NULL;
 
       /* Empty headers part */
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
+      part = tp_asv_new (NULL, NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string ("I'm on a roll\n"));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "content", G_TYPE_STRING, "I'm on a roll\n",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string ("I'm on a roll this time\n"));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "content", G_TYPE_STRING, "I'm on a roll this time\n",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string ("I feel my luck could change\n"));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "content", G_TYPE_STRING, "I feel my luck could change\n",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
       tp_cli_channel_interface_messages_run_send_message (chan, -1,
           send_parts, 0 /* flags */, &token, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       /* wait for pending events to be delivered */
       while (received_count < 1 || message_received_count < 1)
@@ -801,49 +789,38 @@ main (int argc,
       gchar *token = NULL;
 
       /* Empty headers part */
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
+      part = tp_asv_new (NULL, NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("alt"));
-      g_hash_table_insert (part, "lang",
-          tp_g_value_slice_new_string ("fr_CA@collabora"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string (EXPECTED_TEXT));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "alternative", G_TYPE_STRING, "alt",
+          "lang", G_TYPE_STRING, "fr_CA@collabora",
+          "content", G_TYPE_STRING, EXPECTED_TEXT,
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("alt"));
-      g_hash_table_insert (part, "lang",
-          tp_g_value_slice_new_string ("en_GB"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string ("we're fixing the colour of the video stream"));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "alternative", G_TYPE_STRING, "alt",
+          "lang", G_TYPE_STRING, "en_GB",
+          "content", G_TYPE_STRING,
+            "we're fixing the colour of the video stream",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
-      part = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
-          (GDestroyNotify) tp_g_value_slice_free);
-      g_hash_table_insert (part, "content-type",
-          tp_g_value_slice_new_string ("text/plain"));
-      g_hash_table_insert (part, "alternative",
-          tp_g_value_slice_new_string ("alt"));
-      g_hash_table_insert (part, "lang",
-          tp_g_value_slice_new_string ("en_US"));
-      g_hash_table_insert (part, "content",
-          tp_g_value_slice_new_string ("we're fixing the color of the video stream"));
+      part = tp_asv_new (
+          "content-type", G_TYPE_STRING, "text/plain",
+          "alternative", G_TYPE_STRING, "alt",
+          "lang", G_TYPE_STRING, "en_US",
+          "content", G_TYPE_STRING,
+            "we're fixing the color of the video stream",
+          NULL);
       g_ptr_array_add (send_parts, part);
 
       tp_cli_channel_interface_messages_run_send_message (chan, -1,
           send_parts, 0 /* flags */, &token, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       /* wait for pending events to be delivered */
       while (received_count < 1 || message_received_count < 1)
@@ -899,7 +876,7 @@ main (int argc,
 
       tp_cli_channel_interface_messages_run_get_pending_message_content (chan,
           -1, last_received_id, part_numbers, &ret, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       MYASSERT (g_hash_table_size (ret) == 2, ": %u",
           g_hash_table_size (ret));
@@ -928,7 +905,7 @@ main (int argc,
 
       tp_cli_channel_type_text_run_list_pending_messages (chan, -1,
           FALSE, &messages, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_print ("Freeing\n");
       g_boxed_free (TP_ARRAY_TYPE_PENDING_TEXT_MESSAGE_LIST, messages);
@@ -973,7 +950,7 @@ main (int argc,
 
       tp_cli_dbus_properties_run_get_all (chan, -1,
           TP_IFACE_CHANNEL_INTERFACE_MESSAGES, &properties, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_print ("\n\n==== Examining properties ====\n\n");
 
@@ -1025,7 +1002,7 @@ main (int argc,
 
       tp_cli_channel_type_text_run_acknowledge_pending_messages (chan, -1,
           msgid, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_array_free (msgid, TRUE);
     }
@@ -1038,7 +1015,7 @@ main (int argc,
 
       tp_cli_channel_type_text_run_list_pending_messages (chan, -1,
           TRUE, &messages, &error, NULL);
-      test_assert_no_error (error);
+      g_assert_no_error (error);
 
       g_print ("Freeing\n");
       g_boxed_free (TP_ARRAY_TYPE_PENDING_TEXT_MESSAGE_LIST, messages);
@@ -1050,13 +1027,13 @@ main (int argc,
       GPtrArray *channels;
 
       MYASSERT (tp_cli_channel_run_close (chan, -1, &error, NULL), "");
-      test_assert_no_error (error);
+      g_assert_no_error (error);
       MYASSERT (tp_proxy_get_invalidated (chan) != NULL, "");
 
       /* assert that the channel has really gone */
       MYASSERT (tp_cli_connection_run_list_channels (conn, -1,
             &channels, &error, NULL), "");
-      test_assert_no_error (error);
+      g_assert_no_error (error);
       MYASSERT (channels->len == 0, "%u != 0", channels->len);
       g_boxed_free (TP_ARRAY_TYPE_CHANNEL_INFO_LIST, channels);
     }
@@ -1064,14 +1041,11 @@ main (int argc,
   g_print ("\n\n==== End of tests ====\n");
 
   MYASSERT (tp_cli_connection_run_disconnect (conn, -1, &error, NULL), "");
-  test_assert_no_error (error);
+  g_assert_no_error (error);
 
-  tp_handle_unref (contact_repo, handle);
   g_object_unref (chan);
   g_object_unref (conn);
 
-  service_conn_as_base = NULL;
-  g_object_unref (service_conn);
   g_object_unref (dbus);
   g_free (name);
   g_free (conn_path);
