@@ -12,6 +12,7 @@
 
 #include "_gen/svc.h"
 #include "tests/lib/myassert.h"
+#include "tests/lib/util.h"
 
 typedef struct _TestProperties {
     GObject parent;
@@ -101,14 +102,6 @@ test_properties_class_init (TestPropertiesClass *cls)
       G_STRUCT_OFFSET (TestPropertiesClass, props));
 }
 
-static int fail = 0;
-
-static void
-myassert_failed (void)
-{
-  fail = 1;
-}
-
 static void
 print_asv_item (gpointer key,
                 gpointer value,
@@ -132,23 +125,22 @@ int
 main (int argc, char **argv)
 {
   TestProperties *obj;
-  DBusGConnection *bus;
+  TpDBusDaemon *dbus_daemon;
   TpProxy *proxy;
   GValue *value;
   GHashTable *hash;
 
   tp_debug_set_flags ("all");
   g_type_init ();
-  bus = tp_get_bus ();
+  dbus_daemon = tp_tests_dbus_daemon_dup_or_die ();
 
-  obj = g_object_new (TEST_TYPE_PROPERTIES, NULL);
-  dbus_g_connection_register_g_object (bus, "/", (GObject *) obj);
+  obj = tp_tests_object_new_static_class (TEST_TYPE_PROPERTIES, NULL);
+  tp_dbus_daemon_register_object (dbus_daemon, "/", obj);
 
   /* Open a D-Bus connection to myself */
-  proxy = TP_PROXY (g_object_new (TP_TYPE_PROXY,
-      "dbus-connection", bus,
-      "bus-name",
-          dbus_bus_get_unique_name (dbus_g_connection_get_connection (bus)),
+  proxy = TP_PROXY (tp_tests_object_new_static_class (TP_TYPE_PROXY,
+      "dbus-daemon", dbus_daemon,
+      "bus-name", tp_dbus_daemon_get_unique_name (dbus_daemon),
       "object-path", "/",
       NULL));
 
@@ -190,6 +182,7 @@ main (int argc, char **argv)
 
   g_object_unref (obj);
   g_object_unref (proxy);
+  g_object_unref (dbus_daemon);
 
-  return fail;
+  return 0;
 }
