@@ -19,12 +19,11 @@
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/handle-repo-dynamic.h>
+#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
 
-G_DEFINE_TYPE_WITH_CODE (SimpleConnection,
-    simple_connection,
-    TP_TYPE_BASE_CONNECTION,
-    G_STMT_START { } G_STMT_END)
+G_DEFINE_TYPE (TpTestsSimpleConnection, tp_tests_simple_connection,
+    TP_TYPE_BASE_CONNECTION);
 
 /* type definition stuff */
 
@@ -34,16 +33,18 @@ enum
   N_PROPS
 };
 
-struct _SimpleConnectionPrivate
+struct _TpTestsSimpleConnectionPrivate
 {
   gchar *account;
+  guint connect_source;
+  guint disconnect_source;
 };
 
 static void
-simple_connection_init (SimpleConnection *self)
+tp_tests_simple_connection_init (TpTestsSimpleConnection *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, SIMPLE_TYPE_CONNECTION,
-      SimpleConnectionPrivate);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      TP_TESTS_TYPE_SIMPLE_CONNECTION, TpTestsSimpleConnectionPrivate);
 }
 
 static void
@@ -52,7 +53,7 @@ get_property (GObject *object,
               GValue *value,
               GParamSpec *spec)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (object);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (object);
 
   switch (property_id) {
     case PROP_ACCOUNT:
@@ -69,7 +70,7 @@ set_property (GObject *object,
               const GValue *value,
               GParamSpec *spec)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (object);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (object);
 
   switch (property_id) {
     case PROP_ACCOUNT:
@@ -84,23 +85,33 @@ set_property (GObject *object,
 static void
 finalize (GObject *object)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (object);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (object);
+
+  if (self->priv->connect_source != 0)
+    {
+      g_source_remove (self->priv->connect_source);
+    }
+
+  if (self->priv->disconnect_source != 0)
+    {
+      g_source_remove (self->priv->disconnect_source);
+    }
 
   g_free (self->priv->account);
 
-  G_OBJECT_CLASS (simple_connection_parent_class)->finalize (object);
+  G_OBJECT_CLASS (tp_tests_simple_connection_parent_class)->finalize (object);
 }
 
 static gchar *
 get_unique_connection_name (TpBaseConnection *conn)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (conn);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (conn);
 
   return g_strdup (self->priv->account);
 }
 
 static gchar *
-simple_normalize_contact (TpHandleRepoIface *repo,
+tp_tests_simple_normalize_contact (TpHandleRepoIface *repo,
                            const gchar *id,
                            gpointer context,
                            GError **error)
@@ -127,7 +138,7 @@ create_handle_repos (TpBaseConnection *conn,
                      TpHandleRepoIface *repos[NUM_TP_HANDLE_TYPES])
 {
   repos[TP_HANDLE_TYPE_CONTACT] = tp_dynamic_handle_repo_new
-      (TP_HANDLE_TYPE_CONTACT, simple_normalize_contact, NULL);
+      (TP_HANDLE_TYPE_CONTACT, tp_tests_simple_normalize_contact, NULL);
 }
 
 static GPtrArray *
@@ -137,7 +148,7 @@ create_channel_factories (TpBaseConnection *conn)
 }
 
 void
-simple_connection_inject_disconnect (SimpleConnection *self)
+tp_tests_simple_connection_inject_disconnect (TpTestsSimpleConnection *self)
 {
   tp_base_connection_change_status ((TpBaseConnection *) self,
       TP_CONNECTION_STATUS_DISCONNECTED,
@@ -147,7 +158,7 @@ simple_connection_inject_disconnect (SimpleConnection *self)
 static gboolean
 pretend_connected (gpointer data)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (data);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (data);
   TpBaseConnection *conn = (TpBaseConnection *) self;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -155,9 +166,13 @@ pretend_connected (gpointer data)
   conn->self_handle = tp_handle_ensure (contact_repo, self->priv->account,
       NULL, NULL);
 
-  tp_base_connection_change_status (conn, TP_CONNECTION_STATUS_CONNECTED,
-      TP_CONNECTION_STATUS_REASON_REQUESTED);
+  if (conn->status == TP_CONNECTION_STATUS_CONNECTING)
+    {
+      tp_base_connection_change_status (conn, TP_CONNECTION_STATUS_CONNECTED,
+          TP_CONNECTION_STATUS_REASON_REQUESTED);
+    }
 
+  self->priv->connect_source = 0;
   return FALSE;
 }
 
@@ -165,7 +180,7 @@ static gboolean
 start_connecting (TpBaseConnection *conn,
                   GError **error)
 {
-  SimpleConnection *self = SIMPLE_CONNECTION (conn);
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (conn);
 
   tp_base_connection_change_status (conn, TP_CONNECTION_STATUS_CONNECTING,
       TP_CONNECTION_STATUS_REASON_REQUESTED);
@@ -173,8 +188,8 @@ start_connecting (TpBaseConnection *conn,
   /* In a real connection manager we'd ask the underlying implementation to
    * start connecting, then go to state CONNECTED when finished. Here there
    * isn't actually a connection, so we'll fake a connection process that
-   * takes half a second. */
-  g_timeout_add (500, pretend_connected, self);
+   * takes time. */
+  self->priv->connect_source = g_timeout_add (0, pretend_connected, self);
 
   return TRUE;
 }
@@ -182,33 +197,40 @@ start_connecting (TpBaseConnection *conn,
 static gboolean
 pretend_disconnected (gpointer data)
 {
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (data);
+
   tp_base_connection_finish_shutdown (TP_BASE_CONNECTION (data));
+  self->priv->disconnect_source = 0;
   return FALSE;
 }
 
 static void
 shut_down (TpBaseConnection *conn)
 {
+  TpTestsSimpleConnection *self = TP_TESTS_SIMPLE_CONNECTION (conn);
+
   /* In a real connection manager we'd ask the underlying implementation to
    * start shutting down, then call this function when finished. Here there
    * isn't actually a connection, so we'll fake a disconnection process that
-   * takes half a second. */
-  g_timeout_add (500, pretend_disconnected, conn);
+   * takes time. */
+  self->priv->disconnect_source = g_timeout_add (0, pretend_disconnected,
+      conn);
 }
 
 static void
-simple_connection_class_init (SimpleConnectionClass *klass)
+tp_tests_simple_connection_class_init (TpTestsSimpleConnectionClass *klass)
 {
   TpBaseConnectionClass *base_class =
       (TpBaseConnectionClass *) klass;
   GObjectClass *object_class = (GObjectClass *) klass;
   GParamSpec *param_spec;
-  static const gchar *interfaces_always_present[] = { NULL };
+  static const gchar *interfaces_always_present[] = {
+      TP_IFACE_CONNECTION_INTERFACE_REQUESTS, NULL };
 
   object_class->get_property = get_property;
   object_class->set_property = set_property;
   object_class->finalize = finalize;
-  g_type_class_add_private (klass, sizeof (SimpleConnectionPrivate));
+  g_type_class_add_private (klass, sizeof (TpTestsSimpleConnectionPrivate));
 
   base_class->create_handle_repos = create_handle_repos;
   base_class->get_unique_connection_name = get_unique_connection_name;
@@ -226,7 +248,7 @@ simple_connection_class_init (SimpleConnectionClass *klass)
 }
 
 void
-simple_connection_set_identifier (SimpleConnection *self,
+tp_tests_simple_connection_set_identifier (TpTestsSimpleConnection *self,
                                   const gchar *identifier)
 {
   TpBaseConnection *conn = (TpBaseConnection *) self;
@@ -240,3 +262,14 @@ simple_connection_set_identifier (SimpleConnection *self,
   tp_base_connection_set_self_handle (conn, handle);
   tp_handle_unref (contact_repo, handle);
 }
+
+TpTestsSimpleConnection *
+tp_tests_simple_connection_new (const gchar *account,
+    const gchar *protocol)
+{
+  return TP_TESTS_SIMPLE_CONNECTION (g_object_new (
+      TP_TESTS_TYPE_SIMPLE_CONNECTION,
+      "account", account,
+      "protocol", protocol,
+      NULL));
+}
